// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18

package ju.thecompany.android.activity;

import ju.thecompany.android.R;
import ju.thecompany.android.service.DriveService;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;

// 네트워크 이상시 메인화면
public class ConnectActivity extends Activity{
	
	// 디버그 프래그
	private static final String DEBUG_TAG = "ConnectActivity";

	// 다이얼로그 
	private static final int DIALOG_NOCONNECT = 0;
	
	// 노티파이
	private NotificationManager mNM;
	
    // Lifecycle 메소드 시작
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.main);
        //
        Log.d(DEBUG_TAG, "onCreate()");
        //
        
    }
    
    protected void onResume() {
    	super.onResume();
    	mNM = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);
    	
    	mNM.cancel(R.string.noconnect_notify);
    	
    	// 리시버 등록
    	IntentFilter filter = new IntentFilter(RECV);
        mReceiver = new ConnectActivityReceiver();
        registerReceiver(mReceiver, filter);
        filter = new IntentFilter(FINISHRECV);
        mFinishReceiver = new FinishReceiver();
        registerReceiver(mFinishReceiver, filter);
    	
    	showDialog(DIALOG_NOCONNECT);
        //
        Log.d(DEBUG_TAG, "onResume()");
        //
    }

    protected void onPause() {
    	super.onPause();

    	// 리시버 해제
        unregisterReceiver(mReceiver);
    	unregisterReceiver(mFinishReceiver);
        //
        Log.d(DEBUG_TAG, "onPause()");
        //
    }
    
    protected void onDestroy() {
    	super.onDestroy();
    	
        //
        Log.d(DEBUG_TAG, "onDestory");
        //
    }
    // Lifecycle 메소드 끝
    
    // 다이얼로그
	public Dialog onCreateDialog(int id) {
		switch(id) {
		case DIALOG_NOCONNECT  :  
			LayoutInflater factoryMenu = LayoutInflater.from(this);
	        final View menuView = factoryMenu.inflate(R.layout.drive_menu, null);
			return new AlertDialog.Builder(ConnectActivity.this)
    		.setTitle(R.string.noconnection_menu_title)
    		.setCancelable(false)
    		.setItems(R.array.conmenu, 
    				new DialogInterface.OnClickListener() {
    					@Override
    					public void onClick(DialogInterface dialoginterface, int i) {
    						startApp(i);
    					}
    			}).create();
		}
		return null;
	}
	
	// 리턴 프래그

	public static final int CONNECT_ACTIVITY_SEND_SHORTNOTICE= 0;
	public static final int CONNECT_ACTIVITY_SEND_SETTING= 1;
	public static final int CONNECT_ACTIVITY_SEND_CLOSE = 2;
	public static final int CONNECT_ACTIVITY_SEND_EXIT = 3;

	// 다이얼로그 메뉴 실행 선택
	public void startApp(int i) {
		Intent intent = new Intent();
    	switch (i) {
    	case CONNECT_ACTIVITY_SEND_SHORTNOTICE :
			intent.setAction(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_ACTIVITY_CONNECT);
			intent.putExtra("mode", CONNECT_ACTIVITY_SEND_SHORTNOTICE);
			sendBroadcast(intent);
    		break;
    	case CONNECT_ACTIVITY_SEND_SETTING :
			intent.setAction(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_ACTIVITY_CONNECT);
			intent.putExtra("mode", CONNECT_ACTIVITY_SEND_SETTING);
			sendBroadcast(intent);
    		break;
    	case CONNECT_ACTIVITY_SEND_CLOSE :
    		showNotification();
    		finish();
    		break;
    	case CONNECT_ACTIVITY_SEND_EXIT:
			intent.setAction(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_ACTIVITY_CONNECT);
			intent.putExtra("mode", CONNECT_ACTIVITY_SEND_EXIT);
			sendBroadcast(intent);
    		break;
    	}
	}
	
    // 리시버
    public static final String RECV = "ju.thecompany.android.activity.CONNECTACTIVITY";
    ConnectActivityReceiver mReceiver;
	Intent mRecvIntent; 

    public class ConnectActivityReceiver extends BroadcastReceiver {

		@Override
		public void onReceive(Context context, Intent intent) {
			
			mRecvIntent = intent;
			
			Thread threadProcessReceiver = new Thread(null, processReceiverThread, "Background" );
			threadProcessReceiver.start();
   			//
	        Log.d(DEBUG_TAG, "onReceive()");
	        //
		}
    }
    
    private Runnable processReceiverThread = new Runnable() {

		@Override
		public void run() {

	        handler.post(processReceiver);
		}
    	
    };
    
    private Runnable processReceiver = new Runnable() {
    	
		@Override
		public void run() {

			processReceiver();
		}
    	
    };
    
    private void processReceiver() {
    	finish();
    }
    
    private static final String FINISHRECV = "ju.thecompany.android.FINISH";
    FinishReceiver mFinishReceiver;

    private class FinishReceiver extends BroadcastReceiver {

		@Override
		public void onReceive(Context context, Intent intent) {

			boolean flag = intent.getBooleanExtra("flag", true);
			if (flag) finish();
   			//
	        Log.d(DEBUG_TAG, "finishReceiver()");
	        //
		}
    }
    
    // 핸들러
    private Handler handler = new Handler();
	
    private void showNotification() {

        CharSequence text = getText(R.string.noconnect_notify);

        Notification notification = new Notification(R.drawable.stat_sample, text,
                System.currentTimeMillis());

        PendingIntent contentIntent = PendingIntent.getActivity(this, 0,
                new Intent(this, ConnectActivity.class), 0);

        notification.setLatestEventInfo(this, getText(R.string.noconnect_notify),
                       text, contentIntent);

        mNM.notify(R.string.noconnect_notify, notification);
    }
}


