// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18

package ju.thecompany.android.activity;

import ju.thecompany.android.R;
import ju.thecompany.android.service.DriveService;
import android.app.Activity;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

// 로그인을 담당
public class LoginActivity extends Activity {
	
	// 디버그 프래그
	private static final String DEBUG_TAG = "LoginActivity";
	
	// 리턴 프래그
	public static final int LOGIN_ACTIVITY_SEND_LOGIN_REQ= 1;
	public static final int LOGIN_ACTIVITY_SEND_LOGIN_OK= 2;
	public static final int LOGIN_ACTIVITY_SEND_LOGIN_CANCLE= 3;
	public static final int LOGIN_ACTIVITY_SEND_LOGIN_SETTING= 4;

	// 다이얼로그
	private static final int DIALOG_LOGIN_WAIT = 0;
	private ProgressDialog mLoginWaitDlg;

	// 위젯
	EditText mEditTextID;
	EditText mEditTextPW;
	Button mButtonOK;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		Window w = getWindow();
		w.requestFeature(Window.FEATURE_LEFT_ICON);
		 
		setContentView(R.layout.login);
		 
		w.setFeatureDrawableResource(Window.FEATURE_LEFT_ICON,
			R.drawable.icon);
		

		mEditTextID = (EditText)findViewById(R.id.login_edit_id);
		mEditTextPW = (EditText)findViewById(R.id.login_edit_pw);

		String lastID = PreferencesActivity.getUserLast(this);
		if (!lastID.equals("")) {
			mEditTextID.setText(lastID);
			mEditTextPW.requestFocus();
		}

		mButtonOK = (Button)findViewById(R.id.login_button_ok);
		mButtonOK.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				
				boolean loginFlag = true;
				
				
				String loginID =mEditTextID.getText().toString();
				if ( loginID.equals("")) {
					mEditTextID.setHint(R.string.login_id_hint);
					loginFlag = false;
				}
				
				String loginPW = mEditTextPW.getText().toString();
				if ( loginPW.equals("")) {
					mEditTextPW.setHint(R.string.login_pw_hint);
					loginFlag = false;
				}
				
				if (!loginFlag) {
					showToast(getText(R.string.error_check_login).toString());
				} else {
					// 로그인 통신 시작
					showDialog(DIALOG_LOGIN_WAIT);
		            setUserLast(loginID);
					Intent intent = new Intent(DriveService.RECV);
					intent.putExtra("activity", DriveService.SEND_ACTIVITY_LOGIN);
					intent.putExtra("mode", LOGIN_ACTIVITY_SEND_LOGIN_REQ);
					intent.putExtra("object", loginPW);
					sendBroadcast(intent);
				}
			}
		});
		
		Button buttonCancle = (Button)findViewById(R.id.login_button_cancle);
		buttonCancle.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				Intent intent = new Intent(DriveService.RECV);
				intent.putExtra("activity", DriveService.SEND_ACTIVITY_LOGIN);
				intent.putExtra("mode", LOGIN_ACTIVITY_SEND_LOGIN_CANCLE);
				sendBroadcast(intent);
				finish();
			}
		});
		
		Button buttonSetting = (Button)findViewById(R.id.login_button_setting);
		buttonSetting.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				Intent intent = new Intent(DriveService.RECV);
				intent.putExtra("activity", DriveService.SEND_ACTIVITY_LOGIN);
				intent.putExtra("mode", LOGIN_ACTIVITY_SEND_LOGIN_SETTING);
				sendBroadcast(intent);				
				finish();
			}
		});
		
        //
        Log.d(DEBUG_TAG, "onCreate()");
        //
	}
	

    @Override
    protected void onResume() {
    	super.onResume();
    	
    	// 리시버 등록
    	IntentFilter filter = new IntentFilter(RECV);
        mReceiver = new LoginActivityReceiver();
        registerReceiver(mReceiver, filter);
        filter = new IntentFilter(FINISHRECV);
        mFinishReceiver = new FinishReceiver();
        registerReceiver(mFinishReceiver, filter);

        //
        Log.d(DEBUG_TAG, "onResume()");
        //
    }
    
    protected void onPause() {
    	super.onPause();
    	
    	// 리시버 해제
        unregisterReceiver(mReceiver);
    	unregisterReceiver(mFinishReceiver);
        //
        Log.d(DEBUG_TAG, "onPause()");
        //
    }
   
    // 다이얼로그
    @Override
    protected Dialog onCreateDialog(int id) {
        switch (id) {
            case DIALOG_LOGIN_WAIT: {
                mLoginWaitDlg = new ProgressDialog(this);
                mLoginWaitDlg.setMessage(getText(R.string.login_wait).toString());
                mLoginWaitDlg.setIndeterminate(true);
                mLoginWaitDlg.setCancelable(true);
                return mLoginWaitDlg;
            }
        }
        return null;
    }

    // DB콘트롤
	private void setUserLast(String id) {
		SharedPreferences sp = PreferencesActivity.getSharedPreferences(this);
		sp.edit().putString(PreferencesActivity.USER_LAST, id).commit();		
	}
	
	// 토스트 실행 
	public void showToast(String msg) {

		LayoutInflater inflater = getLayoutInflater();
		View layout = inflater.inflate(R.layout.toast_layout,
		                               (ViewGroup) findViewById(R.id.toast_layout_root));

		ImageView image = (ImageView) layout.findViewById(R.id.toast_image);
		image.setImageResource(android.R.drawable.ic_dialog_alert);
		TextView text = (TextView) layout.findViewById(R.id.toast_text);
		text.setText(msg);

		Toast toast = new Toast(getApplicationContext());
		toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
		toast.setDuration(Toast.LENGTH_SHORT);
		toast.setView(layout);
		toast.show();
	}
	
	

    // 리시버 시작
    public static final String FINISHRECV = "ju.thecompany.android.FINISH";
    FinishReceiver mFinishReceiver;
	
    public class FinishReceiver extends BroadcastReceiver {

		@Override
		public void onReceive(Context context, Intent intent) {
			
			boolean flag = intent.getBooleanExtra("flag", true);
			if (!flag) finish();
   			//
	        Log.d(DEBUG_TAG, "finishReceiver()");
	        //
		}
    }

    public static final String RECV = "ju.thecompany.android.activity.LOGINACTIVITY";
    LoginActivityReceiver mReceiver;
	Intent mRecvIntent; 
	
	public static final int LOGIN_ACTIVITY_RECV_LOGIN_OK = 0;
	public static final int LOGIN_ACTIVITY_RECV_LOGIN_NO = 1;
	public static final int LOGIN_ACTIVITY_RECV_LOGIN_ERR = 2;

    public class LoginActivityReceiver extends BroadcastReceiver {

		@Override
		public void onReceive(Context context, Intent intent) {
			
			mRecvIntent = intent;
			
			Thread threadProcessReceiver = new Thread(null, processReceiverThread, "Background" );
			threadProcessReceiver.start();
			//processReceiver();
   			//
	        Log.d(DEBUG_TAG, "onReceive()");
	        //
		}
    }
    
    private Runnable processReceiverThread = new Runnable() {

		@Override
		public void run() {

	        handler.post(processReceiver);
		}
    	
    };
    
    private Runnable processReceiver = new Runnable() {
    	
		@Override
		public void run() {

			processReceiver();
		}
    	
    };
    
    private void processReceiver() {
    	int mode = mRecvIntent.getIntExtra("mode", 0);
    	if (mode == LOGIN_ACTIVITY_RECV_LOGIN_ERR) {
    		mLoginWaitDlg.cancel();
    		showToast(getText(R.string.error_check_server).toString());
    	}
    	if (mode == LOGIN_ACTIVITY_RECV_LOGIN_OK) {
    		mLoginWaitDlg.cancel();  
			Intent intent = new Intent(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_ACTIVITY_LOGIN);
			intent.putExtra("mode", LOGIN_ACTIVITY_SEND_LOGIN_OK);
			sendBroadcast(intent);
			finish();
    	}
    	if (mode == LOGIN_ACTIVITY_RECV_LOGIN_NO) {
    		mLoginWaitDlg.cancel();
    		showToast(getText(R.string.error_check_login).toString());
    	}
    }
    
    // 핸들러
    private Handler handler = new Handler();
}