// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18

package ju.thecompany.android.activity;

import ju.thecompany.android.R;
import ju.thecompany.android.service.DriveService;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;

// 로그인 후 메인화면
public class MainMenuActivity extends Activity {
	
	// 디버그 프래그
	private static final String DEBUG_TAG = "MainMenuActivity";
	
	// 메인메뉴 다이얼로그
	private static final int DIALOG_MAINMENU = 0;
	private static final int DIALOG_LOGOUT = 1;
	
	// 노티파이
	private NotificationManager mNM;
	
	
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.main);
		
        //
        Log.d(DEBUG_TAG, "onCreate()");
        //
	}
	

    @Override
    protected void onResume() {
    	super.onResume();
    	
    	if (!PreferencesActivity.getConnection(this)) finish();

    	mNM = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);
    	
    	mNM.cancel(R.string.mainmenu_notify);
    	
    	// 리시버 등록
    	IntentFilter filter = new IntentFilter(RECV);
        mReceiver = new MainMenuActivityReceiver();
        registerReceiver(mReceiver, filter);
        filter = new IntentFilter(FINISHRECV);
        mFinishReceiver = new FinishReceiver();
        registerReceiver(mFinishReceiver, filter);
        
    	showDialog(DIALOG_MAINMENU);

        //
        Log.d(DEBUG_TAG, "onResume()");
        //
    }
    
    protected void onPause() {
    	super.onPause();

    	// 리시버 해제
        unregisterReceiver(mReceiver);
    	unregisterReceiver(mFinishReceiver);
        //
        Log.d(DEBUG_TAG, "onPause()");
        //
    }
    	
    // 다이얼로그 생성
	@Override
	public Dialog onCreateDialog(int id) {
		switch(id) {
		case DIALOG_MAINMENU  :  
			
			LayoutInflater factoryMenu = LayoutInflater.from(this);
	        final View menuView = factoryMenu.inflate(R.layout.drive_menu, null);
			return new AlertDialog.Builder(MainMenuActivity.this)
    		.setTitle(R.string.drive_menu_title)
    		.setCancelable(false)
    		.setItems(R.array.mainmenu, 
    				new DialogInterface.OnClickListener() {
    					@Override
    					public void onClick(DialogInterface dialoginterface, int i) {
    						startApp(i);
    					}
    			}).create();
		case DIALOG_LOGOUT :
			return new AlertDialog.Builder(this)
			.setTitle(R.string.drive_menu_logout_title)
			.setPositiveButton(getText(R.string.drive_menu_logout_ok), new DialogInterface.OnClickListener() {
	            public void onClick(DialogInterface dialog, int whichButton) {     	
	    	        Intent intent = new Intent();
	    			intent.setAction(DriveService.RECV);
	    			intent.putExtra("activity", DriveService.SEND_ACTIVITY_MAINMENU);
	    			intent.putExtra("mode", MAINMENU_ACTIVITY_SEND_LOGOUT);
	    			sendBroadcast(intent);	
	    			finishApp();
	            }
	        })
	        .setNegativeButton(getText(R.string.drive_menu_logout_cancle), new DialogInterface.OnClickListener() {
	            public void onClick(DialogInterface dialog, int whichButton) {
	            	resumeApp();
	            }
	        }).create();
		}
		return null;
	}
	
	// 리턴 프래그

	public static final int MAINMENU_ACTIVITY_SEND_MESSENGER = 0;
	public static final int MAINMENU_ACTIVITY_SEND_PIM = 1;
	public static final int MAINMENU_ACTIVITY_SEND_SHORTNOTICE= 2;
	public static final int MAINMENU_ACTIVITY_SEND_CLOSE = 3;
	public static final int MAINMENU_ACTIVITY_SEND_LOGOUT= 4;
	
	// 다이얼로그 메뉴 실행 선택
	public void startApp(int i) {
		Intent intent = new Intent();
    	switch (i) {
    	case MAINMENU_ACTIVITY_SEND_MESSENGER :
			intent.setAction(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_ACTIVITY_MAINMENU);
			intent.putExtra("mode", MAINMENU_ACTIVITY_SEND_MESSENGER);
			sendBroadcast(intent);	
    		break;
    	case MAINMENU_ACTIVITY_SEND_PIM :
			intent.setAction(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_ACTIVITY_MAINMENU);
			intent.putExtra("mode", MAINMENU_ACTIVITY_SEND_PIM);
			sendBroadcast(intent);	
    		break;
    	case MAINMENU_ACTIVITY_SEND_SHORTNOTICE :
			intent.setAction(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_ACTIVITY_MAINMENU);
			intent.putExtra("mode", MAINMENU_ACTIVITY_SEND_SHORTNOTICE);
			sendBroadcast(intent);	
    		break;
    	case MAINMENU_ACTIVITY_SEND_LOGOUT :
    		showDialog(DIALOG_LOGOUT);
    		break;
    	case MAINMENU_ACTIVITY_SEND_CLOSE :
    		showNotification();
    		finishApp();
    		break;
    	}
	}
	
	// 내부 메쏘드
	private void finishApp() {
		finish();
	}

	public void resumeApp() {
		onResume();
	}
	
	
    private static final String FINISHRECV = "ju.thecompany.android.FINISH";
    FinishReceiver mFinishReceiver;

    private class FinishReceiver extends BroadcastReceiver {

		@Override
		public void onReceive(Context context, Intent intent) {

			boolean flag = intent.getBooleanExtra("flag", true);
			if (!flag) finish();
   			//
	        Log.d(DEBUG_TAG, "finishReceiver()");
	        //
		}
    }
    
    public static final String RECV = "ju.thecompany.android.activity.MAINMENUACTIVITY";
    MainMenuActivityReceiver mReceiver;
	Intent mRecvIntent; 

    public class MainMenuActivityReceiver extends BroadcastReceiver {

		@Override
		public void onReceive(Context context, Intent intent) {
			
			mRecvIntent = intent;
			
			Thread threadProcessReceiver = new Thread(null, processReceiverThread, "Background" );
			threadProcessReceiver.start();
			//processReceiver();
   			//
	        Log.d(DEBUG_TAG, "onReceive()");
	        //
		}
    }
    
    private Runnable processReceiverThread = new Runnable() {

		@Override
		public void run() {

	        handler.post(processReceiver);
		}
    	
    };
    
    private Runnable processReceiver = new Runnable() {
    	
		@Override
		public void run() {

			processReceiver();
		}
    	
    };
    
    private void processReceiver() {
    	finish();
    }
    
    
    
    // 핸들러
    private Handler handler = new Handler();
    
    private void showNotification() {

        CharSequence text = getText(R.string.mainmenu_notify);

        Notification notification = new Notification(R.drawable.stat_sample, text,
                System.currentTimeMillis());

        PendingIntent contentIntent = PendingIntent.getActivity(this, 0,
                new Intent(this, MainMenuActivity.class), 0);

        notification.setLatestEventInfo(this, getText(R.string.mainmenu_notify),
                       text, contentIntent);

        mNM.notify(R.string.mainmenu_notify, notification);
    }

}
