// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18

package ju.thecompany.android.activity;

import ju.thecompany.android.R;
import ju.thecompany.android.service.DriveService;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.CheckBoxPreference;
import android.preference.EditTextPreference;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceManager;
import android.util.Log;

// 환경설정
public class PreferencesActivity extends PreferenceActivity{
	
	// 디버그 프래그
	private static final String DEBUG_TAG = "PreferencesActivity";

    
    // 환경설정 키
    public static final String 	SERVER_IP 				= "reference_serverip";
    public static final String 	SERVER_PORT 			= "reference_serverport";
    public static final String 	CLIENT_PORT 			= "reference_clientport";
    public static final String 	USER_LAST 				= "reference_user_last";
    public static final String 	START_MODE 			= "start_mode";   
    public static final String 	CON_BASIC 				= "reference_con_basic";   
    public static final String	MY_LOCATION			= "reference_my_location";
    public static final String	CONNECTION				= "connection";
    public static final String	PASSWD					= "passwd";
	
    // 환경설정 디폴트
    public static final String 	SERVER_IP_DEF	 		= "localhost";
    public static final String 	SERVER_PORT_DEF 	= "5455";
    public static final String	CLIENT_PORT_DEF 	= "6002";
    public static final String 	USER_LAST_DEF 		= "";
    
    //
    public static final int		START_MODE_DEF 	= 0;
    
    
    // Lifecycle 메소드 시작
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        addPreferencesFromResource(R.xml.preferences);

    	final EditTextPreference preferenceServerIP = (EditTextPreference)findPreference(SERVER_IP );
    	final EditTextPreference preferenceServerPort =(EditTextPreference)findPreference(SERVER_PORT );
    	final EditTextPreference preferenceClientPort = (EditTextPreference)findPreference(CLIENT_PORT);
    	final EditTextPreference preferenceUserLast = (EditTextPreference)findPreference(USER_LAST );
    	final CheckBoxPreference preferenceConBasic = (CheckBoxPreference)findPreference(CON_BASIC);
    	final CheckBoxPreference preferenceMyLocation = (CheckBoxPreference)findPreference(MY_LOCATION);

        
        // 세팅의 변화가 생기면 개요 변경
        preferenceServerIP.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener() {
			
			@Override
			public boolean onPreferenceChange(Preference preference, Object newValue) {
				
				preferenceServerIP.setSummary(newValue.toString());

				return true;
			}
		});

        preferenceServerPort.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener() {
			
			@Override
			public boolean onPreferenceChange(Preference preference, Object newValue) {
				
				preferenceServerPort.setSummary(newValue.toString());

				return true;
			}
		});
        
        preferenceClientPort.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener() {
			
			@Override
			public boolean onPreferenceChange(Preference preference, Object newValue) {
				
				preferenceClientPort.setSummary(newValue.toString());

				return true;
			}
		});
        
        preferenceUserLast.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener() {
			
			@Override
			public boolean onPreferenceChange(Preference preference, Object newValue) {
				
				preferenceUserLast.setSummary(newValue.toString());

				return true;
			}
		});
        // 세팅값을 개요에 출력, 시작시 화면 출력용.
        preferenceServerIP.setSummary(PreferencesActivity.getServerIP(this));
        preferenceServerPort.setSummary("" + PreferencesActivity.getServerPort(this));
        preferenceClientPort.setSummary(""+ PreferencesActivity.getClientPort(this));
        preferenceUserLast.setSummary(PreferencesActivity.getUserLast(this));

        //
        Log.d(DEBUG_TAG, "onCreate()");
        //
        
    }
    
    
    protected void onPause() {
    	super.onPause();
    	
		Intent intent = new Intent(DriveService.RECV);
		intent.putExtra("activity", DriveService.SEND_ACTIVITY_PREFERENCES);
		sendBroadcast(intent);
		
		finish();
        //
        Log.d(DEBUG_TAG, "onPause()");
        //
    }
    
    protected void onDestroy() {
    	super.onDestroy();
    	
        //
        Log.d(DEBUG_TAG, "onDestory");
        //
    }
    // Lifecycle 메소드 끝
    
    // 게터
    public static SharedPreferences getSharedPreferences(Context context) {
    	return PreferenceManager.getDefaultSharedPreferences(context);
    }
    public static String getServerIP(Context context) {
    	return PreferenceManager.getDefaultSharedPreferences(context).getString(SERVER_IP, SERVER_IP_DEF);
    }
    
    public static int getServerPort(Context context) {
    	return Integer.parseInt(PreferenceManager.getDefaultSharedPreferences(context).getString(SERVER_PORT, SERVER_PORT_DEF));
    }
    
    public static int getClientPort(Context context) {
    	return Integer.parseInt(PreferenceManager.getDefaultSharedPreferences(context).getString(CLIENT_PORT, CLIENT_PORT_DEF));
    }
    
    public static String getUserLast(Context context) {
    	return PreferenceManager.getDefaultSharedPreferences(context).getString(USER_LAST, USER_LAST_DEF);
    }
    
    public static int getStartMode(Context context) {
    	return PreferenceManager.getDefaultSharedPreferences(context).getInt(START_MODE, START_MODE_DEF);
    }
    
    public static boolean getConBasic(Context context) {
    	return PreferenceManager.getDefaultSharedPreferences(context).getBoolean(CON_BASIC, false);
    }
    
    public static boolean getMyLocation(Context context) {
    	return PreferenceManager.getDefaultSharedPreferences(context).getBoolean(MY_LOCATION, false);
    }
    
    public static boolean getConnection(Context context) {
    	return PreferenceManager.getDefaultSharedPreferences(context).getBoolean(CONNECTION, true);
    }
    
    public static String getPW(Context context) {
    	return PreferenceManager.getDefaultSharedPreferences(context).getString(PASSWD, "");
    }
    //게터 끝

}


