// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18

package ju.thecompany.android.db;

import java.util.ArrayList;

import ju.thecompany.android.data.User;
import ju.thecompany.android.data.UserNoticeShort;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.util.Log;

// 쪽지를 위한 DB 어뎁터
public class ShortMsgDBAdapter extends DBAdapter {
	
	// 디버그 프래그
	private static final String DEBUG_TAG = "ShortMsgDBAdapter";
	
	// 테이블 이름
	private static final String DATABASE_TABLE = "shortmsg";
	protected static final String DATABASE_NAME = "shortnotice.db";
	  
	// 키
	public static final String KEY = "ord";
	
	// 칼럼명
	private static final String NAME = "name";
	private static final String ID = "id";
	private static final String DEPT = "dept";
	private static final String TITLE = "title";
	private static final String CONTENTS = "contents";
	private static final String DATE = "date";
	private static final String HP = "hp";
	
	
	// 칼럼
	public static final int COLUMN_ORDER = 0;
	public static final int COLUMN_NAME = 1;
	public static final int COLUMN_ID = 2;
	public static final int COLUMN_DEPT = 3;
	public static final int COLUMN_TITLE = 4;
	public static final int COLUMN_CONTENTS = 5;
	public static final int COLUMN_DATE = 6;
	public static final int COLUMN_HP = 7;

	private final Context mContext;
	
	// 데이타베이스 조작 변수
	private SQLiteDatabase mDb;
	private DBOpenHelper mDbHelper;
	
	public ShortMsgDBAdapter (Context context) {
		mContext = context;
		mDbHelper = new DBOpenHelper(context, DATABASE_NAME, null, DATABASE_VERSION);
	}
	
	// DB 열기
	@Override
	public void open() throws SQLiteException {  
		try {
			mDb = mDbHelper.getWritableDatabase();
		}
		catch (SQLiteException ex) {
			mDb = mDbHelper.getReadableDatabase();
		}	  
	}
	
	// DB 닫기
	@Override
	public void close() {
		mDb.close();
	}

	// 모든 커서 반환
	@Override
	public Cursor getAllUserNoticeShortCursor() {
		return mDb.query(DATABASE_TABLE, new String[] { KEY, NAME, ID, DEPT, TITLE, CONTENTS, DATE, HP}, null, null, null, null, null);
	}

	// 데이타 반환
	@Override
	public UserNoticeShort getUserNoticeShort(long rowIndex)
			throws SQLException {
		Cursor cursor = mDb.query(true, DATABASE_TABLE, new String[] {KEY, NAME, ID, DEPT, TITLE, CONTENTS, DATE, HP},
				KEY + "=" + rowIndex, null, null, null, null, null);
		if ((cursor.getCount() == 0) || !cursor.moveToFirst()) {
			throw new SQLException("No item : " + rowIndex);
		}
		
		ArrayList <String> al = new ArrayList <String>(8);
		al.add(0, "");
		al.add(1, "");
		al.add(2, cursor.getString(COLUMN_NAME));
		al.add(3, cursor.getString(COLUMN_DEPT));
		al.add(4, cursor.getString(COLUMN_HP));
		al.add(5, "");
		al.add(6, "");
		al.add(7, "");
		
		
		UserNoticeShort result = new UserNoticeShort(new User(cursor.getString(COLUMN_ID), al), 
				cursor.getString(COLUMN_DATE), cursor.getString(COLUMN_TITLE), cursor.getString(COLUMN_CONTENTS));

		return result;  
	}

	// 데이타 삽입
	@Override
	public long insertMsg(UserNoticeShort userNoticeShort) {

	    ContentValues newTaskValues = new ContentValues();

	    newTaskValues.put(NAME, userNoticeShort.getName());
	    newTaskValues.put(ID, userNoticeShort.getID());
	    newTaskValues.put(DEPT, userNoticeShort.getDept());
	    newTaskValues.put(TITLE, userNoticeShort.getTitle());
	    newTaskValues.put(CONTENTS, userNoticeShort.getText());
	    newTaskValues.put(DATE, userNoticeShort.getDate());
	    newTaskValues.put(HP, userNoticeShort.getHP());

	    return mDb.insert(DATABASE_TABLE, null, newTaskValues);
	}

	// 데이타 삭제
	@Override
	public boolean removeMsg(long rowIndex) {
		return mDb.delete(DATABASE_TABLE, KEY + "=" + rowIndex, null) > 0;
	}

	// DB 헬퍼
	private static class DBOpenHelper extends SQLiteOpenHelper {
	
		public DBOpenHelper(Context context, String name, CursorFactory factory, int version) {
			super(context, name, factory, version);
		}
	
		private static final String DATABASE_CREATE = "create table " + 
			DATABASE_TABLE + " (" + 
		    KEY + " integer primary key autoincrement, " +
		    NAME + " text not null, " + 
		    ID + " text not null, " + 
		    DEPT + " text not null, " + 
		    TITLE + " text not null, " + 
		    CONTENTS + " text not null, " + 
		    DATE + " text not null, " +
		    HP + " text not null);";
	
		@Override
		public void onCreate(SQLiteDatabase db) {
			db.execSQL(DATABASE_CREATE);
		}
	
		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {

	        Log.d(DEBUG_TAG, "Upgrading from version " + 
                    oldVersion + " to " + newVersion + ", which will destroy all old data");

			db.execSQL("DROP TABLE IF EXISTS " + DATABASE_TABLE);
		    onCreate(db);
		}
	}
}