// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18

package ju.thecompany.android.msg;

import java.util.ArrayList;
import java.util.Date;

import ju.thecompany.android.R;
import ju.thecompany.android.activity.PreferencesActivity;
import ju.thecompany.android.data.IsConv;
import ju.thecompany.android.data.User;
import ju.thecompany.android.data.UserList;
import ju.thecompany.android.data.UserMsg;
import ju.thecompany.android.service.DriveService;
import ju.thecompany.data.DateParser;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.view.View.OnKeyListener;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

// 대화 창
public class MsgConversationActivity extends Activity {

	// 디버그 프래그
	private static final String DEBUG_TAG = "MsgConversationActivity";
	
	// 리턴 프래그
	public static final int MSGCONVERSATION_ACTIVITY_SEND_MSG = 0;
	public static final int MSGCONVERSATION_ACTIVITY_SEND_MSG_SUCCESS= 1;

	// 위젯
	private EditText mUserText;
	private ListView mListView;
	
	// 대화리스트 어뎁터
	private MsgListAdapter mAdapter;
	private ArrayList <UserMsg>mText = new ArrayList<UserMsg>();

	
	// 대화유저
	private User mUserMe;
	private User mUserYou;
	
    // 라이프싸이클 메쏘드 시작
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.conversation_msg);

        //
        Log.d(DEBUG_TAG, "onCreate()");
        //
	}
	

    @Override
    protected void onResume() {
    	super.onResume();
    	
    	// 접속 중지 상태면 자동 종료
    	if (!PreferencesActivity.getConnection(this)) finish();
    	
    	// 대화 상태중 
    	IsConv.getInstance().setConv(true);

    	// 리시버 등록
    	IntentFilter filter = new IntentFilter(RECV);
        mReceiver = new MsgConversationActivityReceiver();
        registerReceiver(mReceiver, filter);
        filter = new IntentFilter(FINISHRECV);
        mFinishReceiver = new FinishReceiver();
        registerReceiver(mFinishReceiver, filter);

        // 어뎁터 등록
		mListView = (ListView)findViewById(R.id.conversation_msg_listview);
        mAdapter = new MsgListAdapter(this, R.layout.tcmessenger_row, mText);
		mListView.setAdapter(mAdapter);
        mUserText = (EditText) findViewById(R.id.conversation_msg_edittext);
        mUserText.setOnClickListener(new OnClickListener() {
        	@Override
			public void onClick(View v) {
        		if (mUserText.getText().toString().trim().equals("")) return;
                sendText();
			}
        });
        mUserText.setOnKeyListener(new OnKeyListener() {

			@Override
			public boolean onKey(View v, int keyCode, KeyEvent event) {
				if (event.getAction() == KeyEvent.ACTION_DOWN) {
		            switch (keyCode) {
		                case KeyEvent.KEYCODE_DPAD_CENTER:
		                case KeyEvent.KEYCODE_ENTER:
		                	if (mUserText.getText().toString().trim().equals("")) return true;
		                    sendText();
		                    return true;
		            }
		        }
		        return false;
			}
        });
        String id = PreferencesActivity.getUserLast(this);
        mUserMe = new User(id, UserList.getInstance().get(id));
        mUserYou = (User)getIntent().getSerializableExtra("object");

        //
        Log.d(DEBUG_TAG, "onResume()");
        //
    }
    
    protected void onPause() {
    	super.onPause();

    	if (mUserText.isEnabled()) {
    		exitMsg();
    	}
    	
    	// 리시버 해제
    	unregisterReceiver(mReceiver);
    	unregisterReceiver(mFinishReceiver);

        IsConv.getInstance().setConv(false);
        //
        Log.d(DEBUG_TAG, "onPause()");
        //
    }
    // 라이프싸이클 메쏘드 끝

    // 내부메소드 시작
    private void sendText() {
    	long time = System.currentTimeMillis();
		Date systemTime = new Date();
		systemTime.setTime(time);
		
		UserMsg um = new UserMsg(mUserMe, DateParser.dateToString(systemTime), mUserText.getText().toString());
		
    	mAdapter.add(um);
    	sendMsg(um);
    	mUserText.setText(null);
    }

    // 메시지를 보냄
    private void sendMsg(UserMsg userMsg) {
    	UserMsg um = new UserMsg(mUserYou, userMsg.getDate(), userMsg.getText());
    	
    	Intent intent = new Intent();
		intent.setAction(DriveService.RECV);
		intent.putExtra("activity", DriveService.SEND_ACTIVITY_MSGCONVERSATION);
		intent.putExtra("mode", MSGCONVERSATION_ACTIVITY_SEND_MSG);
		intent.putExtra("object", um);
		sendBroadcast(intent);
    }
    
    // 종료 메시지를 보냄
    private void exitMsg() {
    	Intent intent = new Intent();
		intent.setAction(DriveService.RECV);
		intent.putExtra("activity", DriveService.SEND_ACTIVITY_MSGCONVERSATION);
		intent.putExtra("mode", MSGCONVERSATION_ACTIVITY_SEND_MSG_SUCCESS);
		intent.putExtra("object", mUserYou);
		sendBroadcast(intent);
    }
    // 내부메소드 끝
    
    // 리시버 시작
    private static final String FINISHRECV = "ju.thecompany.android.FINISH";
    private FinishReceiver mFinishReceiver;

    private class FinishReceiver extends BroadcastReceiver {

		@Override
		public void onReceive(Context context, Intent intent) {

			finish();
   			//
	        Log.d(DEBUG_TAG, "finishReceiver()");
	        //
		}
    }
    
    public static final String RECV = "ju.thecompany.android.msg.MSGCONVERSATIONACTIVITY";
    MsgConversationActivityReceiver mReceiver;
	Intent mRecvIntent; 
	
	public static final int MSGCONVERSATION_ACTIVITY_RECV_MSG= 0;
	public static final int MSGCONVERSATION_ACTIVITY_RECV_MSG_SUCCESS = 1;

    public class MsgConversationActivityReceiver extends BroadcastReceiver {

		@Override
		public void onReceive(Context context, Intent intent) {
			
			mRecvIntent = intent;
			
			Thread threadProcessReceiver = new Thread(null, processReceiverThread, "Background" );
			threadProcessReceiver.start();
   			//
	        Log.d(DEBUG_TAG, "onReceive()");
	        //
		}
    }
    
    private Runnable processReceiverThread = new Runnable() {

		@Override
		public void run() {

	        handler.post(processReceiver);
		}
    	
    };
    
    private Runnable processReceiver = new Runnable() {
    	
		@Override
		public void run() {

			processReceiver();
		}
    	
    };
    
    private void processReceiver() {
    	
    	int mode = mRecvIntent.getIntExtra("mode", 0);

    	if (mode == MSGCONVERSATION_ACTIVITY_RECV_MSG_SUCCESS) {
    		Toast.makeText(this.getApplicationContext(), "상대방이 퇴장하였습니다.", Toast.LENGTH_SHORT).show();
    		mUserText.setEnabled(false);
    	}
    	if (mode == MSGCONVERSATION_ACTIVITY_RECV_MSG) {
    		mAdapter.add(new UserMsg(mUserYou, mRecvIntent.getStringExtra("date"), mRecvIntent.getStringExtra("object")));
    	}
    }
    
    // 리시버 끝
    
    // 어뎁터 시작
    public class MsgListAdapter extends ArrayAdapter<UserMsg>{

    	private ArrayList<UserMsg> items;
    	
    	public MsgListAdapter(Context context, int textViewResourceId,
    			ArrayList<UserMsg> items) {
    		super(context, textViewResourceId, items);
    		
    		this.items = items;
    		
    	}
    	
    	@Override
    	public View getView(int position, View convertView, ViewGroup parent) {
    		View v = convertView;
    		if (v == null) {
    			LayoutInflater vi = (LayoutInflater)getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    			v = vi.inflate(R.layout.conversation_msg_row, null);
    		}
    		ImageView picImageView = (ImageView) v.findViewById(R.id.conversatin_msg_row_pic);
			TextView nameTextView = (TextView) v.findViewById(R.id.conversatin_msg_row_name);
			TextView textTextView = (TextView) v.findViewById(R.id.conversatin_msg_row_text);
			TextView dateTextView = (TextView) v.findViewById(R.id.conversatin_msg_row_date);
    		
    		UserMsg userMsg = getItem(position);

    		if (userMsg != null) {
   			
    			// 데이타 입력
    			if (picImageView != null) {
    				if (!userMsg.getPic().equals("")) 
    					picImageView.setImageBitmap(BitmapFactory.decodeFile("/sdcard/" + userMsg.getPic()));
    				else
    					picImageView.setImageResource(android.R.drawable.ic_menu_camera);
    			}
    			if (nameTextView != null){
    				nameTextView.setText(userMsg.getName() + "(" + userMsg.getID()+ ") : ");
    			}
    			if (textTextView != null){
    				textTextView.setText( userMsg.getText());
    			}
    			if (dateTextView != null){
    				dateTextView.setText("(" + userMsg.getDate()+ ")");
    			}
    		}
    		return v;
    	}
    }
    // 어뎁터 끝
    // 핸들러
    private Handler handler = new Handler();
 

}
