// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18

package ju.thecompany.android.msg;

import ju.thecompany.android.R;
import ju.thecompany.android.activity.PreferencesActivity;
import ju.thecompany.android.data.IsConv;
import ju.thecompany.android.data.User;
import ju.thecompany.android.data.UserList;
import ju.thecompany.android.service.DriveService;
import android.app.Activity;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.widget.Toast;

// 메시지 요구 화면
public class MsgReqActivity extends Activity{

	// 디버그 프래그
	private static final String DEBUG_TAG = "MsgReqActivity";

	// 대화유저
	private User mUserMe;
	private User mUserYou;
	
	// 모드
	private int mMode;

	//프로그래스
	private ProgressDialog mProgressDialog;
	private int mProgress;
	private static final int MAX_PROGRESS = 60;
	
	//리턴프래그
	public static final int MSGREQ_ACTIVITY_SEND_REQ_SEND = 0;
	public static final int MSGREQ_ACTIVITY_SEND_REQ_NO= 1;
	public static final int MSGREQ_ACTIVITY_SEND_REQ_RECV_OK= 2;
	
	// 요청다이얼로그
	private static final int DIALOG_REQUEST_SEND = 0;
	private static final int DIALOG_REQUEST_RECV = 1;
	
	public static final int MODE_REQ_SEND = 0;
	public static final int MODE_REQ_RECV = 1;

    // 라이프싸이클 메쏘드 시작
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.main);

        //
        Log.d(DEBUG_TAG, "onCreate()");
        //
	}
	

    @Override
    protected void onResume() {
    	super.onResume();
    	
    	// 네트워크 종료 상태면 자동 종료
    	if (!PreferencesActivity.getConnection(this)) finish();
    	
    	// 대화중 설정
    	IsConv.getInstance().setConv(true);

    	// 리시버 등록
    	IntentFilter filter = new IntentFilter(RECV);
        mReceiver = new MsgReqActivityReceiver();
        registerReceiver(mReceiver, filter);
        filter = new IntentFilter(FINISHRECV);
        mFinishReceiver = new FinishReceiver();
        registerReceiver(mFinishReceiver, filter);
        
        String id = PreferencesActivity.getUserLast(this);
        mUserMe = new User(id, UserList.getInstance().get(id));
        mUserYou = (User)getIntent().getSerializableExtra("object");
        
       mMode = getIntent().getIntExtra("mode", 0);
        
        if (mMode ==  MODE_REQ_SEND) {
        	sendReq();
        	showDialog(DIALOG_REQUEST_SEND);
        }
        if (mMode == MODE_REQ_RECV)
        	showDialog(DIALOG_REQUEST_RECV);
        
    	mProgress = 0;
        mProgressDialog.setProgress(0);
        handlerProgress.sendEmptyMessage(0);    
        
        //
        Log.d(DEBUG_TAG, "onResume()");
        //
    }
    
    protected void onPause() {
    	super.onPause();
        
    	// 리시버 해제
    	unregisterReceiver(mReceiver);
    	unregisterReceiver(mFinishReceiver);
    	
    	// 대화중 해제
        IsConv.getInstance().setConv(false);
        
        //
        Log.d(DEBUG_TAG, "onPause()");
        //
    }
    // 라이프싸이클 메쏘드 끝
    
    // 다이얼로그 생성

	@Override
	public Dialog onCreateDialog(int id) {
		switch(id) {
		case DIALOG_REQUEST_RECV:
			mProgressDialog = new ProgressDialog(MsgReqActivity.this);
            mProgressDialog.setIcon(android.R.drawable.ic_dialog_info);
            mProgressDialog.setTitle(mUserYou.getName() +"(" + mUserYou.getDept() + ")" + getText(R.string.conversation_req_req_yn).toString());
            mProgressDialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
            mProgressDialog.setMax(MAX_PROGRESS);
            mProgressDialog.setCancelable(false);
            mProgressDialog.setButton(getText(R.string.conversation_req_ok), new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int whichButton) {
                	recvReqOK();
                	finishApp();
                	
                }
            });
            mProgressDialog.setButton2(getText(R.string.conversation_req_cancle), new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int whichButton) {
                	sendReqNo();
                	finishApp();
                }
            });

            return mProgressDialog;
            
		case MODE_REQ_SEND:
			mProgressDialog = new ProgressDialog(MsgReqActivity.this);
            mProgressDialog.setIcon(android.R.drawable.ic_dialog_info);
            mProgressDialog.setTitle(R.string.conversation_req_req_wait);
            mProgressDialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
            mProgressDialog.setMax(MAX_PROGRESS);
            mProgressDialog.setCancelable(false);
            mProgressDialog.setButton(getText(R.string.conversation_req_cancle), new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int whichButton) {
                	sendReqNo();
                	finishApp();
                }
            });

            
            return mProgressDialog;
		}
		return null;
	}
    
    // 내부메소드 시작
	private void sendReq() {
    	Intent intent = new Intent();
		intent.setAction(DriveService.RECV);
		intent.putExtra("activity", DriveService.SEND_ACTIVITY_MSGREQ);
		intent.putExtra("mode", MSGREQ_ACTIVITY_SEND_REQ_SEND);
		intent.putExtra("object", mUserYou);
		sendBroadcast(intent);
	}
	private void sendReqNo() {
    	Intent intent = new Intent();
		intent.setAction(DriveService.RECV);
		intent.putExtra("activity", DriveService.SEND_ACTIVITY_MSGREQ);
		intent.putExtra("mode", MSGREQ_ACTIVITY_SEND_REQ_NO);
		intent.putExtra("object", mUserYou);
		sendBroadcast(intent);
	}
	private void recvReqOK() {
    	Intent intent = new Intent();
		intent.setAction(DriveService.RECV);
		intent.putExtra("activity", DriveService.SEND_ACTIVITY_MSGREQ);
		intent.putExtra("mode", MSGREQ_ACTIVITY_SEND_REQ_RECV_OK);
		intent.putExtra("object", mUserYou);
		sendBroadcast(intent);
	}
	private void finishApp() {
		finish();
	}

    // 내부메소드 끝
    
    // 리시버 시작
    private static final String FINISHRECV = "ju.thecompany.android.FINISH";
    private FinishReceiver mFinishReceiver;

    private class FinishReceiver extends BroadcastReceiver {

		@Override
		public void onReceive(Context context, Intent intent) {

			finish();
   			//
	        Log.d(DEBUG_TAG, "finishReceiver()");
	        //
		}
    }
    
    public static final String RECV = "ju.thecompany.android.msg.MSGREQACTIVITY";
    MsgReqActivityReceiver mReceiver;
	Intent mRecvIntent; 
	
	public static final int MSGREQ_ACTIVITY_RECV_REQ_RECV_OK = 0;
	public static final int MSGREQ_ACTIVITY_RECV_REQ_NO = 1;
	public static final int MSGREQ_ACTIVITY_RECV_REQ_FAIL = 2;

    public class MsgReqActivityReceiver extends BroadcastReceiver {

		@Override
		public void onReceive(Context context, Intent intent) {
			
			mRecvIntent = intent;
			
			Thread threadProcessReceiver = new Thread(null, processReceiverThread, "Background" );
			threadProcessReceiver.start();
   			//
	        Log.d(DEBUG_TAG, "onReceive()");
	        //
		}
    }
    
    private Runnable processReceiverThread = new Runnable() {

		@Override
		public void run() {

	        handler.post(processReceiver);
		}
    	
    };
    
    private Runnable processReceiver = new Runnable() {
    	
		@Override
		public void run() {

			processReceiver();
		}
    	
    };
    
    private void processReceiver() {
    	
    	int mode = mRecvIntent.getIntExtra("mode", 0);

    	if (mode == MSGREQ_ACTIVITY_RECV_REQ_NO) {
    		if (mMode == MODE_REQ_SEND)
    			Toast.makeText(getApplicationContext(), R.string.conversation_req_recv_no, Toast.LENGTH_SHORT).show();
    		if (mMode == MODE_REQ_RECV)
    			Toast.makeText(getApplicationContext(), R.string.conversation_req_send_no, Toast.LENGTH_SHORT).show();
    	}
    	if (mode == MSGREQ_ACTIVITY_RECV_REQ_FAIL) {
    		Toast.makeText(getApplicationContext(), R.string.conversation_req_fail , Toast.LENGTH_SHORT).show();
    	}
    	finishApp();
    	
    }
    
    // 리시버 끝
 
    // 핸들러
    private Handler handler = new Handler();
    private Handler handlerProgress = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            super.handleMessage(msg);

            if (mProgress >= MAX_PROGRESS) {
                mProgressDialog.dismiss();
            } else {
                mProgress++;
                mProgressDialog.incrementProgressBy(1);
                handlerProgress.sendEmptyMessageDelayed(0, 1000);
            }
            
        }
    };
}
