// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18

package ju.thecompany.android.pool.buffer;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;

// 버퍼 풀
public class ByteBufferPool implements ByteBufferPoolIF {

	private static final int MEMORY_BLOCKSIZE = 1024*50;
	
	private final List memoryQueue = new ArrayList();
	
	private boolean isWait = false;
	
	public ByteBufferPool(int memorySize) throws IOException {
		if (memorySize > 0) 
			initMemoryBuffer(memorySize);
	}
	
	private void initMemoryBuffer(int size) {
		int bufferCount = size / MEMORY_BLOCKSIZE;
		size = bufferCount * MEMORY_BLOCKSIZE;
		ByteBuffer directBuf = ByteBuffer.allocateDirect(size);
		divideBuffer(directBuf, MEMORY_BLOCKSIZE, memoryQueue);
	}

	private void divideBuffer(ByteBuffer buf, int blockSize, List list) {
		int bufferCount = buf.capacity() / blockSize;
		int position = 0;
		for (int i = 0; i < bufferCount; i++) {
			int max = position + blockSize;
			buf.limit(max);
			list.add(buf.slice());
			position = max;
			buf.position(position);
		}
	}
	
	public ByteBuffer getMemoryBuffer() {
		return getBuffer(memoryQueue);
	}

	private ByteBuffer getBuffer(List firstQueue) {
		ByteBuffer buffer = getBuffer(firstQueue, false);
		if (buffer == null) {
			if (isWait)
				buffer = getBuffer(firstQueue, true);
			else
				buffer = ByteBuffer.allocate(MEMORY_BLOCKSIZE);
			
		}
		return buffer;
	}

	private ByteBuffer getBuffer(List queue, boolean wait) {
		synchronized (queue) {
			if (queue.isEmpty()) {
				if (wait) {
					try {
						queue.wait();
					} catch (InterruptedException e) {
						return null;
					}
				} else {
					return null;
				}
			}
			return (ByteBuffer) queue.remove(0);
		}
	}

	public void putBuffer(ByteBuffer buffer) {

		if (buffer.isDirect()) {
			putBuffer(buffer, memoryQueue);
		}
	}
	
	private void putBuffer(ByteBuffer buffer, List queue) {
		buffer.clear();
		synchronized (queue) {
			queue.add(buffer);
			queue.notify();
		}
	}
	
	public synchronized void setWait(boolean wait) { this.isWait = wait; }	
	public synchronized boolean isWait() { return isWait; }
}
