// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18

package ju.thecompany.android.service;

import ju.thecompany.android.R;

import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Binder;
import android.os.Handler;
import android.os.IBinder;
import android.util.Log;
import android.widget.Toast;

// 네트워크 상태 변화를 감지해서 각 액티비티와 리시버에 알린다.
public class ConnectStatusService extends Service{
	
	// 디버그 프래그
	private static final String DEBUG_TAG = "ConnectStatusService";


	// 바인더 시작
    public class ConnectStatusServiceBinder extends Binder {
    	public ConnectStatusService getService() {
            return ConnectStatusService.this;
        }
    }
 
	@Override
    public IBinder onBind(Intent intent) {
        return mBinder;
    }

    private final IBinder mBinder = new ConnectStatusServiceBinder();
    // 바인더 끝

	
    // Lifecycle 메소드 시작
	@Override
	public void onCreate() {

        //
        Log.d(DEBUG_TAG, "onCreate()");
        //
	}

    @Override
	public void onStart(Intent intent, int startId) {
    	
        startService(new Intent(this, DriveService.class));
        startService(new Intent(this, ProcessService.class));

    	IntentFilter intentFilter = new IntentFilter(CONRECV);
    	mConReceiver = new ConnectionChangeReceiver();
        registerReceiver(mConReceiver, intentFilter);
    	intentFilter = new IntentFilter(RECV);
    	mReceiver = new ConnectStatusServiceReceiver();
        registerReceiver(mReceiver, intentFilter);

        //
        Log.d(DEBUG_TAG, "onStart()");
        //
	}

    @Override
    public void onDestroy() {
    	
        stopService(new Intent(this, DriveService.class));
        stopService(new Intent(this, ProcessService.class));

        unregisterReceiver(mConReceiver);
        unregisterReceiver(mReceiver);
        
    	Toast.makeText(this, R.string.exit, Toast.LENGTH_SHORT).show();
        //
        Log.d(DEBUG_TAG, "onDestroy()");
        //
    }
    // Lifecycle 메소드 끝
    

    // 리시버
    public static final String RECV = "ju.thecompany.android.service.CONNECTSTATUSSERVICE";

    ConnectStatusServiceReceiver mReceiver;
	Intent mRecvIntent; 
	
	public static final int CONNECTSTATUS_SERVICE_RECV_DESTROY = 0;

    public class ConnectStatusServiceReceiver extends BroadcastReceiver {

		@Override
		public void onReceive(Context context, Intent intent) {
			
			mRecvIntent = intent;
			
			Thread threadProcessReceiver = new Thread(null, processReceiverThread, "Background" );
			threadProcessReceiver.start();
			//processReceiver();
   			//
	        Log.d(DEBUG_TAG, "onReceive()");
	        //
		}
    }
    
    private Runnable processReceiverThread = new Runnable() {

		@Override
		public void run() {

	        handler.post(processReceiver);
		}
    	
    };
    
    private Runnable processReceiver = new Runnable() {
    	
		@Override
		public void run() {

			processReceiver();
		}
    	
    };
    
    private void processReceiver() {
    	int mode = mRecvIntent.getIntExtra("mode", 0);
    	if (mode == CONNECTSTATUS_SERVICE_RECV_DESTROY) {
    		stopSelf();
    	}
    }
    
    // 네트워크 상태 리시버
    private ConnectionChangeReceiver mConReceiver;
	private static final String CONRECV = "android.net.conn.CONNECTIVITY_CHANGE";
	private static final String FINISHRECV = "ju.thecompany.android.FINISH";
	
    private NetworkInfo mActiveNetInfo;
    private NetworkInfo mMobNetInfo;
    
    private class ConnectionChangeReceiver extends BroadcastReceiver {

    	@Override
    	public void onReceive(Context context, Intent intent) {
    		
    		// 인터넷 상태가 바뀌면 커넥티브 메니저를 살펴봄
    		ConnectivityManager connectivityManager =
    			(ConnectivityManager)context.getSystemService(Context.CONNECTIVITY_SERVICE);
    		mActiveNetInfo = connectivityManager.getActiveNetworkInfo();
    		
    		Thread threadConReceiver = new Thread(null, conReceiverThread, "BackgroundCon" );
    		threadConReceiver.start();

    		//
            Log.d(DEBUG_TAG, "onReceive()");
            //
    	}
    }
    
    private Runnable conReceiverThread = new Runnable() {

		@Override
		public void run() {

	        handler.post(conReceiver);
		}
    	
    };
    
    private Runnable conReceiver = new Runnable() {
    	
		@Override
		public void run() {

			conProcess();
		}
    	
    };
    
    private void conProcess() {

    	// 접속이 끊겼을때
    	Intent intent = new Intent();
    	if (mActiveNetInfo == null) {
			intent.setAction(FINISHRECV);
			intent.putExtra("flag", false);
			sendBroadcast(intent);
	        stopService(new Intent(this, ProcessService.class));
	     // 접속이 될때
    	} else {
			intent.setAction(FINISHRECV);
			intent.putExtra("flag", true);
			sendBroadcast(intent);
	        startService(new Intent(this, ProcessService.class));
	        
    	}
    		
    }
    
    // 핸들러
    private Handler handler = new Handler();
}
