// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18

package ju.thecompany.android.service;

import ju.thecompany.android.R;
import ju.thecompany.android.activity.ConnectActivity;
import ju.thecompany.android.activity.LoginActivity;
import ju.thecompany.android.activity.MainMenuActivity;
import ju.thecompany.android.activity.PreferencesActivity;
import ju.thecompany.android.data.User;
import ju.thecompany.android.msg.MsgConversationActivity;
import ju.thecompany.android.msg.MsgMainActivity;
import ju.thecompany.android.msg.MsgReqActivity;
import ju.thecompany.android.noticeshort.NoticeActivity;
import ju.thecompany.android.noticeshort.ShortMsgActivity;
import ju.thecompany.android.noticeshort.ShortMsgWriteActivity;
import ju.thecompany.android.noticeshort.ShortNoticeActivity;
import ju.thecompany.android.pim.PimActivity;
import android.app.Service;
import android.content.ActivityNotFoundException;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.net.ConnectivityManager;
import android.net.Uri;
import android.os.Binder;
import android.os.Handler;
import android.os.IBinder;
import android.util.Log;
import android.widget.Toast;

// 모든 액티비티를 제어, 이벤트 제어.
public class DriveService extends Service {
	
	// 디버그 프래그
	private static final String DEBUG_TAG = "DriveService";
	
	// 재접속용
	private String mPW;

	// 바인더 시작
    public class DriveServicelBinder extends Binder {
    	DriveService getService() {
            return DriveService.this;
        }
    }
    
	@Override
    public IBinder onBind(Intent intent) {
        return mBinder;
    }

    private final IBinder mBinder = new DriveServicelBinder();
    // 바인더 끝
	
    // Lifecycle 메소드 시작
	@Override
	public void onCreate() {
		
		// 커낵션 초기화
		changeConnection(true);

    	// 리시버 등록
    	IntentFilter filter = new IntentFilter(RECV);
        mReceiver = new DriveServiceReceiver();
        registerReceiver(mReceiver, filter);
        filter = new IntentFilter(FINISHRECV);
        mFinishReceiver = new FinishReceiver();
        registerReceiver(mFinishReceiver, filter);

        //
        Log.d(DEBUG_TAG , "onCreate()");
        //
	}

    @Override
	public void onStart(Intent intent, int startId) {
    	
        startFirstActivity();
        //
        Log.d(DEBUG_TAG , "onStart()");
        //
	}

    @Override
    public void onDestroy() {

    	// 리시버 해제
        unregisterReceiver(mReceiver);
    	unregisterReceiver(mFinishReceiver);
    	
    	changePW("");
    	
        //
        Log.d(DEBUG_TAG , "onDestroy()");
        //
    }
    // Lifecycle 메소드 끝
    
	// 리시버
    private static final String FINISHRECV = "ju.thecompany.android.FINISH";
    FinishReceiver mFinishReceiver;

    private class FinishReceiver extends BroadcastReceiver {

		@Override
		public void onReceive(Context context, Intent intent) {

			boolean flag = intent.getBooleanExtra("flag", true);
			if (!flag) {
				changeConnection(false);
				refreshActivity(0);
			}
			else {
				changeConnection(true);
				Thread threadProcessReceiver = new Thread(null, changeConOKThread, "BackgroundOK" );
				threadProcessReceiver.start();
			}
   			//
	        Log.d(DEBUG_TAG, "finishReceiver()");
	        //
		}
		
		
	    private Runnable changeConOKThread = new Runnable() {

			@Override
			public void run() {
				try {
					Thread.sleep(2000);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
		        handler.post(changeConOK);
			}
	    	
	    };
	    
	    private Runnable changeConOK = new Runnable() {
	    	
			@Override
			public void run() {

				changeOK();
			}
	    	
	    };
	    
	    private void changeOK() {
	    	if (PreferencesActivity.getStartMode(getApplicationContext()) == MODE_MENU) {
		    	Intent intent = new Intent();
				intent.setAction(ProcessService.RECV);
				intent.putExtra("mode", ProcessService.PROCESS_SERVICE_RECV_LOGIN_REQ);
				PreferencesActivity.getPW(getApplicationContext());
				intent.putExtra("object", PreferencesActivity.getPW(getApplicationContext()));
				sendBroadcast(intent);
	    	}
			
	    	startFirstActivity();
	    }
		
		
    }
    
    DriveServiceReceiver mReceiver;
	public static final String RECV = "ju.thecompany.android.service.DriveService";
    private Intent mRecvIntent;
	
    public static final int SEND_ACTIVITY_LOGIN = 0;
    public static final int SEND_ACTIVITY_PREFERENCES= 1;
    public static final int SEND_ACTIVITY_MAINMENU= 3;
    public static final int SEND_ACTIVITY_MSGMAIN= 4;
    public static final int SEND_ACTIVITY_MSGREQ= 5;
    public static final int SEND_ACTIVITY_SHORTMSG = 6;
    public static final int SEND_ACTIVITY_SHORTMSGWRITE = 8;
    public static final int SEND_ACTIVITY_MSGCONVERSATION = 9;
    public static final int SEND_ACTIVITY_CONNECT = 10;
    public static final int SEND_ACTIVITY_NOTICE= 12;
    
    public static final int SEND_SERVICE_PROCESS= 2;
    public static final int SEND_SERVICE_CONNECT= 11;
    

    public class DriveServiceReceiver extends BroadcastReceiver {

		@Override
		public void onReceive(Context context, Intent intent) {
			
			mRecvIntent = intent;
			Thread threadProcessReceiver = new Thread(null, processReceiverThread, "Background" );
			threadProcessReceiver.start();
   			//
	        Log.d(DEBUG_TAG, "onReceive()");
	        //
		}
    }
    
    private Runnable processReceiverThread = new Runnable() {

		@Override
		public void run() {

	        handler.post(processReceiver);
		}
    	
    };
    
    private Runnable processReceiver = new Runnable() {
    	
		@Override
		public void run() {

			processReceiver();
		}
    	
    };
    
    private void processReceiver() {
    	int ac = mRecvIntent.getIntExtra("activity", 0);
    	int mode = mRecvIntent.getIntExtra("mode", 0);
    	Intent intent = new Intent();
    	switch (ac) {
    	// LoginActivity 에서 수신
    	case SEND_ACTIVITY_LOGIN:
    		if (mode == LoginActivity.LOGIN_ACTIVITY_SEND_LOGIN_REQ) {
    			intent.setAction(ProcessService.RECV);
    			intent.putExtra("mode", ProcessService.PROCESS_SERVICE_RECV_LOGIN_REQ);
    			mPW = mRecvIntent.getStringExtra("object");
    			intent.putExtra("object", mPW);
    			sendBroadcast(intent);
    		}
    		if (mode == LoginActivity.LOGIN_ACTIVITY_SEND_LOGIN_OK) {
    			changeMode(MODE_MENU);
    			refreshActivity(MODE_MENU);
    		}
    		if (mode == LoginActivity.LOGIN_ACTIVITY_SEND_LOGIN_CANCLE) {
    			intent.setAction(ConnectStatusService.RECV);
    			intent.putExtra("mode", ConnectStatusService.CONNECTSTATUS_SERVICE_RECV_DESTROY);
    			sendBroadcast(intent);
    		}
    		if (mode == LoginActivity.LOGIN_ACTIVITY_SEND_LOGIN_SETTING) {refreshActivity(MODE_PREFERENCES);}
    		break;
    	// PreferencesActivity에서 수신
    	case SEND_ACTIVITY_PREFERENCES:
    		changeMode(MODE_LOGIN);
    		refreshActivity(MODE_LOGIN);
    		break;
    	// ProcessService에서 수신
    	case SEND_SERVICE_PROCESS:
    		if (mode == ProcessService.PROCESS_SERVICE_SEND_ERROR_INIT) {
        		changeMode(MODE_LOGIN);
        		finishService();
    		}
    		if (mode == ProcessService.PROCESS_SERVICE_SEND_ERROR_EXIT) {
        		changeMode(MODE_LOGIN);
        		finishService();
    		}
    		if (mode == ProcessService.PROCESS_SERVICE_SEND_LOGIN_OK) {
    			changePW(mPW);
    			mPW="";
    			intent.setAction(LoginActivity.RECV);
    			intent.putExtra("mode", LoginActivity.LOGIN_ACTIVITY_RECV_LOGIN_OK );
    			sendBroadcast(intent);
    		}
    		if (mode == ProcessService.PROCESS_SERVICE_SEND_LOGIN_NO) {
    			intent.setAction(LoginActivity.RECV);
    			intent.putExtra("mode", LoginActivity.LOGIN_ACTIVITY_RECV_LOGIN_NO );
    			sendBroadcast(intent);
    		}
    		if (mode == ProcessService.PROCESS_SERVICE_SEND_LOGIN_ERROR) {
    			intent.setAction(LoginActivity.RECV);
    			intent.putExtra("mode", LoginActivity.LOGIN_ACTIVITY_RECV_LOGIN_ERR );
    			sendBroadcast(intent);
    		}
    		if (mode == ProcessService.PROCESS_SERVICE_SEND_USER_ADD) {
    			intent.setAction(MsgMainActivity.RECV);
    			intent.putExtra("mode", MsgMainActivity.MSGMAIN_ACTIVITY_RECV_USER_ADD );
    			intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
    			sendBroadcast(intent);
    		}
    		if (mode == ProcessService.PROCESS_SERVICE_SEND_USER_DEL) {
    			intent.setAction(MsgMainActivity.RECV);
    			intent.putExtra("mode", MsgMainActivity.MSGMAIN_ACTIVITY_RECV_USER_DEL );
    			intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
    			sendBroadcast(intent);
    		}
    		if (mode == ProcessService.PROCESS_SERVICE_SEND_LOGOUT_OK) {
    			changeMode(MODE_LOGIN);
    			intent.setAction(ConnectStatusService.RECV);
    			intent.putExtra("mode", ConnectStatusService.CONNECTSTATUS_SERVICE_RECV_DESTROY);
    			sendBroadcast(intent);
    		}
    		if (mode == ProcessService.PROCESS_SERVICE_SEND_SHORTMSG_REQ_ERROR) {
    			Toast.makeText(getApplicationContext(), R.string.error_short_err, Toast.LENGTH_SHORT).show();
    		}
    		if (mode == ProcessService.PROCESS_SERVICE_SEND_MSG_REQ) {
    			intent.setClass(this, MsgReqActivity.class);
    	        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
    	        intent.putExtra("mode", MsgReqActivity.MODE_REQ_RECV);
    	        intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
    	        startActivity(intent );
    		}
    		if (mode == ProcessService.PROCESS_SERVICE_SEND_MSG_REQ_OK) {
    			intent.setAction(MsgReqActivity.RECV);
    			intent.putExtra("mode", MsgReqActivity.MSGREQ_ACTIVITY_RECV_REQ_RECV_OK );
    			sendBroadcast(intent);
    			intent.setClass(this, MsgConversationActivity.class);
    	        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
    	        intent.putExtra("mode", MsgReqActivity.MODE_REQ_RECV);
    	        intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
    	        startActivity(intent );
    		}
    		if (mode == ProcessService.PROCESS_SERVICE_SEND_MSG_REQ_NO) {
    			intent.setAction(MsgReqActivity.RECV);
    			intent.putExtra("mode", MsgReqActivity.MSGREQ_ACTIVITY_RECV_REQ_NO);
    			intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
    			sendBroadcast(intent);
    		}
    		if (mode == ProcessService.PROCESS_SERVICE_SEND_MSG_FAIL) {
      			intent.setAction(MsgReqActivity.RECV);
    			intent.putExtra("mode", MsgReqActivity.MSGREQ_ACTIVITY_RECV_REQ_FAIL);
    			intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
    			sendBroadcast(intent);
    		}
    		if (mode == ProcessService.PROCESS_SERVICE_SEND_MSG) {
    			intent.setAction(MsgConversationActivity.RECV);
    			intent.putExtra("mode", MsgConversationActivity.MSGCONVERSATION_ACTIVITY_RECV_MSG);
    			intent.putExtra("object", mRecvIntent.getStringExtra("object"));
    			intent.putExtra("date", mRecvIntent.getStringExtra("date"));
    			sendBroadcast(intent);
    		}
    		if (mode == ProcessService.PROCESS_SERVICE_SEND_MSG_SUCCESS) {
    			intent.setAction(MsgConversationActivity.RECV);
    			intent.putExtra("mode", MsgConversationActivity.MSGCONVERSATION_ACTIVITY_RECV_MSG_SUCCESS);
    			sendBroadcast(intent);
    		}
    		if (mode == ProcessService.PROCESS_SERVICE_SEND_MAP) {
    			String location = mRecvIntent.getStringExtra("object");
    			if (location.equals("nomap")) 
    				Toast.makeText(getApplicationContext(), R.string.map_nofity_info, Toast.LENGTH_SHORT).show();
    			else if (location.equals("nogps"))
    				Toast.makeText(getApplicationContext(), R.string.map_nofity_gps, Toast.LENGTH_SHORT).show();
    			else {
    				intent.setAction(Intent.ACTION_VIEW);
    				intent.setData(Uri.parse("geo:" + location + "?z=19"));
        	        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
    				try {
    					startActivity(intent);
    				} catch (ActivityNotFoundException e) {
    					Toast.makeText(getApplicationContext(), R.string.map_error, Toast.LENGTH_SHORT).show();
    				}
    			}
    			
    		}
    		
    		break;
    	// MainMenuActivity 수신
    	case SEND_ACTIVITY_MAINMENU:
    		if (mode == MainMenuActivity.MAINMENU_ACTIVITY_SEND_LOGOUT) {
    			intent.setAction(ProcessService.RECV);
    			intent.putExtra("mode", ProcessService.PROCESS_SERVICE_RECV_LOGOUT);
    			sendBroadcast(intent);
    		}
    		if (mode == MainMenuActivity.MAINMENU_ACTIVITY_SEND_MESSENGER) {
    			intent.setClass(this, MsgMainActivity.class);
    	        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
    	        startActivity(intent );
    		}
    		if (mode == MainMenuActivity.MAINMENU_ACTIVITY_SEND_SHORTNOTICE) {
    			intent.setClass(this, ShortNoticeActivity.class);
    	        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
    	        startActivity(intent );
    		}
    		if (mode == MainMenuActivity.MAINMENU_ACTIVITY_SEND_PIM) {
    			intent.setClass(this, PimActivity.class);
    	        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
    	        startActivity(intent );
    		}
    		break;
    	// MsgMainActivity 수신
    	case SEND_ACTIVITY_MSGMAIN:
    		if (mode == MsgMainActivity.MSGMAIN_ACTIVITY_SEND_CONVERSATION) {
    			intent.setClass(this, MsgReqActivity.class);
    	        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
    	        intent.putExtra("mode", MsgReqActivity.MODE_REQ_SEND);
    	        intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
    	        startActivity(intent );
    		}
    		if (mode == MsgMainActivity.MSGMAIN_ACTIVITY_SEND_SHORT) {
    			intent.setClass(this, ShortMsgWriteActivity.class);
    	        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
    	        intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
    	        startActivity(intent );
    		}
    		if (mode == MsgMainActivity.MSGMAIN_ACTIVITY_SEND_CALL) {
    			User user = (User)mRecvIntent.getSerializableExtra("object");
            	intent.setAction(Intent.ACTION_CALL);
            	intent.setData(Uri.parse("tel:" + user.getHP()));
            	intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
            	startActivity(intent);
    		}
    		if (mode == MsgMainActivity.MSGMAIN_ACTIVITY_SEND_SMS) {
    			User user = (User)mRecvIntent.getSerializableExtra("object");
            	intent.setAction(Intent.ACTION_VIEW );
            	intent.putExtra("address",  user.getHP());
            	intent.putExtra("sms_body", "The SMS text"); 
            	intent.setType("vnd.android-dir/mms-sms");
            	intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
            	startActivity(intent);   
    		}
    		if (mode == MsgMainActivity.MSGMAIN_ACTIVITY_SEND_MAP) {
    			intent.setAction(ProcessService.RECV);
    			intent.putExtra("mode", ProcessService.PROCESS_SERVICE_RECV_MAP_REQ);
    			intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
    			sendBroadcast(intent);
    		}
    		break;
    	// MsgReq 수신
    	case SEND_ACTIVITY_MSGREQ:
    		if (mode == MsgReqActivity.MSGREQ_ACTIVITY_SEND_REQ_NO) {
    			intent.setAction(ProcessService.RECV);
				intent.putExtra("mode", ProcessService.PROCESS_SERVICE_RECV_MSG_REQ_NO);
		        intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
		        sendBroadcast(intent);
    		}
    		if (mode == MsgReqActivity.MSGREQ_ACTIVITY_SEND_REQ_SEND) {
    			intent.setAction(ProcessService.RECV);
				intent.putExtra("mode", ProcessService.PROCESS_SERVICE_RECV_MSG_REQ_SEND);
		        intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
		        sendBroadcast(intent);
    		}
    		if (mode == MsgReqActivity.MSGREQ_ACTIVITY_SEND_REQ_RECV_OK) {
    			intent.setAction(ProcessService.RECV);
				intent.putExtra("mode", ProcessService.PROCESS_SERVICE_RECV_MSG_REQ_RECV_OK);
		        intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
		        sendBroadcast(intent);
    			intent.setClass(this, MsgConversationActivity.class);
    	        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
    	        intent.putExtra("mode", MsgReqActivity.MODE_REQ_RECV);
    	        intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
    	        startActivity(intent );
    		}
    		break;
    	
    	case SEND_ACTIVITY_SHORTMSG:
			intent.setClass(this, ShortMsgActivity.class);
	        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
	        intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
	        startActivity(intent );
	        break;
    	case SEND_ACTIVITY_NOTICE:
			intent.setClass(this, NoticeActivity.class);
	        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
	        intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
	        startActivity(intent );
	        break;
	    // ShortMsgWrite 수신
    	case SEND_ACTIVITY_SHORTMSGWRITE:
			intent.setAction(ProcessService.RECV);
			intent.putExtra("mode", ProcessService.PROCESS_SERVICE_RECV_SHORTMSGWRITE);
			intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
			sendBroadcast(intent);
    		break;
    	case SEND_ACTIVITY_MSGCONVERSATION:
    		if (mode == MsgConversationActivity.MSGCONVERSATION_ACTIVITY_SEND_MSG) {
				intent.setAction(ProcessService.RECV);
				intent.putExtra("mode", ProcessService.PROCESS_SERVICE_RECV_MSG);
				intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
				sendBroadcast(intent);
    		}
    		if (mode == MsgConversationActivity.MSGCONVERSATION_ACTIVITY_SEND_MSG_SUCCESS) {
				intent.setAction(ProcessService.RECV);
				intent.putExtra("mode", ProcessService.PROCESS_SERVICE_RECV_MSG_SUCCESS);
				intent.putExtra("object", mRecvIntent.getSerializableExtra("object"));
				sendBroadcast(intent);
    		}
    		break;
    	case SEND_ACTIVITY_CONNECT:
    		if (mode == ConnectActivity.CONNECT_ACTIVITY_SEND_SHORTNOTICE) {
    			intent.setClass(this, ShortNoticeActivity.class);
    	        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
    	        startActivity(intent );
    		}
    		if (mode == ConnectActivity.CONNECT_ACTIVITY_SEND_SETTING) {
    			intent.setClass(this, PreferencesActivity.class);
    	        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
    	        startActivity(intent );
    		}
    		
    		if (mode == ConnectActivity.CONNECT_ACTIVITY_SEND_EXIT) {
    			
    			intent.setAction(ConnectActivity.RECV);
    			sendBroadcast(intent);

    			changeMode(MODE_LOGIN);
    			
    			intent.setAction(ConnectStatusService.RECV);
    			intent.putExtra("mode", ConnectStatusService.CONNECTSTATUS_SERVICE_RECV_DESTROY);
    			sendBroadcast(intent);
    		}
    		break;
    	}
    }
    
    
    private void finishService() {
    	this.stopSelf();
    }
    
    private static final int MODE_LOGIN  = 0;
    private static final int MODE_PREFERENCES  = 1;
    private static final int MODE_MENU = 2;
    private static final int MODE_NOCONNECT = 3;

    // 시작 엑티비티
    private void startFirstActivity() {
		switch (PreferencesActivity.getStartMode(this)) {
		case MODE_LOGIN :
			refreshActivity(MODE_LOGIN);
			break;
		case MODE_PREFERENCES :
			refreshActivity(MODE_PREFERENCES);
			break;
		case MODE_MENU :
			refreshActivity(MODE_MENU);
			break;
		case MODE_NOCONNECT :
			refreshActivity(MODE_NOCONNECT);
			break;
		}
    }
    
    // 엑티비티 리스타트
    private void refreshActivity(int mode) {
    	Intent intent = new Intent();
    	
    	ConnectivityManager connectivityManager =
			(ConnectivityManager)this.getSystemService(Context.CONNECTIVITY_SERVICE);
		if (connectivityManager.getActiveNetworkInfo() == null) {
			intent.setClass(this, ConnectActivity.class);
	        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
	        startActivity(intent );
		} else {
	    	switch (mode) {
			case MODE_LOGIN :
				intent.setClass(this, LoginActivity.class);
		        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
		        startActivity(intent );
				break;
			case MODE_PREFERENCES :
				intent.setClass(this, PreferencesActivity.class);
		        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
		        startActivity(intent );
				break;
			case MODE_MENU :
				intent.setClass(this, MainMenuActivity.class);
		        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
		        startActivity(intent );
				break;   
			case MODE_NOCONNECT :
				intent.setClass(this, ConnectActivity.class);
		        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); 
		        startActivity(intent );
				break;    
	    	}
		}

    }
    
	// 디비 영역
	private void changeMode(int startMode) {
		SharedPreferences sp = PreferencesActivity.getSharedPreferences(this);
		sp.edit().putInt(PreferencesActivity.START_MODE, startMode).commit();
	}
	
	private void changeConnection(boolean flag) {
		SharedPreferences sp = PreferencesActivity.getSharedPreferences(this);
		sp.edit().putBoolean(PreferencesActivity.CONNECTION, flag).commit();
	}
	
	private void changePW(String pw) {
		SharedPreferences sp = PreferencesActivity.getSharedPreferences(this);
		sp.edit().putString(PreferencesActivity.PASSWD, pw).commit();
	}
    
    // 핸들러
    private Handler handler = new Handler();
}
