// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18

package ju.thecompany.android.service;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import ju.thecompany.android.activity.PreferencesActivity;
import ju.thecompany.android.data.IsConv;
import ju.thecompany.android.data.User;
import ju.thecompany.android.data.UserList;
import ju.thecompany.android.data.UserMsg;
import ju.thecompany.android.data.UserNoticeShort;
import ju.thecompany.android.data.WriteQueue;
import ju.thecompany.android.db.DBAdapterIF;
import ju.thecompany.android.db.NoticeMsgDBAdapter;
import ju.thecompany.android.db.ShortMsgDBAdapter;
import ju.thecompany.android.pool.PoolManager;
import ju.thecompany.android.pool.buffer.ByteBufferPool;
import ju.thecompany.android.pool.buffer.ByteBufferPoolIF;
import ju.thecompany.data.ByteStream;
import ju.thecompany.data.CodeDesc;
import ju.thecompany.data.DateParser;
import ju.thecompany.data.TCXMLParser;
import ju.thecompany.data.TheCompanyDataToken;

import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.location.Location;
import android.location.LocationManager;
import android.os.Binder;
import android.os.Handler;
import android.os.IBinder;
import android.util.Log;

// Net 통신을 전담
public class ProcessService extends Service {
	
	// 디버그 프래그
	private static final String DEBUG_TAG = "ProcessServicee";

	// net 변수
	private Selector mSelector = null;
	private ServerSocketChannel mServerSocketChannel = null;
	private SocketChannel mConnectedServerSc = null;

	// 파일전송
	private Charset charset ;
	private CharsetDecoder decoder;
	private String filename = null;
	
	// 버퍼풀
	private ByteBufferPoolIF mByteBufferPool = null;
	
	// 쓰레드 변수
	private Thread readerThread = null; 
	private Thread connectThread = null; 
	private Thread blockingThread = null;

	// 구분자
	private static final String SEPT = "/!`/~";
	
	// 바인더 시작
    public class ProcessServiceBinder extends Binder {
    	public ProcessService getService() {
            return ProcessService.this;
        }
    }
 
	@Override
    public IBinder onBind(Intent intent) {
        return mBinder;
    }

    private final IBinder mBinder = new ProcessServiceBinder();
    // 바인더 끝

	
    // Lifecycle 메소드 시작
	@Override
	public void onCreate() {
		
		// 리시버 등록
        IntentFilter filter = new IntentFilter(RECV);
        mReceiver = new ProcessServiceReceiver();
        registerReceiver(mReceiver, filter);
		try {
			mByteBufferPool = new ByteBufferPool(1024*50*5);
			PoolManager.registByteBufferPool(mByteBufferPool);
			
			charset = Charset.forName("UTF-8");
			decoder = charset.newDecoder();
			
		} catch (IOException e) {

			Intent intent = new Intent(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
			intent.putExtra("mode", PROCESS_SERVICE_SEND_ERROR_INIT);
			sendBroadcast(intent);	
			stopSelf();
		}
        //
        Log.d(DEBUG_TAG, "onCreate()");
        //
	}

    @Override
	public void onStart(Intent intent, int startId) {
    	
    	try {
			initSelect();
		} catch (IOException e) {
			e.printStackTrace();
		}
        //
        Log.d(DEBUG_TAG, "onStart()");
        //
	}

    @Override
    public void onDestroy() {

    	// 리시버 등록 해제
        unregisterReceiver(mReceiver);
        
		try {
			unregistMyClient();
			stopReader();
			closeSelect();
			closeAllSocketChannel();
			stopBlockingServer();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			Intent intent = new Intent(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
			intent.putExtra("mode", PROCESS_SERVICE_SEND_ERROR_EXIT);
			sendBroadcast(intent);	
			stopSelf();
		}
		
        //
        Log.d(DEBUG_TAG, "onDestroy()");
        //
    }
    // Lifecycle 메소드 끝
    
    // net 메소드 시작
    // 내 클라이언트용 서버 등록
    public void initSelect() throws IOException {
    	mSelector = Selector.open();
    }
    
    public void closeSelect() throws IOException {
    	mSelector.close();
    }
 
    // 내 클라이언트를 등록
    private void registMyClient(String ServerIP, int ServerPort) throws IOException {

    	mConnectedServerSc = SocketChannel.open(new InetSocketAddress(ServerIP, ServerPort));
    	mConnectedServerSc.configureBlocking(false); 
    	//mConnectedServerSc.register(mSelector, SelectionKey.OP_READ | SelectionKey.OP_WRITE);
    	mConnectedServerSc.register(mSelector, SelectionKey.OP_READ);
        //
        Log.d(DEBUG_TAG, "registMyClient()");
        //
    }
    
    // 클라이언트 접속 종료
    public void unregistMyClient() throws IOException {
		if (mConnectedServerSc != null) mConnectedServerSc.close();
		mConnectedServerSc = null;
    }
    
    public SocketChannel getSC() {
    		return mConnectedServerSc;
    }

    // 리더 시작
    public void startReader() {
    	readerThread = new Thread(null, threadReaderProcess, "Background" );
    	readerThread.start();
    }
    
	private Runnable threadReaderProcess = new Runnable() {
		@Override
		public void run() {
	        //
	        Log.d(DEBUG_TAG, "readerStart!!!");
	        //
			try {
				while (!Thread.currentThread().isInterrupted()) {

					mSelector.select(500);
					
					Iterator it = mSelector.selectedKeys().iterator();
					while (it.hasNext()) {
						SelectionKey key = (SelectionKey) it.next();
						
						if (key.isReadable()) {
							read(key);
						}
						/*
						if (key.isWritable()) {
							write(key);
						}
*/
						it.remove();
					}								
				}      
			} catch (Exception e) {
				//e.printStackTrace();
			} 
			finally {
		        //
		        Log.d(DEBUG_TAG, "readerDown!!!");
		        //
			}
		}
	};

	// 데이타 읽어 들임
	private  void read(SelectionKey key) throws IOException, ParserConfigurationException, SAXException {
		
		ByteBufferPoolIF bufferPool = PoolManager.getByteBufferPool();
		ByteBuffer buffer = bufferPool.getMemoryBuffer();
		
		SocketChannel sc = (SocketChannel) key.channel();

		TheCompanyDataToken sendToken = new TheCompanyDataToken();
		TheCompanyDataToken recvToken = new TheCompanyDataToken();
		
		String result = "";
		Object obj = null;

		int byteRead = 0;

		for( int i =0 ; i < 2 ; i++) {
			byteRead += sc.read(buffer);
		}
		
		if (byteRead < 0 ) return;

        //
        Log.d(DEBUG_TAG, "read() : read size : " + byteRead + "byte");
        //

		buffer.flip();
		
		result = decoder.decode(buffer).toString();
		obj = TCXMLParser.parseData(sendToken, recvToken,result);
		
		processTask(sendToken, recvToken, obj);

		bufferPool.putBuffer(buffer);

	}
	
	// 버퍼를 서버로 전송
	private  void write(SelectionKey key) throws IOException, ParserConfigurationException, SAXException {
		
		if (WriteQueue.getInstance().size() == 0) return;
		
		SocketChannel sc = (SocketChannel) key.channel();
		
		ArrayList al = WriteQueue.getInstance().getQueue();

		if (al == null || al.get(0) != sc) return;
		
		ByteBuffer buffer = (ByteBuffer) al.get(1);
		
		int bt = 0;
		
		if (buffer.hasRemaining()) {
			bt += sc.write(buffer);
		}
		
        //
        Log.d(DEBUG_TAG, "write : write size : " + bt + "byte");
        //
		
	}

	// 리더 종료
	public void stopReader() {
		if (readerThread != null)
			readerThread.interrupt();
	}
	
	// 리턴 프래그
	public static final int PROCESS_SERVICE_SEND_ERROR_INIT = 0;
	public static final int PROCESS_SERVICE_SEND_ERROR_EXIT = 1;
	public static final int PROCESS_SERVICE_SEND_LOGIN_NO = 2;
	public static final int PROCESS_SERVICE_SEND_LOGIN_ERROR = 3;
	public static final int PROCESS_SERVICE_SEND_USER_ADD = 4;
	public static final int PROCESS_SERVICE_SEND_LOGIN_OK = 5; 
	public static final int PROCESS_SERVICE_SEND_LOGOUT_OK = 6;
	public static final int PROCESS_SERVICE_SEND_USER_DEL = 7;
	public static final int PROCESS_SERVICE_SEND_SHORTMSG_REQ_ERROR = 8;
	public static final int PROCESS_SERVICE_SEND_SHORTWRITE_ERROR = 9;
	public static final int PROCESS_SERVICE_SEND_MSG_REQ= 10;
	public static final int PROCESS_SERVICE_SEND_MSG_REQ_OK= 11;
	public static final int PROCESS_SERVICE_SEND_MSG_REQ_NO= 12;
	public static final int PROCESS_SERVICE_SEND_MSG = 13;
	public static final int PROCESS_SERVICE_SEND_MSG_SUCCESS = 14;
	public static final int PROCESS_SERVICE_SEND_MSG_FAIL = 15;
	public static final int PROCESS_SERVICE_SEND_MAP = 16;
	
	// 받아온 데이터로 처리
	private void processTask(TheCompanyDataToken sendToken, TheCompanyDataToken recvToken, Object obj) {
		Intent intent = new Intent();
		switch(sendToken.code) {
		case CodeDesc.CONNECT: {
				sendToken.code = CodeDesc.CONNECT_OK;
				sendToken.id = PreferencesActivity.getUserLast(getApplicationContext());
				obj = null;
				
				try {
					sendMsg(sendToken, recvToken, obj);
				} catch (IOException e2) {
					e2.printStackTrace();
				}
			}
			break;
		case CodeDesc.LOGIN_OK:
    		try {
				if (obj instanceof Hashtable <?, ?>) {
					UserList.getInstance().setAllMember((Hashtable <String, ArrayList<String>>)obj);
					
					intent.setAction(DriveService.RECV);
					intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
					intent.putExtra("mode", PROCESS_SERVICE_SEND_LOGIN_OK);
					intent.putExtra("object", new User(sendToken.id, UserList.getInstance().get(sendToken.id)));
					sendBroadcast(intent);
					
		    		sendToken.code = CodeDesc.LOGIN;
		    		sendToken.id = recvToken.id;
		    		sendToken.port = PreferencesActivity.getClientPort(getApplicationContext());
		    		long time = System.currentTimeMillis();
		    		Date systemTime = new Date();
		    		systemTime.setTime(time);
		    		sendToken.date = DateParser.dateToString(systemTime);
		    		obj = null;
				} else {
		    		sendToken.code = CodeDesc.LOGIN_FAIL;
		    		sendToken.id = recvToken.id;
		    		sendToken.port = PreferencesActivity.getClientPort(getApplicationContext());
		    		long time = System.currentTimeMillis();
		    		Date systemTime = new Date();
		    		systemTime.setTime(time);
		    		sendToken.date = DateParser.dateToString(systemTime);
		    		obj = null;
				}
			sendMsg(sendToken, recvToken, obj);
			} catch (IOException e) {
				intent.setAction(DriveService.RECV);
				intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
				intent.putExtra("mode", PROCESS_SERVICE_SEND_LOGIN_ERROR);
				sendBroadcast(intent);
				e.printStackTrace();
			}
			break;
		case CodeDesc.LOGIN_NO:
			stopReader();
			try {
				closeAllSocketChannel();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
			stopBlockingServer();
			
			intent.setAction(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
			intent.putExtra("mode", PROCESS_SERVICE_SEND_LOGIN_NO);
			sendBroadcast(intent);
			break;
		case CodeDesc.LOGIN_SUCCESS: 
			Hashtable <String, ArrayList<String>> ht = (Hashtable <String, ArrayList<String>>)obj;
			if (ht.containsKey(PreferencesActivity.getUserLast(ProcessService.this))) {
	    		sendToken.code = CodeDesc.SHORTMSG_REQ;
	    		sendToken.id = recvToken.id;
	    		sendToken.port = PreferencesActivity.getClientPort(getApplicationContext());
	    		long time = System.currentTimeMillis();
	    		Date systemTime = new Date();
	    		systemTime.setTime(time);
	    		sendToken.date = DateParser.dateToString(systemTime);
	    		obj = null;
	    		try {
					sendMsg(sendToken, recvToken, obj);
				} catch (IOException e) {
					intent.setAction(DriveService.RECV);
					intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
					intent.putExtra("mode", PROCESS_SERVICE_SEND_SHORTMSG_REQ_ERROR);
					sendBroadcast(intent);
					e.printStackTrace();
				}
			} else {	
				UserList.getInstance().setMember(ht);
				intent.setAction(DriveService.RECV);
				intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
				intent.putExtra("mode", PROCESS_SERVICE_SEND_USER_ADD);
				intent.putExtra("object", new User(sendToken.id, UserList.getInstance().get(sendToken.id)));
				sendBroadcast(intent); 
			}
			break;
		case CodeDesc.LOGOUT_OK :
			UserList.getInstance().setMember((Hashtable <String, ArrayList<String>>)obj);
			intent.setAction(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
			intent.putExtra("mode", PROCESS_SERVICE_SEND_USER_DEL);
			intent.putExtra("object", new User(sendToken.id, UserList.getInstance().get(sendToken.id)));
			sendBroadcast(intent);
			break;
		case CodeDesc.SHORTMSG_OK :
			if (obj instanceof Vector<?> ) {
				Iterator <HashMap<String, String>> i = ((Vector <HashMap<String, String>>)obj).iterator();
				
				while (i.hasNext()) {
					DBAdapterIF dba = null;
					HashMap<String, String> hm = i.next();
					if (hm.get("isshort").equals("true"))
						dba = new ShortMsgDBAdapter(this);
					else 
						dba = new NoticeMsgDBAdapter(this);
					dba.open();
					UserNoticeShort userNoticeShort = new UserNoticeShort(new User(hm.get("send_id"), 
							UserList.getInstance().get(hm.get("send_id"))),hm.get("date"), hm.get("title"), hm.get("text"));
					dba.insertMsg(userNoticeShort);
					dba.close();
				}
				
				sendToken.code = CodeDesc.SHORTMSG_SUCCESS;
	    		sendToken.id = recvToken.id;
	    		sendToken.port = PreferencesActivity.getClientPort(getApplicationContext());
	    		long time = System.currentTimeMillis();
	    		Date systemTime = new Date();
	    		systemTime.setTime(time);
	    		sendToken.date = DateParser.dateToString(systemTime);
	    		obj = null;
	    		try {
					sendMsg(sendToken, recvToken, obj);
				} catch (IOException e) { }
			}
			break;
		case CodeDesc.SHORTMSG: {
			String []result = ((String)obj).split(SEPT);
			UserNoticeShort userNoticeShort = new UserNoticeShort(new User(sendToken.id, UserList.getInstance().get(sendToken.id)), sendToken.date, result[0], result[1]);
			DBAdapterIF dba = new ShortMsgDBAdapter(this);
			dba.open();
			dba.insertMsg(userNoticeShort);
			dba.close();
			intent.setAction(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_ACTIVITY_SHORTMSG);
			intent.putExtra("object", userNoticeShort);
			sendBroadcast(intent);
			break;
		}
		case CodeDesc.NOTICE: {
			String []result = ((String)obj).split(SEPT);
			UserNoticeShort userNoticeShort = new UserNoticeShort(new User(sendToken.id, UserList.getInstance().get(sendToken.id)), sendToken.date, result[0], result[1]);
			DBAdapterIF dba = new NoticeMsgDBAdapter(this);
			dba.open();
			dba.insertMsg(userNoticeShort);
			dba.close();
			intent.setAction(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_ACTIVITY_NOTICE);
			intent.putExtra("object", userNoticeShort);
			sendBroadcast(intent);
			break;
		}
		case CodeDesc.MSG_REQ :
			if (IsConv.getInstance().isConv()) {
				recvToken.id = sendToken.id;
				
				sendToken.code = CodeDesc.MSG_FAIL;
				sendToken.id = PreferencesActivity.getUserLast(ProcessService.this);
				
				long time = System.currentTimeMillis();
	    		Date systemTime = new Date();
	    		systemTime.setTime(time);
	    		sendToken.date = DateParser.dateToString(systemTime);
	    		
	    		obj = null;
				
				try {
					sendMsg(sendToken, recvToken, obj);
				} catch (IOException e) {
					e.printStackTrace();
				}
			} else {
				User user = new User(sendToken.id, UserList.getInstance().get(sendToken.id));
				intent.setAction(DriveService.RECV);
				intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
				intent.putExtra("mode", PROCESS_SERVICE_SEND_MSG_REQ);
				intent.putExtra("object", user);
				sendBroadcast(intent);
			}
			break;
		case CodeDesc.MSG_NO :
			intent.setAction(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
			intent.putExtra("mode", PROCESS_SERVICE_SEND_MSG_REQ_NO);
			sendBroadcast(intent);
			break;
		case CodeDesc.MSG_OK :
			intent.setAction(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
			intent.putExtra("mode", PROCESS_SERVICE_SEND_MSG_REQ_OK);
			intent.putExtra("object", new User(sendToken.id, UserList.getInstance().get(sendToken.id)));
			sendBroadcast(intent);
			break;
		case CodeDesc.MSG_SUCCESS :
			intent.setAction(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
			intent.putExtra("mode", PROCESS_SERVICE_SEND_MSG_SUCCESS);
			sendBroadcast(intent);
			break;
		case CodeDesc.MSG:
			intent.setAction(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
			intent.putExtra("mode", PROCESS_SERVICE_SEND_MSG);
			intent.putExtra("date", sendToken.date);
			intent.putExtra("object", (String)obj);
			sendBroadcast(intent);
			break;
		case CodeDesc.MSG_FAIL:
			intent.setAction(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
			intent.putExtra("mode", PROCESS_SERVICE_SEND_MSG_FAIL);
			intent.putExtra("date", sendToken.date);
			intent.putExtra("object", (String)obj);
			sendBroadcast(intent);
			break;
		case CodeDesc.MAP_REQ:
			recvToken.id = sendToken.id;
			
			sendToken.code = CodeDesc.MAP;
			sendToken.id = PreferencesActivity.getUserLast(ProcessService.this);
			
			long time = System.currentTimeMillis();
    		Date systemTime = new Date();
    		systemTime.setTime(time);
    		sendToken.date = DateParser.dateToString(systemTime);

			if(PreferencesActivity.getMyLocation(ProcessService.this)) 
				try {
					if ((obj = getMyLocation()) == null) obj = "nogps";
				} catch (Exception e) {
					obj = "nogps";
				}
			else 
				obj = "nomap";
			
    		try {
				sendMsg(sendToken, recvToken, obj);
			} catch (IOException e) { }
			
			break;
		case CodeDesc.MAP:

				String location = (String)obj;

				intent.setAction(DriveService.RECV);
				intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
				intent.putExtra("mode", PROCESS_SERVICE_SEND_MAP);
				intent.putExtra("object", location);
				sendBroadcast(intent);

			break;

		}
		
	}
	
	// 채널
	public void closeAllSocketChannel() throws IOException {
		if ((mServerSocketChannel != null ) && mServerSocketChannel.isOpen()) mServerSocketChannel.close();
		if ((mConnectedServerSc != null) && mConnectedServerSc.isOpen()) mConnectedServerSc.close();
	}
	
	// 내부 메쏘드
	// 내 위치 계산
	private String getMyLocation() {
		String result =null;

	    LocationManager locationManager;
	    locationManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);
	    Location location = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
			

		if (location != null)
			 result = "" + location.getLatitude() + ", " + location.getLongitude();

	    return result;
	}
	
	// 메시지 전송
    private void sendMsg(TheCompanyDataToken sendToken, TheCompanyDataToken recvToken, Object obj) throws IOException {
    	
    	SocketChannel sc = getSC();
		String sendXML = TCXMLParser.parseString(sendToken, recvToken, obj);
		ByteBuffer encodedBuffer = ByteBuffer.wrap(sendXML.getBytes("UTF-8"));
		
		int bt = 0;
		
		if (encodedBuffer.hasRemaining()) {
			bt += sc.write(encodedBuffer);
		}
		
		//WriteQueue.getInstance().addQueue(sc, encodedBuffer);
    }
    
    // 블록킹 서버 시작
    public void startBlockingServer() {
    	blockingThread = new Thread(null, threadFileServerProcess, "BackgroundBlocking" );
    	blockingThread.start();
    }
    
    // 블록킹 서버 끝
    public void stopBlockingServer() {
    	if (blockingThread == null) return;
		if (blockingThread.isAlive())
			blockingThread.interrupt();
    }
    
    // 블록킹 서버
    private Runnable threadFileServerProcess = new Runnable() {
		private BufferedReader br = null;;
		private InputStream is = null;
		private ServerSocket serverSocket = null;
	    private Socket socket = null;
    	
		@Override
		public void run() {
	        //
	        Log.d(DEBUG_TAG, "blockingServerStart!!!");
	        //
		    try {
		    	serverSocket = new ServerSocket(PreferencesActivity.getClientPort(ProcessService.this));
		    	serverSocket.setSoTimeout(5000);
		    	//while (!Thread.currentThread().isInterrupted()) {
			    	try {
			    		socket = serverSocket.accept();
				        //
				        Log.d(DEBUG_TAG, "threadFileServerProcess : connected");
				        //
			    		processDownload();
			    	} catch (SocketTimeoutException e) { 
			    		//if (Thread.currentThread().isInterrupted()) break;
			    	}	finally {
				        //
				        Log.d(DEBUG_TAG, "blockingServerDown!!!");
				        //
					}
					
		    	//}	        
			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				if (is != null) try { is.close(); } catch (IOException e) {e.printStackTrace();}
				if (br != null) try { br.close(); } catch (IOException e) {e.printStackTrace();}
				if (socket != null) try { socket.close(); } catch (IOException e) {e.printStackTrace();}
				if (serverSocket != null) try { serverSocket.close(); } catch (IOException e) {e.printStackTrace();}

			}
		}

		public void processDownload() {
			
			try {
				is = socket.getInputStream();
				int nof_files = ByteStream.toInt(is);
				
				for (int cur_file=0;cur_file < nof_files; cur_file++) {
					
					String file_name = ByteStream.toString(is);
					File file=new File("/sdcard/" + file_name);
					ByteStream.toFile(is, file);
					
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	};
	
    // 리시버 시작
    public static final String RECV = "ju.thecompany.android.service.PROCESSSERVICE";
    ProcessServiceReceiver mReceiver;
	Intent mRecvIntent; 
	
	public static final int PROCESS_SERVICE_RECV_LOGIN_REQ = 0;
	public static final int PROCESS_SERVICE_RECV_LOGOUT = 1;
	public static final int PROCESS_SERVICE_RECV_SHORTMSGWRITE = 2;
	public static final int PROCESS_SERVICE_RECV_MSG_REQ_SEND = 3;
	public static final int PROCESS_SERVICE_RECV_MSG_REQ_NO = 4;
	public static final int PROCESS_SERVICE_RECV_MSG_REQ_RECV_OK = 5;
	public static final int PROCESS_SERVICE_RECV_MSG = 6;
	public static final int PROCESS_SERVICE_RECV_MSG_SUCCESS = 7;
	public static final int PROCESS_SERVICE_RECV_MAP_REQ = 8;

	
    public class ProcessServiceReceiver extends BroadcastReceiver {

		@Override
		public void onReceive(Context context, Intent intent) {
			
			mRecvIntent = intent;
			
			Thread threadProcessReceiver = new Thread(null, processReceiverThread, "Background" );
			threadProcessReceiver.start();
			//processReceiver();
   			//
	        Log.d(DEBUG_TAG, "onReceive()");
	        //
		}
    }
    
    private Runnable processReceiverThread = new Runnable() {

		@Override
		public void run() {

	        handler.post(processReceiver);
		}
    	
    };
    
    private Runnable processReceiver = new Runnable() {
    	
		@Override
		public void run() {

			processReceiver();
		}
    	
    };
    
    private void processReceiver() {
    	Intent intent = new Intent();
    	
    	TheCompanyDataToken sendToken = new TheCompanyDataToken();
    	TheCompanyDataToken recvToken = new TheCompanyDataToken();
    	Object obj = null;
    	
    	int mode = mRecvIntent.getIntExtra("mode", 0);
    	// LOGIN_REQ메시지 준비
    	if (mode == PROCESS_SERVICE_RECV_LOGIN_REQ) {
    		
    		try {
    			// 클라이언트 등록
				registMyClient(PreferencesActivity.getServerIP(getApplicationContext()),
						PreferencesActivity.getServerPort(getApplicationContext()));
				startReader();					// 셀렉터 리더 스타트
				startBlockingServer();		// 블록킹 서버 스타트
			} catch (IOException e) {
				stopReader();
				stopBlockingServer();
				try {
					closeAllSocketChannel();
				} catch (IOException e1) {
					e1.printStackTrace();
				}

				intent.setAction(DriveService.RECV);
				intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
				intent.putExtra("mode", PROCESS_SERVICE_SEND_LOGIN_ERROR);
				sendBroadcast(intent);
				e.printStackTrace();
			}

    		sendToken.code = CodeDesc.LOGIN_REQ;
    		sendToken.id = PreferencesActivity.getUserLast(getApplicationContext());
    		sendToken.port = PreferencesActivity.getClientPort(getApplicationContext());
 
    		obj = mRecvIntent.getStringExtra("object");
    	}
    	if (mode == PROCESS_SERVICE_RECV_LOGOUT) {
    		sendToken.code = CodeDesc.LOGOUT;
    		sendToken.id = PreferencesActivity.getUserLast(getApplicationContext());
    		
			intent.setAction(DriveService.RECV);
			intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
			intent.putExtra("mode", PROCESS_SERVICE_SEND_LOGOUT_OK);
			sendBroadcast(intent);
    	}
		if (mode == PROCESS_SERVICE_RECV_SHORTMSGWRITE) {
			UserNoticeShort usn = (UserNoticeShort)mRecvIntent.getSerializableExtra("object");
			sendToken.code = CodeDesc.SHORTMSG;
    		sendToken.id = PreferencesActivity.getUserLast(getApplicationContext());
    		
    		recvToken.id = usn.getID();
    		obj = (String)(usn.getTitle() + SEPT + usn.getText());
		}
		if (mode == PROCESS_SERVICE_RECV_MSG_REQ_SEND) {
			User user = (User)mRecvIntent.getSerializableExtra("object");
			sendToken.code = CodeDesc.MSG_REQ;
    		sendToken.id = PreferencesActivity.getUserLast(getApplicationContext());
    		
    		recvToken.id = user.getID();
    		obj = null;
		}
		if (mode == PROCESS_SERVICE_RECV_MSG_REQ_NO) {
			User user = (User)mRecvIntent.getSerializableExtra("object");
			sendToken.code = CodeDesc.MSG_NO;
    		sendToken.id = PreferencesActivity.getUserLast(getApplicationContext());
    		
    		recvToken.id = user.getID();
    		obj = null;
		}
		if (mode == PROCESS_SERVICE_RECV_MSG_REQ_RECV_OK) {
			User user = (User)mRecvIntent.getSerializableExtra("object");
			sendToken.code = CodeDesc.MSG_OK;
    		sendToken.id = PreferencesActivity.getUserLast(getApplicationContext());
    		
    		recvToken.id = user.getID();
    		obj = null;
		}
		if (mode == PROCESS_SERVICE_RECV_MSG) {
			UserMsg um = (UserMsg)mRecvIntent.getSerializableExtra("object");
			sendToken.code = CodeDesc.MSG;
    		sendToken.id = PreferencesActivity.getUserLast(getApplicationContext());
    		recvToken.id = um.getID();
    		obj = um.getText();
		}
		if (mode == PROCESS_SERVICE_RECV_MSG_SUCCESS) {
			User um = (User)mRecvIntent.getSerializableExtra("object");
			sendToken.code = CodeDesc.MSG_SUCCESS;
    		sendToken.id = PreferencesActivity.getUserLast(getApplicationContext());
    		recvToken.id = um.getID();
    		obj = null;
		}
		if (mode == PROCESS_SERVICE_RECV_MAP_REQ) {
			User um = (User)mRecvIntent.getSerializableExtra("object");
			sendToken.code = CodeDesc.MAP_REQ;
    		sendToken.id = PreferencesActivity.getUserLast(getApplicationContext());
    		recvToken.id = um.getID();
    		obj = null;
		}
    	
		long time = System.currentTimeMillis();
		Date systemTime = new Date();
		systemTime.setTime(time);
		sendToken.date = DateParser.dateToString(systemTime);
		
		try {
			// LOGIN_REQ메시지 보냄
			sendMsg(sendToken, recvToken, obj);
		} catch (IOException e) {
			e.printStackTrace();
			
			if (mode == PROCESS_SERVICE_RECV_LOGIN_REQ) {  
				intent.setAction(DriveService.RECV);
				intent.putExtra("activity", DriveService.SEND_SERVICE_PROCESS);
				intent.putExtra("mode", PROCESS_SERVICE_SEND_LOGIN_ERROR);
				sendBroadcast(intent);
			}
		}

    }
    
    // 핸들러
    private Handler handler = new Handler();
	
	

}
