// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18

package ju.thecompany.data;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


public class TCXMLParser {
	public static synchronized String parseString(TheCompanyDataToken sendToken, TheCompanyDataToken recvToken, Object obj) throws XMLStreamException {

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
	    XMLOutputFactory factory = XMLOutputFactory.newInstance();
	    XMLStreamWriter writer = factory.createXMLStreamWriter(baos);

	    writer.writeStartDocument("UTF-8", "1.0");

	    writer.writeStartElement("document");
	    writer.writeAttribute("key", "code");
	    writer.writeAttribute("value", "" + sendToken.code);

	    writer.writeStartElement("category");
	    writer.writeAttribute("key", "datatype");
	    writer.writeAttribute("value", "userdata");
	    writer.writeAttribute("flag", ""); 

	    writer.writeStartElement("item");
	    writer.writeEmptyElement("element");
	    writer.writeAttribute("key", "id");
	    writer.writeAttribute("value", sendToken.id);
	    writer.writeEmptyElement("element");
	    writer.writeAttribute("key", "ip");
	    writer.writeAttribute("value", sendToken.ip);
	    writer.writeEmptyElement("element");
	    writer.writeAttribute("key", "port");
	    writer.writeAttribute("value", "" + sendToken.port);
	    writer.writeEmptyElement("element");
	    writer.writeAttribute("key", "date");
	    writer.writeAttribute("value", sendToken.date);
	    writer.writeEndElement();
	    
	    writer.writeEndElement();

	    writer.writeStartElement("category");
	    writer.writeAttribute("key", "datatype");
	    writer.writeAttribute("value", "targetuserdata");
	    writer.writeAttribute("flag", ""); 
	    
	    writer.writeStartElement("item");
	    writer.writeEmptyElement("element");
	    writer.writeAttribute("key", "id");
	    writer.writeAttribute("value", recvToken.id);
	    writer.writeEndElement();
	    
	    writer.writeEndElement();
	    
	    writer.writeStartElement("category");
	    writer.writeAttribute("key", "datatype");
	    writer.writeAttribute("value", "userdata");

	    if ((obj != null) && (obj instanceof Hashtable<?, ?>) ) {
	    	writer.writeAttribute("flag", "hash"); 
	    	Hashtable <String, ArrayList <String>> ht = (Hashtable <String, ArrayList<String>>)obj;
		    Enumeration<String> e = ht.keys();
		    
		    if ( e != null) {
		    	while(e.hasMoreElements()) {
				    writer.writeStartElement("item");

				    String id = e.nextElement();
				    writer.writeEmptyElement("element");
				    writer.writeAttribute("key", "id");
				    writer.writeAttribute("value", id);
				    
				    ArrayList <String> al = ht.get(id);
				    writer.writeEmptyElement("element");
				    writer.writeAttribute("key", "ip");
				    writer.writeAttribute("value", al.get(0));
				    writer.writeEmptyElement("element");
				    writer.writeAttribute("key", "port");
				    writer.writeAttribute("value", al.get(1));
				    writer.writeEmptyElement("element");
				    writer.writeAttribute("key", "name");
				    writer.writeAttribute("value", al.get(2));
				    writer.writeEmptyElement("element");
				    writer.writeAttribute("key", "dept");
				    writer.writeAttribute("value", al.get(3));
				    writer.writeEmptyElement("element");
				    writer.writeAttribute("key", "hp");
				    writer.writeAttribute("value", al.get(4));
				    writer.writeEmptyElement("element");
				    writer.writeAttribute("key", "logoutdate");
				    writer.writeAttribute("value", al.get(5));
				    writer.writeEmptyElement("element");
				    writer.writeAttribute("key", "flag");
				    writer.writeAttribute("value", al.get(6));
				    writer.writeEmptyElement("element");
				    writer.writeAttribute("key", "pic");
				    writer.writeAttribute("value", al.get(7));
				    writer.writeEndElement();
		    	}
		    }
	    }
	    
	    if ((obj != null) && (obj instanceof Vector<?>) ) {
	    	writer.writeAttribute("flag", "vector"); 
	    	Vector <HashMap <String, String>> v = (Vector <HashMap <String, String>>)obj;
		    
	    	Iterator <HashMap <String, String>> i = v.iterator();
		    
		    while( i.hasNext() ) {
		    	HashMap <String, String> hm = i.next();
		    	
		    	writer.writeStartElement("item");
		    	
			    writer.writeEmptyElement("element");
			    writer.writeAttribute("key", "send_id");
			    writer.writeAttribute("value", hm.get("send_id"));
			    
			    writer.writeEmptyElement("element");
			    writer.writeAttribute("key", "recv_id");
			    writer.writeAttribute("value", hm.get("recv_id"));
			    
			    writer.writeEmptyElement("element");
			    writer.writeAttribute("key", "date");
			    writer.writeAttribute("value", hm.get("date"));
			    
			    writer.writeEmptyElement("element");
			    writer.writeAttribute("key", "title");
			    writer.writeAttribute("value", hm.get("title"));
			    
			    writer.writeEmptyElement("element");
			    writer.writeAttribute("key", "text");
			    writer.writeAttribute("value", hm.get("text"));
			    
			    writer.writeEmptyElement("element");
			    writer.writeAttribute("key", "isshort");
			    writer.writeAttribute("value", hm.get("isshort"));

			    writer.writeEndElement();

		    }
	    }
	    
	    if ((obj != null) && (obj instanceof String) ) {
	    	writer.writeAttribute("flag", "text"); 
	    	writer.writeStartElement("item");
		    writer.writeEmptyElement("element");
		    writer.writeAttribute("key", "text");
		    writer.writeAttribute("value", (String)obj);
		    writer.writeEndElement();
	    }
	    
	    if (obj == null) {
	    	writer.writeAttribute("flag", "empty"); 
	    	writer.writeStartElement("item");
		    writer.writeEndElement();
	    }

	    writer.writeEndDocument();

	    writer.flush();
	    writer.close();
	    
	    return  new String(baos.toByteArray());
	}

	public static synchronized Object parseData(TheCompanyDataToken emptySendToken, TheCompanyDataToken emptyRecvToken, String xmlString) throws ParserConfigurationException, SAXException, IOException {
		
		Object result = null;
	
		InputStream is = new ByteArrayInputStream(xmlString.getBytes("UTF-8"));
		DocumentBuilder documentBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		Document doc = documentBuilder.parse(is);
		
		NodeList docList = doc.getChildNodes();

		NamedNodeMap namedNode = docList.item(0).getAttributes();
		String key = namedNode.getNamedItem("key").getNodeValue();
		emptySendToken.code = Integer.parseInt(namedNode.getNamedItem("value").getNodeValue());
		docList = docList.item(0).getChildNodes();
		for(int i =0; i < docList.getLength() ; i++) {
			namedNode = docList.item(i).getAttributes();
			key = namedNode.getNamedItem("key").getNodeValue();
			String flag = namedNode.getNamedItem("flag").getNodeValue();
			String value = namedNode.getNamedItem("value").getNodeValue();
			if ( value.equals("userdata") && ( flag.equals("")) ) {
				Node dataNode = docList.item(i).getFirstChild();
				NodeList categoryList = dataNode.getChildNodes();
				for (int j = 0; j < categoryList.getLength(); j++) {
					NamedNodeMap usermap = categoryList.item(j).getAttributes();
					key = usermap.getNamedItem("key").getNodeValue();
					value = usermap.getNamedItem("value").getNodeValue();
					if (key.equals("id")) emptySendToken.id = value;
					if (key.equals("ip")) emptySendToken.ip = value;
					if (key.equals("port")) emptySendToken.port = Integer.parseInt(value);
					if (key.equals("date")) emptySendToken.date = value;
				}
			}
			if ( value.equals("targetuserdata") && ( flag.equals("")) ) {
				Node dataNode = docList.item(i).getFirstChild();
				NodeList categoryList = dataNode.getChildNodes();
				for (int j = 0; j < categoryList.getLength(); j++) {
					NamedNodeMap usermap = categoryList.item(j).getAttributes();
					key = usermap.getNamedItem("key").getNodeValue();
					value = usermap.getNamedItem("value").getNodeValue();
					if (key.equals("id")) emptyRecvToken.id = value;
				}
			}
			if ( value.equals("userdata") && ( flag.equals("hash")) ) {
				
				NodeList itemList =  docList.item(i).getChildNodes();
				Hashtable <String, ArrayList<String>> ht = new Hashtable <String, ArrayList<String>>();
				for (int j = 0; j < itemList.getLength(); j++) {
					NodeList elementList = itemList.item(j).getChildNodes();
					String id = null;
					ArrayList <String> al = new ArrayList <String> (8);
					for (int k = 0; k < elementList.getLength(); k++) {
						NamedNodeMap usermap = elementList.item(k).getAttributes();
						key = usermap.getNamedItem("key").getNodeValue();
						value = usermap.getNamedItem("value").getNodeValue();
						if (key.equals("id")) id = value;
						if (key.equals("ip")) al.add(0, value);
						if (key.equals("port")) al.add(1, value);
						if (key.equals("name")) al.add(2, value);
						if (key.equals("dept")) al.add(3, value);
						if (key.equals("hp")) al.add(4, value);
						if (key.equals("logoutdate")) al.add(5, value);
						if (key.equals("flag")) al.add(6, value);
						if (key.equals("pic")) al.add(7, value);
					}
					ht.put(id, al);
				}
				result = ht;
			}
			if ( value.equals("userdata") && ( flag.equals("vector")) ) {
				
				NodeList itemList =  docList.item(i).getChildNodes();
				Vector <HashMap <String, String>> v = new Vector <HashMap <String, String>>();
				for (int j = 0; j < itemList.getLength(); j++) {
					NodeList elementList = itemList.item(j).getChildNodes();
					HashMap <String, String> hm = new HashMap <String, String>();
					for (int k = 0; k < elementList.getLength(); k++) {
						NamedNodeMap usermap = elementList.item(k).getAttributes();
						key = usermap.getNamedItem("key").getNodeValue();
						value = usermap.getNamedItem("value").getNodeValue();
						if (key.equals("send_id")) hm.put(key, value);
						if (key.equals("recv_id")) hm.put(key, value);
						if (key.equals("date")) hm.put(key, value);
						if (key.equals("title")) hm.put(key, value);
						if (key.equals("text")) hm.put(key, value);
						if (key.equals("isshort")) hm.put(key, value);
					}
					v.add(hm);
				}
				result = v;
			}
			if ( value.equals("userdata") && ( flag.equals("text")) ) {
				Node itemNode =  docList.item(i).getFirstChild().getFirstChild();
				NamedNodeMap textmap =itemNode.getAttributes();
				result = textmap.getNamedItem("value").getNodeValue();
			}
			if ( value.equals("userdata") && ( flag.equals("empty")) ) {
				result = null;
			}
		}


		return result;
	
	}
}
