// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18

package ju.thecompany.server.controlserver.dbprocess;

import ju.thecompany.data.DateParser;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Date;
import java.util.HashMap;
import java.util.Vector;

// DB 제어 클래스
public class DBControl {
	private static DBControl singleton  = new DBControl();
	private MysqlDBControl mysqlDBControl;
	private Connection connection;
	
	private DBControl() {
		mysqlDBControl = new MysqlDBControl();
	}
	
	public static DBControl getInstance() {
		return singleton;
	}
	
	public void DBOpen(String ip, String port, String dbName, String id, String pw) throws SQLException {
		connection = mysqlDBControl.Open(ip, port, dbName, id, pw);	// DB의 커넥션을 획득해 온다.
	}
	
	public void DBClose() {
		mysqlDBControl.DBClose();	// DB를 닫는다.
	}
	
	// 로그인을 확인 한 후, 결과를 반환
	public boolean Login(String id, String pw) throws SQLException {
		ResultSet rs = null;
		PreparedStatement pstmt = null;
		boolean result = false;
		
		// 로그인을 확인하기 위해 해당 id로 select한 후 존재여부와 passwd를 확인한 후 리턴
		try {
			pstmt = connection.prepareStatement("SELECT id, passwd FROM MEMBER WHERE id = ?");
			pstmt.setString(1, id);
			rs = pstmt.executeQuery();
			if ( (rs.last()) && (rs.getRow() == 1)) {
				String dbPassword = rs.getString("passwd");
				if ( dbPassword.equals(pw) ) {
					result = true;
				}
			}
		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (pstmt != null) try { pstmt.close(); } catch (SQLException e) { e.printStackTrace(); }		
		}
		return result;

	}
	
	// 지정된 ID를 가진 사용자의 정보를 리턴
	// (필드, 값) 형태의 HashMap 객체를 반환
	public HashMap<String, String> GetUser(String id) throws SQLException {
		ResultSet rs = null;
		PreparedStatement pstmt = null;
		HashMap<String, String> userInformationHashMap = null;
		
		try {
			pstmt = connection.prepareStatement("SELECT * FROM MEMBER WHERE id = ?");
			pstmt.setString(1, id);
			rs = pstmt.executeQuery();
			userInformationHashMap = new HashMap<String, String>();
			// 결과 ResultSet를 분석하여 HashMap 객체에 저장한 후 반환
			if ( (rs.last()) && (rs.getRow() == 1)) {
				userInformationHashMap.put("id", rs.getString("id"));
				userInformationHashMap.put("name", rs.getString("name"));
				userInformationHashMap.put("email", rs.getString("email"));
				userInformationHashMap.put("addr", rs.getString("addr"));
				userInformationHashMap.put("tel", rs.getString("tel"));
				userInformationHashMap.put("hp", rs.getString("hp"));
				userInformationHashMap.put("pic", rs.getString("pic"));
				userInformationHashMap.put("chklogin", (rs.getBoolean("chklogin") ? "true" : "false" ));
				userInformationHashMap.put("deptname", rs.getString("deptname"));
				userInformationHashMap.put("posname", rs.getString("posname"));	
				userInformationHashMap.put("logoutdate",DateParser.dateToString(new java.util.Date(rs.getTimestamp("logoutdate").getTime())));
				userInformationHashMap.put("enterdate",DateParser.dateToString(new java.util.Date(rs.getTimestamp("enterdate").getTime())));
			}
			rs.close();
			pstmt.close();
		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (pstmt != null) try { pstmt.close(); } catch (SQLException e) { e.printStackTrace(); }		
		}
		return userInformationHashMap;
	}
	
	// 지정된 ID를 가진 사용자의 정보를 리턴
	// (필드, 값) 형태의 HashMap 객체를 반환
	public Vector <HashMap<String, String>> GetUserList() throws SQLException {
		ResultSet rs = null;
		Statement stmt = null;
		Vector <HashMap<String, String>> userList = new Vector <HashMap<String, String>>();
		
		HashMap<String, String> userInformationHashMap = null;
		try {
			stmt = connection.createStatement();
			rs = stmt.executeQuery("SELECT * FROM MEMBER");

			while (rs.next()) {
				
				userInformationHashMap = new HashMap<String, String>();
				// 결과 ResultSet를 분석하여 HashMap 객체에 저장

				userInformationHashMap.put("id", rs.getString("id"));
				userInformationHashMap.put("name", rs.getString("name"));
				userInformationHashMap.put("email", rs.getString("email"));
				userInformationHashMap.put("addr", rs.getString("addr"));
				userInformationHashMap.put("tel", rs.getString("tel"));
				userInformationHashMap.put("hp", rs.getString("hp"));
				userInformationHashMap.put("pic", rs.getString("pic"));
				userInformationHashMap.put("chklogin", (rs.getBoolean("chklogin") ? "true" : "false" ));
				userInformationHashMap.put("deptname", rs.getString("deptname"));
				userInformationHashMap.put("posname", rs.getString("posname"));	
				userInformationHashMap.put("logoutdate",DateParser.dateToString(new java.util.Date(rs.getTimestamp("logoutdate").getTime())));
				userInformationHashMap.put("enterdate",DateParser.dateToString(new java.util.Date(rs.getTimestamp("enterdate").getTime())));
				
				userList.add(userInformationHashMap);

			}
			rs.close();
			stmt.close();
		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }		
		}
		return userList;
	}
	
	public void setLogoutDate(String key, Date date) throws SQLException {
		Statement stmt = null;
		
		try {
			stmt = connection.createStatement();
			stmt.executeUpdate("UPDATE MEMBER SET logoutdate = '" + DateParser.dateToString(date) + "' WHERE id = '" + key + "'");
			stmt.close();
		} finally {
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
		}
	}
	
	public void setChkLogin(String key, boolean flag) throws SQLException {
		Statement stmt = null;
		
		try {
			stmt = connection.createStatement();
			stmt.executeUpdate("UPDATE MEMBER SET chklogin = '" + (flag ? 1 : 0) + "' WHERE id = '" + key + "'");
			stmt.close();
		} finally {
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
		}
	}
	
	public void setShortNotice(String sendID, String recvID, String title, String text, boolean isShort) throws SQLException {
		Statement stmt = null;

		try {
			stmt = connection.createStatement();
			stmt.executeUpdate("INSERT INTO SHORTNOTICE(send_id, recv_id, date, title, text, isshort) VALUES ('" + 
					sendID + "', '" +recvID + "', now(), '" + title  + "', '" + text + "', '"  + (isShort ? "1'" : "0'") + ")" );
			stmt.close();
		} finally {
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
		}
	}
	
	public Vector <HashMap<String, String>> getShortNotice(String recvID) throws SQLException {
		ResultSet rs = null;
		Statement stmt = null;
		Vector <HashMap<String, String>> shortNotice = new Vector <HashMap<String, String>>();
		
		HashMap<String, String> shortNoticeMap = null;
		try {
			stmt = connection.createStatement();
			rs = stmt.executeQuery("SELECT * FROM SHORTNOTICE WHERE recv_id='" + recvID + "'");

			while (rs.next()) {
				
				shortNoticeMap = new HashMap<String, String>();
				// 결과 ResultSet를 분석하여 HashMap 객체에 저장

				shortNoticeMap.put("send_id", rs.getString("send_id"));
				shortNoticeMap.put("recv_id", rs.getString("recv_id"));
				shortNoticeMap.put("title", rs.getString("title"));
				shortNoticeMap.put("text", rs.getString("text"));
				shortNoticeMap.put("isshort", (rs.getBoolean("isshort") ? "true" : "false" ));
				shortNoticeMap.put("date",DateParser.dateToString(new java.util.Date(rs.getTimestamp("date").getTime())));
				
				shortNotice.add(shortNoticeMap);

			}
			rs.close();
			stmt.close();

			
		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }		
		}
		return shortNotice;
	}
	
	public void removeShortNotice(String recvID) throws SQLException {
		Statement stmt = null;

		try {
			stmt = connection.createStatement();
			stmt.executeUpdate("DELETE FROM SHORTNOTICE WHERE recv_id='" + recvID + "'");
			stmt.close();
		} finally {
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
		}
	}
}
