// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18

package ju.thecompany.server.controlserver.serverprocess;

import java.io.File;
import java.io.IOException;
import java.sql.SQLException;

import ju.thecompany.data.Setting;
import ju.thecompany.server.controlserver.dbprocess.DBControl;

import ju.thecompany.server.controlserver.serverprocess.jobqueue.BlockingEventQueue;
import ju.thecompany.server.controlserver.serverprocess.jobqueue.Queue;
import ju.thecompany.server.controlserver.serverprocess.pool.PoolManager;
import ju.thecompany.server.controlserver.serverprocess.pool.buffer.ByteBufferPool;
import ju.thecompany.server.controlserver.serverprocess.pool.buffer.ByteBufferPoolIF;
import ju.thecompany.server.controlserver.serverprocess.pool.selector.AcceptSelectorPool;
import ju.thecompany.server.controlserver.serverprocess.pool.selector.RequestSelectorPool;
import ju.thecompany.server.controlserver.serverprocess.pool.selector.SelectorPoolIF;
import ju.thecompany.server.controlserver.serverprocess.pool.thread.ThreadPool;
import ju.thecompany.server.controlserver.serverprocess.pool.thread.ThreadPoolIF;

public class ControlServerMain {

	// 싱글레톤 영역
	private static ControlServerMain singleton = new ControlServerMain();
	
	private ControlServerMain() {
	}
	
	public static ControlServerMain getInstance() {
		return singleton;
	}
	// 싱글레톤 영역 끝
	
	
	// 글로벌 변수 영역
	private Queue queue = null;
	private SelectorPoolIF acceptSelectorPool = null;
	private SelectorPoolIF requestSelectorPool = null;
	private ByteBufferPoolIF byteBufferPool = null;
	ThreadPoolIF acceptThreadPool = null;
	ThreadPoolIF readWriteThreadPool = null;
	// 글로벌 변수 영역 끝
	
	
	// 서버 초기화 영역
	public void initServer() throws IOException {
		// ByteBufferPool 생성..
		File bufferFile = new File("Buffer.tmp");
		if (!bufferFile.exists()) {
			bufferFile.createNewFile();
		}
		bufferFile.deleteOnExit();
		byteBufferPool = new ByteBufferPool(1024*50*10, 1024*100*10, bufferFile);
		
		// Queue 생성..
		queue = BlockingEventQueue.getInstance();		
		
		// PoolManager 에 ByteBufferPool 등록..
		PoolManager.registByteBufferPool(byteBufferPool);
		
		// ThreadPool 생성..
		acceptThreadPool = new ThreadPool(
				queue, "ju.thecompany.server.controlserver.serverprocess.pool.thread.processor.AcceptProcessor");
		readWriteThreadPool = new ThreadPool(
				queue, "ju.thecompany.server.controlserver.serverprocess.pool.thread.processor.ReadWriteProcessor");

		// SelectorPool 생성..
		acceptSelectorPool = new AcceptSelectorPool(queue);
		requestSelectorPool = new RequestSelectorPool(queue);
		
		// PoolManager 에 SelectorPool 등록..
		PoolManager.registAcceptSelectorPool(acceptSelectorPool);
		PoolManager.registRequestSelectorPool(requestSelectorPool);	
	}
	// 서버 초기화 영역 끝
	
	
	// 서버 시작 영역
	public void startServer() throws SQLException {
		Setting setting = Setting.getInstance();
		DBControl dbc = DBControl.getInstance();
		
		dbc.DBOpen(setting.getData("MysqlIP"),
				setting.getData("MysqlPort"),
				setting.getData("MysqlDBName"), 
				setting.getData("MysqlID"), 
				setting.getData("MysqlPassword"));
		
		acceptThreadPool.startAll();
		readWriteThreadPool.startAll();
		
		acceptSelectorPool.startAll();
		requestSelectorPool.startAll();		
		
		(new FileBlockingServer()).start();
		(new ConnectionStatus()).start();
	}
	
	// 서버 시작 영역 끝
}
