// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18

package ju.thecompany.server.controlserver.serverprocess.distributor;

import java.nio.channels.SocketChannel;

import ju.thecompany.data.CodeDesc;
import ju.thecompany.data.TCXMLParser;
import ju.thecompany.data.TheCompanyDataToken;
import ju.thecompany.server.controlserver.serverprocess.task.ConnectOKTask;
import ju.thecompany.server.controlserver.serverprocess.task.ConnectTask;
import ju.thecompany.server.controlserver.serverprocess.task.LoginReqTask;
import ju.thecompany.server.controlserver.serverprocess.task.LoginTask;
import ju.thecompany.server.controlserver.serverprocess.task.LogoutTask;
import ju.thecompany.server.controlserver.serverprocess.task.MapReqTask;
import ju.thecompany.server.controlserver.serverprocess.task.MapTask;
import ju.thecompany.server.controlserver.serverprocess.task.MsgFailTask;
import ju.thecompany.server.controlserver.serverprocess.task.MsgNOTask;
import ju.thecompany.server.controlserver.serverprocess.task.MsgOKTask;
import ju.thecompany.server.controlserver.serverprocess.task.MsgReqTask;
import ju.thecompany.server.controlserver.serverprocess.task.MsgSuccessTask;
import ju.thecompany.server.controlserver.serverprocess.task.MsgTask;
import ju.thecompany.server.controlserver.serverprocess.task.NoticeTask;
import ju.thecompany.server.controlserver.serverprocess.task.ShortMsgReqTask;
import ju.thecompany.server.controlserver.serverprocess.task.ShortMsgSuccessTask;
import ju.thecompany.server.controlserver.serverprocess.task.ShortMsgTask;
import ju.thecompany.server.controlserver.serverprocess.task.TaskIF;

// 제어 토큰의 코드 명으로 작업을 선택
public class TaskDistributor {
	
	private String xml;
	private SocketChannel sc;
	
	private TaskIF taskProcess;
	
	public TaskDistributor(String xml, SocketChannel sc) throws Exception {
		this.xml = xml;
		this.sc = sc;
		processDistribute();
	}
	
	private void processDistribute() throws Exception {
		TheCompanyDataToken emptySendToken = new TheCompanyDataToken();
		TheCompanyDataToken emptyRecvToken = new TheCompanyDataToken();
		Object obj = null;
		
		obj = TCXMLParser.parseData(emptySendToken, emptyRecvToken, xml);
		
		switch (emptySendToken.code) {
		case CodeDesc.LOGIN_REQ:
			taskProcess = new LoginReqTask(emptySendToken, emptyRecvToken, obj, sc);
			break;
		case CodeDesc.LOGIN:
			taskProcess = new LoginTask(emptySendToken, emptyRecvToken, obj, sc);
			break;
		case CodeDesc.LOGOUT:
			taskProcess = new LogoutTask(emptySendToken, emptyRecvToken, obj, sc);
			break;
		case CodeDesc.MSG_REQ:
			taskProcess = new MsgReqTask(emptySendToken, emptyRecvToken, obj, sc);
			break;
		case CodeDesc.MSG_OK:
			taskProcess = new MsgOKTask(emptySendToken, emptyRecvToken, obj, sc);
			break;
		case CodeDesc.MSG_NO:
			taskProcess = new MsgNOTask(emptySendToken, emptyRecvToken, obj, sc);
			break;
		case CodeDesc.MSG:
			taskProcess = new MsgTask(emptySendToken, emptyRecvToken, obj, sc);
			break;
		case CodeDesc.MSG_SUCCESS:
			taskProcess = new MsgSuccessTask(emptySendToken, emptyRecvToken, obj, sc);
			break;
		case CodeDesc.MSG_FAIL:
			taskProcess = new MsgFailTask(emptySendToken, emptyRecvToken, obj, sc);
			break;
		case CodeDesc.SHORTMSG:
			taskProcess = new ShortMsgTask(emptySendToken, emptyRecvToken, obj, sc);
			break;
		case CodeDesc.SHORTMSG_REQ:
			taskProcess = new ShortMsgReqTask(emptySendToken, emptyRecvToken, obj, sc);
			break;
		case CodeDesc.SHORTMSG_SUCCESS:
			taskProcess = new ShortMsgSuccessTask(emptySendToken, emptyRecvToken, obj, sc);
			break;
		case CodeDesc.MAP_REQ:
			taskProcess = new MapReqTask(emptySendToken, emptyRecvToken, obj, sc);
			break;
		case CodeDesc.MAP:
			taskProcess = new MapTask(emptySendToken, emptyRecvToken, obj, sc);
			break;		
		case CodeDesc.NOTICE:
			taskProcess = new NoticeTask(emptySendToken, emptyRecvToken, obj, sc);
			break;
		case CodeDesc.CONNECT:
			taskProcess = new ConnectTask(emptySendToken, emptyRecvToken, obj, sc);
			break;
		case CodeDesc.CONNECT_OK:
			taskProcess = new ConnectOKTask(emptySendToken, emptyRecvToken, obj, sc);
			break;
		}
	}
	
	public TheCompanyDataToken getSendToken() {
		return taskProcess.getSendToken();
	}
	
	public TheCompanyDataToken getRecvToken() {
		return taskProcess.getRecvToken();
	}
	
	public Object getObjec() {
		return taskProcess.getObject();
	}
	
	public boolean isBroadcast() {
		return taskProcess.isBroadcast();
	}	
	
	public boolean isSendXML() {
		return taskProcess.isSendXML();
	}
}
