package ju.thecompany.server.controlserver.serverprocess.pool.selector.handler;


import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import ju.thecompany.data.Setting;
import ju.thecompany.server.controlserver.serverprocess.event.Job;
import ju.thecompany.server.controlserver.serverprocess.event.NIOEvent;
import ju.thecompany.server.controlserver.serverprocess.jobqueue.Queue;

// 서버소켓을 셀렉터에 등록, 그리고 접속이 있으면 해당 SocketChannel을 큐에 저장
public class AcceptHandler extends Thread {
	
	private Queue queue = null;
	private Selector selector = null;
	private int port = 9090;
	private String name = "AcceptHandler-";
	
	public AcceptHandler(Queue queue, Selector selector, int port, int index) {
		this.queue = queue;
		this.selector = selector;
		this.port = port;
		setName(name + index);
		init();
	}
	
	private void init() {
		try {
			Setting setting = Setting.getInstance();
			
			ServerSocketChannel ssc = ServerSocketChannel.open();
			ssc.configureBlocking(false);
			
			InetSocketAddress address = new InetSocketAddress(setting.getData("ControlServerIP"), port);
			ssc.socket().bind(address);
			
			System.out.println(getName() + " Bound to " + address);
			
			ssc.register(this.selector, SelectionKey.OP_ACCEPT);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void run() {
		try {
			while (!Thread.currentThread().isInterrupted()) {
				int keysReady = selector.select();
				acceptPendingConnections();
			}
		} catch (Exception e) {
		}
	}
	
	private void acceptPendingConnections() throws Exception {
		Iterator iter = selector.selectedKeys().iterator();	
		while (iter.hasNext()) {
			SelectionKey key = (SelectionKey) iter.next();
			iter.remove();
			
			ServerSocketChannel readyChannel = (ServerSocketChannel) key.channel();
			SocketChannel sc = readyChannel.accept();
			
			System.out.println(getName() + " connection accepted from " + sc.socket().getInetAddress());
			
			pushMyJob(sc);
		}
	}
	
	private void pushMyJob(SocketChannel sc) {
		Map session = new HashMap();
		session.put("SocketChannel", sc);
		Job job = new Job(NIOEvent.ACCEPT_EVENT, session);			
		queue.push(job);
	}

	public int READ_EVENT = 2;

}
