package ju.thecompany.server.controlserver.serverprocess.pool.thread.processor;


import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharsetEncoder;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;

import ju.thecompany.data.CodeDesc;
import ju.thecompany.data.DateParser;
import ju.thecompany.data.Setting;
import ju.thecompany.data.TCXMLParser;
import ju.thecompany.data.TheCompanyDataToken;
import ju.thecompany.server.controlserver.dbprocess.DBControl;
import ju.thecompany.server.controlserver.serverprocess.distributor.TaskDistributor;
import ju.thecompany.server.controlserver.serverprocess.event.Job;
import ju.thecompany.server.controlserver.serverprocess.event.NIOEvent;
import ju.thecompany.server.controlserver.serverprocess.jobqueue.Connections;
import ju.thecompany.server.controlserver.serverprocess.jobqueue.LoginMember;
import ju.thecompany.server.controlserver.serverprocess.jobqueue.LoginMemberData;
import ju.thecompany.server.controlserver.serverprocess.jobqueue.Queue;
import ju.thecompany.server.controlserver.serverprocess.pool.PoolManager;
import ju.thecompany.server.controlserver.serverprocess.pool.buffer.ByteBufferPoolIF;


// READ_EVENT로 등록 된 아이템을 꺼낸 후 키에서 sc를 꺼내서 읽어서
// TaskDistributor를 이용해서 작업 처리 후
// 다시 대상에서 메시지를 보냄
public class ReadWriteProcessor extends Thread {
	
	private Queue queue = null;
	
	public ReadWriteProcessor(Queue queue) {
		this.queue = queue;

	}
	
	public void run() {
		try {
			while (!Thread.currentThread().isInterrupted()) {
				Job job = queue.pop(NIOEvent.READ_EVENT);
				SelectionKey key = (SelectionKey) job.getSession().get("SelectionKey");
				SocketChannel sc = (SocketChannel) key.channel();
				
				try {
					process(sc);
				} catch (IOException e) {
					closeChannel(sc);
					//e.printStackTrace();
				}
			}
		} catch (Exception e) {
			
		}
	}
	
	private void process(SocketChannel sc) throws Exception {
		
		ByteBufferPoolIF bufferPool = PoolManager.getByteBufferPool();
		ByteBuffer buffer = null;
		
		Charset charset = Charset.forName("UTF-8");
		CharsetDecoder decoder = charset.newDecoder();
		
		try {
			buffer = bufferPool.getMemoryBuffer();
			
			String result = "";
			int byteRead = 0;

			for( int i =0 ; i < 2 ; i++) {
				byteRead += sc.read(buffer);
			}
			
			if (byteRead == 0) {
				return;
			}
			if (byteRead < 0) {
				closeChannel(sc);
				return;
			}
			
			buffer.flip();
			CharBuffer charBuffer = decoder.decode(buffer);
			result = charBuffer.toString();
	
			//System.err.println("recv : " +result);
			
			///////////////////////////////////////////////////////////////////////
			// 연산
			
			TaskDistributor td = new TaskDistributor(result, sc);
			TheCompanyDataToken sendToken = td.getSendToken();
			TheCompanyDataToken recvToken = td.getRecvToken();
			Object obj = td.getObjec();
			boolean isBroadcast = td.isBroadcast();
			boolean isSendXML = td.isSendXML();
			String sendXML = TCXMLParser.parseString(sendToken, recvToken, obj);
			
			//////////////////////////////////////////////////////////////////////
			
			//System.err.println("send : " +sendXML);
			
			ByteBuffer encodedBuffer = ByteBuffer.wrap(sendXML.getBytes("UTF-8"));
			
			if (isSendXML && isBroadcast) {
				Enumeration <String>e = LoginMember.getInstance().keys();
				
				if (e!=null) {
					while (e.hasMoreElements()) {
						int bt = 0;
						String key = e.nextElement();
						SocketChannel member = (SocketChannel)LoginMember.getInstance().getConnMember(key).get(0);
						if (member != null && member.isConnected()) {
							System.err.println(member.toString() +": id - " + key + " broadcast send");
							while (encodedBuffer.hasRemaining()) {
								bt += member.write(encodedBuffer);
							}
							encodedBuffer.rewind();
						}
						System.err.println("write size : " + bt +"byte");
					}
				}
			} else if (isSendXML) {
				
				int bt = 0;

				if (LoginMember.getInstance().containsKey(recvToken.id)) {
					sc = (SocketChannel)LoginMember.getInstance().getConnMember(recvToken.id).get(0);
					System.err.println(sc.toString() +": id - " + recvToken.id + " 1 send");
					while ( encodedBuffer.hasRemaining()) {
						bt += sc.write(encodedBuffer);
					}
				} else {
					System.err.println(sc.toString() +": id - noid" + " send close");
					while (encodedBuffer.hasRemaining()) {
						bt += sc.write(encodedBuffer);
					}
					closeChannel(sc);
				}
				System.err.println("write size : " + bt +"byte ");
			}
 
		} finally {
			bufferPool.putBuffer(buffer);
		}		
	}
	
	private void closeChannel(SocketChannel sc) {
		try {
			sc.close();
			Connections.getInstance().remove(sc);
			
			Enumeration <String>e = LoginMember.getInstance().keys();
			
			
			if (e!=null) {
				while (e.hasMoreElements()) {
					String key = e.nextElement();
					SocketChannel member = (SocketChannel)LoginMember.getInstance().getConnMember(key).get(0);
					if (member.equals(sc)) {
						LoginMember.getInstance().remove(key);
					}
				}
			}
			
			
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
