// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18
//
package ju.thecompany.web.client;

import java.util.Iterator;
import java.util.Vector;

import ju.thecompany.web.client.data.DeptInfo;
import ju.thecompany.web.client.data.Notice;
import ju.thecompany.web.client.data.PosInfo;
import ju.thecompany.web.client.data.UserInfo;
import ju.thecompany.web.client.rpc.service.DeptService;
import ju.thecompany.web.client.rpc.service.DeptServiceAsync;
import ju.thecompany.web.client.rpc.service.PosService;
import ju.thecompany.web.client.rpc.service.PosServiceAsync;
import ju.thecompany.web.client.rpc.service.UserService;
import ju.thecompany.web.client.rpc.service.UserServiceAsync;
import ju.thecompany.web.client.rpc.service.UtilService;
import ju.thecompany.web.client.rpc.service.UtilServiceAsync;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.DecoratedStackPanel;
import com.google.gwt.user.client.ui.DecoratorPanel;
import com.google.gwt.user.client.ui.DialogBox;
import com.google.gwt.user.client.ui.FileUpload;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.FormPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.PasswordTextBox;
import com.google.gwt.user.client.ui.TextArea;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.FormPanel.SubmitCompleteEvent;
import com.google.gwt.user.client.ui.FormPanel.SubmitEvent;

public class BottomMenuPanel extends DecoratorPanel{

	private static BottomMenuPanel singleton = new BottomMenuPanel();
	
	// DB 칼럼 
	private static final String SORT_COL_NAME = "name";
	
	private final DialogBox mUserDlg = new DialogBox();
	private final Image mUserPic = new Image();
	private final TextBox mIdTextBox = new TextBox();
	private final PasswordTextBox mPwTextBox = new PasswordTextBox();
	private final TextBox mNameTextBox = new TextBox();
	private final TextBox mTelTextBox = new TextBox();
	private final TextBox mHpTextBox = new TextBox();
	private final TextBox mEmailTextBox = new TextBox();
	private final TextBox mAddrTextBox = new TextBox();
	private final ListBox mDeptListBox = new ListBox();
	private final ListBox mPosListBox = new ListBox();
	private final Button mCloseButton = new Button();
	private final Button mInsertButton = new Button();
	private final Button mUpdateButton = new Button();
	private final Button mDeleteButton = new Button();
	
	private final DialogBox mDeptDlg = new DialogBox();
	private final ListBox mOriDeptListBox = new ListBox();
	private final HTML mOriDeptHTML = new HTML("원부서명");
	private final TextBox mDeptNameTextBox = new TextBox();
	private final HTML mDeptNameHTML = new HTML("새부서명");
	private final TextBox mDeptTelTextBox = new TextBox();
	private final HTML mDeptTelHTML = new HTML("내선");
	private final TextBox mDeptWheTextBox = new TextBox();	
	private final HTML mDeptWheHTML = new HTML("위치");
	private final Button mInsertDeptButton = new Button();
	private final Button mUpdateDeptButton = new Button();
	private final Button mDeleteDeptButton = new Button();

	private final DialogBox mPosDlg = new DialogBox();
	private final ListBox mOriPosListBox = new ListBox();
	private final HTML mOriPosHTML = new HTML("원직위");
	private final TextBox mPosNameTextBox = new TextBox();
	private final HTML mPosNameHTML = new HTML("새직위");
	private final ListBox mAuthListBox = new ListBox();
	private final HTML mAuthHTML = new HTML("권한");
	private final Button mInsertPosButton = new Button();
	private final Button mUpdatePosButton = new Button();
	private final Button mDeletePosButton = new Button();
	
	private final Button mPicButton = new Button();
	private final FlowPanel mDialogUserButtonWrapPanel = new FlowPanel();
	private final FlowPanel mDialogDeptButtonWrapPanel = new FlowPanel();
	private final FlowPanel mDialogPosButtonWrapPanel = new FlowPanel();
	
	private final HTML mPicHTML = new HTML("사진");
	private final FormPanel mUploadForm = new FormPanel();
    private FileUpload mFileUpload;
    
    private String mFilename = "";
    
    
    // 유저, 부서, 직위 서비스 선언
	private final UserServiceAsync mUserServiceAsync = (UserServiceAsync)GWT.create(UserService.class);
	private final DeptServiceAsync mDeptServiceAsync = (DeptServiceAsync)GWT.create(DeptService.class);
	private final PosServiceAsync mPosServiceAsync = (PosServiceAsync)GWT.create(PosService.class);
	
	private BottomMenuPanel() {
		initPanel();
	}
	
	public static BottomMenuPanel getInstance() {
		return singleton;
	}
	
	public void initPanel() {

		createUserDlg();
		createButton();
		createUserUpload();
		createDeptDlg();
		createPosDlg();
		refleshDeptList();
		refleshPosList();
		
		DecoratedStackPanel stackPanel = new DecoratedStackPanel();
		stackPanel.setSize("200px", "100%");

		String userHeader = "사용자";
		stackPanel.add(getUserItem(), userHeader, true);
		
		String deptHeader = "부서";
		stackPanel.add(getDeptItem(), deptHeader, true);

		String positionHeader = "직위";
		stackPanel.add(getPositionItem(), positionHeader, true);
		
		String utilHeader = "기능";
		stackPanel.add(getUtilItem(), utilHeader, true);

		setWidget(stackPanel);
		setSize("210px", "100%");
		setStyleName("bottomMenuPanel");
	}
	
	private VerticalPanel getUserItem() {
		VerticalPanel filtersPanel = new VerticalPanel();
		filtersPanel.setSpacing(4);
		
		Button [] buttonUser = new Button[3];
		buttonUser[0] = new Button("사용자추가");
		buttonUser[0].addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				mUserDlg.setText("사용자추가");

				initUserDlg();
				mFileUpload = new FileUpload();
				mFileUpload.setName("upload"); 
				mUploadForm.clear();
				mUploadForm.add(mFileUpload);
				
				mDialogUserButtonWrapPanel.add(mPicButton);
				mDialogUserButtonWrapPanel.add(mInsertButton);
				mDialogUserButtonWrapPanel.add(mCloseButton);

				mUserDlg.center();
			}
		});
		buttonUser[1] = new Button("사용자수정");
		buttonUser[1].addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				
				final UserInfo user = BottomTableContentsPanel.getInstance().getSelectedItem();
				if (user == null) return;
				
				
				
				mUserDlg.setText("사용자수정");

				initUserDlg();
				setDlgItem(user);
				
				mFilename = user.getPic();
				
				mFileUpload = new FileUpload();
				mFileUpload.setName("upload"); 
				mUploadForm.clear();
				mUploadForm.add(mFileUpload); 
				
				mDialogUserButtonWrapPanel.add(mPicButton);
				mDialogUserButtonWrapPanel.add(mUpdateButton);
				mDialogUserButtonWrapPanel.add(mCloseButton);
				
				if (!user.getPic().equals("")) mUserPic.setUrl("/pictc/" +user.getPic());

				mIdTextBox.setEnabled(false);
				mNameTextBox.setEnabled(false);
				
				mUserDlg.center();
			}
		});
		buttonUser[2] = new Button("사용자삭제");
		buttonUser[2].addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				
				final UserInfo user = BottomTableContentsPanel.getInstance().getSelectedItem();
				if (user == null) return;
				
				mUserDlg.setText("사용자삭제");

				setDlgItem(user);
				
				mDialogUserButtonWrapPanel.clear();
				mDialogUserButtonWrapPanel.add(mDeleteButton);
				mDialogUserButtonWrapPanel.add(mCloseButton);
				
				mPicHTML.setVisible(false);
				mUploadForm.setVisible(false);
				
				if (!user.getPic().equals("")) mUserPic.setUrl("/pictc/" +user.getPic());
				else mUserPic.setUrl("_nopic.gif");
				
				mIdTextBox.setEnabled(false);
				mPwTextBox.setEnabled(false);
				mNameTextBox.setEnabled(false);
				mTelTextBox.setEnabled(false);
				mHpTextBox.setEnabled(false);
				mEmailTextBox.setEnabled(false);
				mAddrTextBox.setEnabled(false);
				mDeptListBox.setEnabled(false);
				mPosListBox.setEnabled(false);


				mUserDlg.center();
			}
		});

		for (Button buttonAdd: buttonUser) {
			buttonAdd.setStyleName("bottomMenuButton");
			filtersPanel.add(buttonAdd);
		}

		return filtersPanel;
	}
	
	private VerticalPanel getDeptItem() {
		VerticalPanel filtersPanel = new VerticalPanel();
		filtersPanel.setSpacing(4);

		Button [] buttonDept = new Button[3];
		buttonDept[0] = new Button("부서추가");
		buttonDept[0].addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {

				mDeptDlg.setText("부서추가");

				initDeptDlg();

				mDialogDeptButtonWrapPanel.add(mInsertDeptButton);
				mDialogDeptButtonWrapPanel.add(mCloseButton);
				
				mOriDeptHTML.setVisible(false);
				mOriDeptListBox.setVisible(false);
				
				mDeptDlg.center();
			}
		});
		buttonDept[1] = new Button("부서수정");
		buttonDept[1].addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {

				mDeptDlg.setText("부서수정");

				initDeptDlg();

				mDialogDeptButtonWrapPanel.add(mUpdateDeptButton);
				mDialogDeptButtonWrapPanel.add(mCloseButton);

				mDeptDlg.center();
			}
		});
		buttonDept[2] = new Button("부서삭제");
		buttonDept[2].addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {

				mDeptDlg.setText("부서삭제");

				initDeptDlg();

				mDialogDeptButtonWrapPanel.add(mDeleteDeptButton);
				mDialogDeptButtonWrapPanel.add(mCloseButton);
				
				mDeptNameHTML.setVisible(false);
				mDeptNameTextBox.setVisible(false);
				mDeptTelHTML.setVisible(false);
				mDeptTelTextBox.setVisible(false);
				mDeptWheHTML.setVisible(false);
				mDeptWheTextBox.setVisible(false);

				mDeptDlg.center();
			}
		});

		for (Button buttonAdd: buttonDept) {
			buttonAdd.setStyleName("bottomMenuButton");
			filtersPanel.add(buttonAdd);
		}

		return filtersPanel;
	}
	
	private VerticalPanel getPositionItem() {
		VerticalPanel filtersPanel = new VerticalPanel();
		filtersPanel.setSpacing(4);
		
		Button [] buttonPos = new Button[3];
		buttonPos[0] = new Button("직위추가");
		buttonPos[0].addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {

				mPosDlg.setText("직위추가");

				initPosDlg();

				mDialogPosButtonWrapPanel.add(mInsertPosButton);
				mDialogPosButtonWrapPanel.add(mCloseButton);
				
				mOriPosHTML.setVisible(false);
				mOriPosListBox.setVisible(false);
				
				mPosDlg.center();
			}
		});
		buttonPos[1] = new Button("직위수정");
		buttonPos[1].addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {

				mPosDlg.setText("직위수정");

				initPosDlg();

				mDialogPosButtonWrapPanel.add(mUpdatePosButton);
				mDialogPosButtonWrapPanel.add(mCloseButton);

				mPosDlg.center();
			}
		});
		buttonPos[2] = new Button("직위삭제");
		buttonPos[2].addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {

				mPosDlg.setText("직위삭제");

				initPosDlg();

				mDialogPosButtonWrapPanel.add(mDeletePosButton);
				mDialogPosButtonWrapPanel.add(mCloseButton);
				
				mPosNameTextBox.setVisible(false);
				mPosNameHTML.setVisible(false);
				mAuthListBox.setVisible(false);
				mAuthHTML.setVisible(false);

				mPosDlg.center();
			}
		});
		for (Button buttonAdd: buttonPos) {
			buttonAdd.setStyleName("bottomMenuButton");
			filtersPanel.add(buttonAdd);
		}
		
		return filtersPanel;
	}
	
	private VerticalPanel getUtilItem() {
		VerticalPanel filtersPanel = new VerticalPanel();
		filtersPanel.setSpacing(4);
		
		Button [] buttonUtil = new Button[5];
		buttonUtil[0] = new Button("일정보기");
		buttonUtil[0].addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				Window.open("http://www.google.com/calendar/", "_blink", "disabled");
			}
		});
		buttonUtil[1] = new Button("공지보내기");
		buttonUtil[1].addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				final DialogBox dlg = new DialogBox();
				dlg.setText("공지보내기");
				dlg.setAnimationEnabled(true);
				
				VerticalPanel parentPanel = new VerticalPanel();
				HorizontalPanel bottomPanel = new HorizontalPanel();
				FlowPanel buttonHolder = new FlowPanel();
				
				final TextBox subject = new TextBox();
				final TextArea contents = new TextArea();
				final Button sendButton = new Button("보내기");
				final Button closeButton = new Button("닫기");
				
				subject.setWidth("210px");
				contents.setWidth("250px");
				contents.setVisibleLines(10);
				
				FlexTable flexTable = new FlexTable();
				flexTable.setBorderWidth(0);
				flexTable.setWidget(0, 0, new HTML("제목"));
				flexTable.setWidget(0, 1, subject);
				
				buttonHolder.add(sendButton);
				buttonHolder.add(closeButton);
				bottomPanel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_RIGHT);
				bottomPanel.setWidth("250px");
				bottomPanel.add(buttonHolder);				
				
				parentPanel.add(flexTable);
				parentPanel.add(contents);
				parentPanel.add(bottomPanel);
				
				dlg.add(parentPanel);
				
				sendButton.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						
						if (subject.getText().trim().equals("")) {
							Window.alert("제목을 입력하세요.");
						} else if (contents.getText().trim().equals("")) {
							Window.alert("내용을 입력하세요.");
						} else {
							Notice notice = new Notice();
							notice.setSendID(TheCompanyWeb.myUserInfo.getID());
							notice.setTitle(subject.getText());
							notice.setText(contents.getText());
							System.out.println(notice.getSendID() + " : " + notice.getTitle() + " : " + notice.getText());
							
							// 공지 보내기 RPC
							final UtilServiceAsync utilServiceAsync = (UtilServiceAsync)GWT.create(UtilService.class);
							
					        ServiceDefTarget serviceDefTarget = (ServiceDefTarget)utilServiceAsync;
					        serviceDefTarget.setServiceEntryPoint("util");
					        
					        AsyncCallback<Boolean> asyncCallback = new AsyncCallback<Boolean>() {
					    		public void onSuccess(Boolean result) {
					    			if(result){
					    				BottomTableContentsPanel.getInstance().sortItem(SORT_COL_NAME);
					    				dlg.hide();
					    			}else{
					    				Window.alert("수정할 자료에 문제점이 있습니다.");
					    			}
					    			
					    		}
					            public void onFailure(Throwable caught) {
					                System.out.println(caught);
					                dlg.hide();
					            }
					        };
					        utilServiceAsync.sendNotice(notice, asyncCallback);
					        //
						}
					}
				});
				
				
				closeButton.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						dlg.hide();
					}
				});
				
				dlg.center();
			}
		});
		
		buttonUtil[2] = new Button("쪽지보내기");
		buttonUtil[2].addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				Window.alert("다음 버전에 공개 예정입니다.");
			}
		});
		buttonUtil[3] = new Button("대화하기");
		buttonUtil[3].addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				Window.alert("다음 버전에 공개 예정입니다.");
			}
		});
		buttonUtil[4] = new Button("위치확인");
		buttonUtil[4].addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				Window.alert("다음 버전에 공개 예정입니다.");
			}
		});

		for (Button buttonAdd: buttonUtil) {
			buttonAdd.setStyleName("bottomMenuButton");
			filtersPanel.add(buttonAdd);
		}
		
		return filtersPanel;
	}
	
	private void createUserDlg() {
		mUserDlg.setAnimationEnabled(true);
		
		VerticalPanel parentContainer = new VerticalPanel();
		HorizontalPanel userPopupContainer = new HorizontalPanel();
		VerticalPanel userpicPopupContainer = new VerticalPanel();
	    VerticalPanel userinfoPopupContainer = new VerticalPanel();
	    
	   
	    userpicPopupContainer.add(mUserPic);
	    mUserPic.setSize("115px", "150px");
	    
		mAddrTextBox.setWidth("260px");
	    
	    FlexTable flexTable = new FlexTable();
	    flexTable.setBorderWidth(0);
	    
	    mUploadForm.add(mFileUpload);

	    flexTable.setText(0, 0, "아이디");
	    flexTable.setWidget(0, 1, mIdTextBox);
	    flexTable.setText(1, 0, "비밀번호");
	    flexTable.setWidget(1, 1, mPwTextBox);
	    flexTable.setText(2, 0, "이름");
	    flexTable.setWidget(2, 1, mNameTextBox);
	    flexTable.setText(3, 0, "연락처");
	    flexTable.setWidget(3, 1, mTelTextBox);
	    flexTable.setText(4, 0, "휴대폰");
	    flexTable.setWidget(4, 1, mHpTextBox);
	    flexTable.setText(5, 0, "이메일");
	    flexTable.setWidget(5, 1, mEmailTextBox);
	    flexTable.setText(6, 0, "주소");
	    flexTable.setWidget(6, 1, mAddrTextBox);
	    flexTable.setText(7, 0, "부서");
	    flexTable.setWidget(7, 1, mDeptListBox);
	    flexTable.setText(8, 0, "직위");
	    flexTable.setWidget(8, 1, mPosListBox);
	    flexTable.setWidget(9, 0, mPicHTML);
	    flexTable.setWidget(9, 1, mUploadForm);

	    userinfoPopupContainer.setSpacing(5);
	    userinfoPopupContainer.add(flexTable);
	    
	    userPopupContainer.add(userpicPopupContainer);
	    userPopupContainer.add(userinfoPopupContainer);

		HorizontalPanel dialogButtonPanel = new HorizontalPanel();
		dialogButtonPanel.setWidth("380px");
		dialogButtonPanel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_RIGHT);
		dialogButtonPanel.add(mDialogUserButtonWrapPanel);
	    
	    parentContainer.add(userPopupContainer);
	    parentContainer.add(dialogButtonPanel);

	    mUserDlg.setWidget(parentContainer);
	}
	
	private void createDeptDlg() {
		mDeptDlg.setAnimationEnabled(true);
		
		VerticalPanel parentContainer = new VerticalPanel();

	    FlexTable flexTable = new FlexTable();
	    flexTable.setBorderWidth(0);

	    flexTable.setWidget(0, 0, mOriDeptHTML);
	    flexTable.setWidget(0, 1, mOriDeptListBox);
	    flexTable.setWidget(1, 0, mDeptNameHTML);
	    flexTable.setWidget(1, 1, mDeptNameTextBox);
	    flexTable.setWidget(2, 0, mDeptTelHTML);
	    flexTable.setWidget(2, 1, mDeptTelTextBox);
	    flexTable.setWidget(3, 0, mDeptWheHTML);
	    flexTable.setWidget(3, 1, mDeptWheTextBox);

		HorizontalPanel dialogButtonPanel = new HorizontalPanel();
		dialogButtonPanel.setWidth("220px");
		dialogButtonPanel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_RIGHT);
		dialogButtonPanel.add(mDialogDeptButtonWrapPanel);

		parentContainer.add(flexTable);
	    parentContainer.add(dialogButtonPanel);

	    mDeptDlg.setWidget(parentContainer);
	}
	
	private void createPosDlg() {
		mPosDlg.setAnimationEnabled(true);
		
		VerticalPanel parentContainer = new VerticalPanel();

	    FlexTable flexTable = new FlexTable();
	    flexTable.setBorderWidth(0);

	    flexTable.setWidget(0, 0, mOriPosHTML);
	    flexTable.setWidget(0, 1, mOriPosListBox);
	    flexTable.setWidget(1, 0, mPosNameHTML);
	    flexTable.setWidget(1, 1, mPosNameTextBox);
	    flexTable.setWidget(2, 0, mAuthHTML);
	    flexTable.setWidget(2, 1, mAuthListBox);
	    
	    mAuthListBox.addItem("웹접속가능", "1");
	    mAuthListBox.addItem("웹접속불가", "0");

		HorizontalPanel dialogButtonPanel = new HorizontalPanel();
		dialogButtonPanel.setWidth("220px");
		dialogButtonPanel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_RIGHT);
		dialogButtonPanel.add(mDialogPosButtonWrapPanel);

		parentContainer.add(flexTable);
	    parentContainer.add(dialogButtonPanel);

	    mPosDlg.setWidget(parentContainer);
	}

	// 부서를 리플래쉬
	private void refleshDeptList() {
        ServiceDefTarget serviceDefTarget = (ServiceDefTarget)mDeptServiceAsync;
        serviceDefTarget.setServiceEntryPoint("dept");
        
        AsyncCallback<Vector <DeptInfo>> asyncCallback = new AsyncCallback<Vector <DeptInfo>>() {
    		public void onSuccess(Vector <DeptInfo> result) {
    			if(result != null){

    				Iterator <DeptInfo>i = result.iterator();
    				
    				mDeptListBox.clear();
    				mOriDeptListBox.clear();
    				while(i.hasNext()) {
    					String name = i.next().getDeptName();
    					mDeptListBox.addItem(name);
    					mOriDeptListBox.addItem(name);
    				}

    			}else{
                    Window.alert("쿼리이상!");
    			}
        	}
    			
            public void onFailure(Throwable caught) {
                System.out.println(caught);
            }

        };
		mDeptServiceAsync.getDeptList(asyncCallback);
	}

	// 직책을 리플래쉬
	private void refleshPosList() {
        ServiceDefTarget serviceDefTarget = (ServiceDefTarget)mPosServiceAsync;
        serviceDefTarget.setServiceEntryPoint("pos");
        
        AsyncCallback<Vector <PosInfo>> asyncCallback = new AsyncCallback<Vector <PosInfo>>() {
    		public void onSuccess(Vector <PosInfo> result) {
    			if(result != null){

    				Iterator <PosInfo>i = result.iterator();
    				
    				mPosListBox.clear();
    				mOriPosListBox.clear();
    				while(i.hasNext()) {
    					String name = i.next().getPosName();
    					mPosListBox.addItem(name);
    					mOriPosListBox.addItem(name);
    				}
    			}else{
                    Window.alert("쿼리이상!");
    			}
        	}
    			
            public void onFailure(Throwable caught) {
                System.out.println(caught);
            }

        };
		mPosServiceAsync.getPosList(asyncCallback);
	}
	
	private boolean isEmptyUserDlg() {
		if (mIdTextBox.getText().equals("")) {
			Window.alert("ID를 입력하세요");
			return false;
		}
		if (mPwTextBox.getText().equals("")) {
			Window.alert("Passwd를 입력하세요");
			return false;
		}
		if (mNameTextBox.getText().equals("")) {
			Window.alert("이름을 입력하세요");
			return false;
		}
		if (mTelTextBox.getText().equals("")) {
			Window.alert("연락처를 입력하세요");
			return false;
		}
		if (mHpTextBox.getText().equals("")) {
			Window.alert("휴대폰을 입력하세요");
			return false;
		}
		if (mEmailTextBox.getText().equals("")) {
			Window.alert("이메일을 입력하세요");
			return false;
		}
		if (mAddrTextBox.getText().equals("")) {
			Window.alert("주소를 입력하세요");
			return false;
		}
		return true;
	}
	
	private boolean isEmptyDeptDlg() {
		if (mDeptNameTextBox.getText().equals("")) {
			Window.alert("부서명을 입력하세요");
			return false;
		}
		if (mDeptTelTextBox.getText().equals("")) {
			Window.alert("부서 내선번호를 입력하세요");
			return false;
		}
		if (mDeptWheTextBox.getText().equals("")) {
			Window.alert("부서 위치를 입력하세요");
			return false;
		}

		return true;
	}
	
	private boolean isEmptyPosDlg() {
		if (mPosNameTextBox.getText().equals("")) {
			Window.alert("직책명을 입력하세요");
			return false;
		}

		return true;
	}
	
	private void initUserDlg() {

		mFilename = "";
		
		mIdTextBox.setEnabled(true);
		mPwTextBox.setEnabled(true);
		mNameTextBox.setEnabled(true);
		mTelTextBox.setEnabled(true);
		mHpTextBox.setEnabled(true);
		mEmailTextBox.setEnabled(true);
		mAddrTextBox.setEnabled(true);
		mDeptListBox.setEnabled(true);
		mPosListBox.setEnabled(true);
	    mPicHTML.setVisible(true);
	    mUploadForm.setVisible(true);
		
		mIdTextBox.setText("");
		mPwTextBox.setText("");
		mNameTextBox.setText("");
		mTelTextBox.setText("");
		mHpTextBox.setText("");
		mEmailTextBox.setText("");
		mAddrTextBox.setText("");
		mDeptListBox.setSelectedIndex(0);
		mPosListBox.setSelectedIndex(0);
		mUserPic.setUrl("_nopic.gif");
		
		mDialogUserButtonWrapPanel.clear();
	}
	
	private void initDeptDlg() {

		mOriDeptListBox.setVisible(true);
		mOriDeptHTML.setVisible(true);
		mDeptNameTextBox.setVisible(true);
		mDeptNameHTML.setVisible(true);
		mDeptTelTextBox.setVisible(true);
		mDeptTelHTML.setVisible(true);
		mDeptWheTextBox.setVisible(true);
		mDeptWheHTML.setVisible(true);
		
		mDeptNameTextBox.setText("");
		mDeptTelTextBox.setText("");
		mDeptWheTextBox.setText("");

		mOriDeptListBox.setSelectedIndex(0);
		
		mDialogDeptButtonWrapPanel.clear();
	}
	
	private void initPosDlg() {

		mOriPosListBox.setVisible(true);
		mOriPosHTML.setVisible(true);
		mPosNameTextBox.setVisible(true);
		mPosNameHTML.setVisible(true);
		mAuthListBox.setVisible(true);
		mAuthHTML.setVisible(true);

		mPosNameTextBox.setText("");
		
		mOriPosListBox.setSelectedIndex(0);
		mAuthListBox.setSelectedIndex(0);
		
		mDialogPosButtonWrapPanel.clear();
	}
	
	private void setDlgItem(UserInfo user){
		mIdTextBox.setText(user.getID());
		mPwTextBox.setText(user.getPW());
		mNameTextBox.setText(user.getName());
		mTelTextBox.setText(user.getTel());
		mHpTextBox.setText(user.getHP());
		mEmailTextBox.setText(user.getEmail());
		mAddrTextBox.setText(user.getAddr());
		
		int i;
		for (i = 0 ; i < mDeptListBox.getItemCount(); i++) {
			if (mDeptListBox.getItemText(i).equals(user.getDept())) {
				mDeptListBox.setSelectedIndex(i);
				break;
			}
		}
		
		
		
		for (i = 0 ; i < mPosListBox.getItemCount(); i++) {
			if (mPosListBox.getItemText(i).equals(user.getPos())){
				mPosListBox.setSelectedIndex(i);
				break;
			}
		}
	}
	
	// 사진 업로드를 위해 Form action 설정함.
	private void createUserUpload() {
		mUploadForm.setEncoding(FormPanel.ENCODING_MULTIPART);        
		mUploadForm.setMethod(FormPanel.METHOD_POST);    
		String path = GWT.getModuleBaseURL();
		mUploadForm.setAction(path+"fileupload");

	    mPicButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				GWT.log("You selected: " + mFileUpload.getFilename(), null); 
				mUploadForm.submit(); 
			}
		});
	         
		mUploadForm.addSubmitHandler(new FormPanel.SubmitHandler() {                
			public void onSubmit(SubmitEvent event) {
				String filename = mFileUpload.getFilename();
				if (!"".equalsIgnoreCase(filename)) {   
					}                        
				else{                                
					event.cancel(); // cancel the event                        
					}                
				}        
			});    
		
		mUploadForm.addSubmitCompleteHandler(new  FormPanel.SubmitCompleteHandler() {                
			public void onSubmitComplete(SubmitCompleteEvent event) {                        
				mUserPic.setUrl("pictc/" + event.getResults());  
				mFilename = event.getResults();
				}        
			});    

	}
	
	private void createButton() {
	    mPicButton.setWidth("90px");
	    mPicButton.setText("사진업로드");
	    mInsertButton.setWidth("90px");
	    mInsertButton.setText("추가");
	    mUpdateButton.setWidth("90px");
	    mUpdateButton.setText("수정");
	    mDeleteButton.setWidth("90px");
	    mDeleteButton.setText("삭제");
		mCloseButton.setWidth("90px");
		mCloseButton.setText("닫기");
		
	    mCloseButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				mUserDlg.hide();
				mDeptDlg.hide();
				mPosDlg.hide();
			}
		});
		

	    
	    mInsertButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				
				if (!isEmptyUserDlg()) return;
				
				UserInfo user = new UserInfo();
				user.setID(mIdTextBox.getText());
				user.setPW(mPwTextBox.getText());
				user.setName(mNameTextBox.getText());
				user.setTel(mTelTextBox.getText());
				user.setHP(mHpTextBox.getText());
				user.setEmail(mEmailTextBox.getText());
				user.setAddr(mAddrTextBox.getText());
				user.setDept(mDeptListBox.getValue(mDeptListBox.getSelectedIndex()));
				user.setPos(mPosListBox.getValue(mPosListBox.getSelectedIndex()));
				user.setPic(mFilename);
				
				// 유저 추가
		        ServiceDefTarget serviceDefTarget = (ServiceDefTarget)mUserServiceAsync;
		        serviceDefTarget.setServiceEntryPoint("user");
		        
		        AsyncCallback<Boolean> asyncCallback = new AsyncCallback<Boolean>() {
		    		public void onSuccess(Boolean result) {
		    			if(result){
		    				BottomTableContentsPanel.getInstance().sortItem(SORT_COL_NAME);
		    				mUserDlg.hide();
		    			}else{
		    				Window.alert("등록할 자료에 문제점이 있습니다.");
		    			}
		    			
		    		}
		            public void onFailure(Throwable caught) {
		                System.out.println(caught);
		                mUserDlg.hide();
		            }
		        };
				mUserServiceAsync.insertUser(user, asyncCallback);
			}
			
		});
	    mUpdateButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {

				if (!isEmptyUserDlg()) return;
				
				UserInfo user = new UserInfo();
				user.setID(mIdTextBox.getText());
				user.setPW(mPwTextBox.getText());
				user.setName(mNameTextBox.getText());
				user.setTel(mTelTextBox.getText());
				user.setHP(mHpTextBox.getText());
				user.setEmail(mEmailTextBox.getText());
				user.setAddr(mAddrTextBox.getText());
				user.setDept(mDeptListBox.getValue(mDeptListBox.getSelectedIndex()));
				user.setPos(mPosListBox.getValue(mPosListBox.getSelectedIndex()));
				user.setPic(mFilename);

				// 유저 수정
		        ServiceDefTarget serviceDefTarget = (ServiceDefTarget)mUserServiceAsync;
		        serviceDefTarget.setServiceEntryPoint("user");
		        
		        AsyncCallback<Boolean> asyncCallback = new AsyncCallback<Boolean>() {
		    		public void onSuccess(Boolean result) {
		    			if(result){
		    				BottomTableContentsPanel.getInstance().sortItem(SORT_COL_NAME);
		    				mUserDlg.hide();
		    			}else{
		    				Window.alert("수정할 자료에 문제점이 있습니다.");
		    			}
		    			
		    		}
		            public void onFailure(Throwable caught) {
		                System.out.println(caught);
		                mUserDlg.hide();
		            }
		        };
				mUserServiceAsync.updateUser(user, asyncCallback);
			}
			
		});
	    
	    mDeleteButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {

				String id = mIdTextBox.getText();
				
				// 유저 삭제
		        ServiceDefTarget serviceDefTarget = (ServiceDefTarget)mUserServiceAsync;
		        serviceDefTarget.setServiceEntryPoint("user");
		        
		        AsyncCallback<Boolean> asyncCallback = new AsyncCallback<Boolean>() {
		    		public void onSuccess(Boolean result) {
		    			if(result){
		    				BottomTableContentsPanel.getInstance().sortItem(SORT_COL_NAME);
		    				mUserDlg.hide();
		    			}else{
		                    Window.alert("삭제할 자료에 문제점이 있습니다.");
		    			}
		    		}
		            public void onFailure(Throwable caught) {
		                System.out.println(caught);
		                mUserDlg.hide();
		            }
		        };
				mUserServiceAsync.deleteUser(id, asyncCallback);
			}
			
		});
	    
	    mInsertDeptButton.setWidth("90px");
	    mInsertDeptButton.setText("추가");
	    mUpdateDeptButton.setWidth("90px");
	    mUpdateDeptButton.setText("수정");
	    mDeleteDeptButton.setWidth("90px");
	    mDeleteDeptButton.setText("삭제");
	    
		mInsertDeptButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				
				if (!isEmptyDeptDlg()) return;
				
				DeptInfo dept = new DeptInfo();
				dept.setDeptName(mDeptNameTextBox.getText());
				dept.setTel(mDeptTelTextBox.getText());
				dept.setWhere(mDeptWheTextBox.getText());
				
				// 부서 추가
		        ServiceDefTarget serviceDefTarget = (ServiceDefTarget)mDeptServiceAsync;
		        serviceDefTarget.setServiceEntryPoint("dept");
		        
		        AsyncCallback<Boolean> asyncCallback = new AsyncCallback<Boolean>() {
		    		public void onSuccess(Boolean result) {
		    			if(result){
		    				refleshDeptList();
		    				mDeptDlg.hide();
		    			}else{
		    				Window.alert("등록할 자료에 문제점이 있습니다.");
		    			}
		    			
		    		}
		            public void onFailure(Throwable caught) {
		                System.out.println(caught);
		                mDeptDlg.hide();
		            }
		        };
		        mDeptServiceAsync.insertDept(dept, asyncCallback);
			}
			
		});
		mUpdateDeptButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				
				if (!isEmptyDeptDlg()) return;
				
				DeptInfo dept = new DeptInfo();
				dept.setDeptName(mDeptNameTextBox.getText());
				dept.setTel(mDeptTelTextBox.getText());
				dept.setWhere(mDeptWheTextBox.getText());
				
				DeptInfo oridept = new DeptInfo();
				oridept.setDeptName(mOriDeptListBox.getValue(mOriDeptListBox.getSelectedIndex()));
				
				// 부서 수정
		        ServiceDefTarget serviceDefTarget = (ServiceDefTarget)mDeptServiceAsync;
		        serviceDefTarget.setServiceEntryPoint("dept");
		        
		        AsyncCallback<Boolean> asyncCallback = new AsyncCallback<Boolean>() {
		    		public void onSuccess(Boolean result) {
		    			if(result){
		    				BottomTableContentsPanel.getInstance().sortItem(SORT_COL_NAME);
		    				refleshDeptList();
		    				mDeptDlg.hide();
		    			}else{
		    				Window.alert("수정할 자료에 문제점이 있습니다.");
		    			}
		    			
		    		}
		            public void onFailure(Throwable caught) {
		                System.out.println(caught);
		                mDeptDlg.hide();
		            }
		        };
		        mDeptServiceAsync.updateDept(dept, oridept, asyncCallback);
			}
			
		});
		mDeleteDeptButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
			
				DeptInfo dept = new DeptInfo();
				dept.setDeptName(mOriDeptListBox.getValue(mOriDeptListBox.getSelectedIndex()));
				
				// 부서 삭제
		        ServiceDefTarget serviceDefTarget = (ServiceDefTarget)mDeptServiceAsync;
		        serviceDefTarget.setServiceEntryPoint("dept");
		        
		        AsyncCallback<Boolean> asyncCallback = new AsyncCallback<Boolean>() {
		    		public void onSuccess(Boolean result) {
		    			if(result){
		    				refleshDeptList();
		    				mDeptDlg.hide();
		    			}else{
		    				Window.alert("삭제할 자료에 문제점이 있습니다.");
		    			}
		    			
		    		}
		            public void onFailure(Throwable caught) {
		                System.out.println(caught);
		                mDeptDlg.hide();
		            }
		        };
		        mDeptServiceAsync.deleteDept(dept, asyncCallback);
			}
			
		});
		
	    mInsertPosButton.setWidth("90px");
	    mInsertPosButton.setText("추가");
	    mUpdatePosButton.setWidth("90px");
	    mUpdatePosButton.setText("수정");
	    mDeletePosButton.setWidth("90px");
	    mDeletePosButton.setText("삭제");
		
		mInsertPosButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				
				if (!isEmptyPosDlg()) return;
				
				PosInfo pos = new PosInfo();
				pos.setPosName(mPosNameTextBox.getText());
				pos.setAuth(mAuthListBox.getValue(mAuthListBox.getSelectedIndex()));

				// 직책 추가
		        ServiceDefTarget serviceDefTarget = (ServiceDefTarget)mPosServiceAsync;
		        serviceDefTarget.setServiceEntryPoint("pos");
		        
		        AsyncCallback<Boolean> asyncCallback = new AsyncCallback<Boolean>() {
		    		public void onSuccess(Boolean result) {
		    			if(result){
		    				refleshPosList();
		    				mPosDlg.hide();
		    			}else{
		    				Window.alert("등록할 자료에 문제점이 있습니다.");
		    			}
		    			
		    		}
		            public void onFailure(Throwable caught) {
		                System.out.println(caught);
		                mPosDlg.hide();
		            }
		        };
		        mPosServiceAsync.insertPos(pos, asyncCallback);
			}
			
		});
		mUpdatePosButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				
				if (!isEmptyPosDlg()) return;
				
				PosInfo pos = new PosInfo();
				pos.setPosName(mPosNameTextBox.getText());
				pos.setAuth(mAuthListBox.getValue(mAuthListBox.getSelectedIndex()));
				
				PosInfo oripos = new PosInfo();
				oripos.setPosName(mOriPosListBox.getValue(mOriPosListBox.getSelectedIndex()));
				
				// 직책 수정
		        ServiceDefTarget serviceDefTarget = (ServiceDefTarget)mPosServiceAsync;
		        serviceDefTarget.setServiceEntryPoint("pos");
		        
		        AsyncCallback<Boolean> asyncCallback = new AsyncCallback<Boolean>() {
		    		public void onSuccess(Boolean result) {
		    			if(result){
		    				refleshPosList();
		    				BottomTableContentsPanel.getInstance().sortItem(SORT_COL_NAME);
		    				mPosDlg.hide();
		    			}else{
		    				Window.alert("수정할 자료에 문제점이 있습니다.");
		    			}
		    			
		    		}
		            public void onFailure(Throwable caught) {
		                System.out.println(caught);
		                mPosDlg.hide();
		            }
		        };
		        mPosServiceAsync.updatePos(pos, oripos, asyncCallback);
			}
			
		});
		
		mDeletePosButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
			
				PosInfo pos = new PosInfo();
				pos.setPosName(mOriPosListBox.getValue(mOriPosListBox.getSelectedIndex()));
				
				// 직책 삭제
		        ServiceDefTarget serviceDefTarget = (ServiceDefTarget)mPosServiceAsync;
		        serviceDefTarget.setServiceEntryPoint("pos");
		        
		        AsyncCallback<Boolean> asyncCallback = new AsyncCallback<Boolean>() {
		    		public void onSuccess(Boolean result) {
		    			if(result){
		    				refleshPosList();
		    				mPosDlg.hide();
		    			}else{
		    				Window.alert("삭제할 자료에 문제점이 있습니다.");
		    			}
		    			
		    		}
		            public void onFailure(Throwable caught) {
		                System.out.println(caught);
		                mPosDlg.hide();
		            }
		        };
		        mPosServiceAsync.deletePos(pos, asyncCallback);
			}
		});
	}
}
