// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18
//

package ju.thecompany.web.client;

import java.util.Iterator;
import java.util.Vector;

import ju.thecompany.web.client.data.UserInfo;
import ju.thecompany.web.client.rpc.service.UserService;
import ju.thecompany.web.client.rpc.service.UserServiceAsync;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

public class BottomTableContentsPanel extends FlexTable{

	private static BottomTableContentsPanel singleton = new BottomTableContentsPanel();

	// DB 칼럼 
	private static final String SORT_COL_NAME = "name";

	// 선택 Row
	private int mSelectedRow = -1;
	
	private Vector <UserInfo> mUserList;
	
	// 유저info popup
	private final PopupPanel mUserInfoPopup = new PopupPanel(true, false);
	private final Image mUserPic = new Image();
	private final HTML mUserName = new HTML();
	private final HTML mUserID = new HTML();
	private final HTML mUserDept = new HTML();
	private final HTML mUserDeptTel = new HTML();
	private final HTML mUserPos = new HTML();
	private final HTML mUserPosAuth = new HTML();
	private final HTML mUserTel = new HTML();
	private final HTML mUserHP = new HTML();
	private final HTML mUserEmail = new HTML();
	private final HTML mUserAddr = new HTML();
	private final HTML mUserEnter = new HTML();
	private final HTML mLogout = new HTML();
	
	
	private BottomTableContentsPanel() {
		initPanel();
	}
	
	public static BottomTableContentsPanel getInstance() {
		return singleton;
	}
	
	public void initPanel() {
		setWidth("778px");
	    setCellSpacing(0);
	    setCellPadding(0);
	    getCellFormatter().setWidth(0, 0, "150px");
	    getCellFormatter().setWidth(0, 1, "250px");
	    getCellFormatter().setWidth(0, 2, "100px");
	    getCellFormatter().setWidth(0, 3, "100px");
	    getCellFormatter().setWidth(0, 4, "178px");
	    
	    sortItem(SORT_COL_NAME);

        addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				Cell cell = getCellForEvent(event);
				if (cell != null) {
					
					int row = cell.getRowIndex();
					
					if (row > -1) {
						selectRow(row-1);
					}
				}
			}
        });
				
        createDialog();
	}
	
	public void sortItem(String sort) {
		// 아이템을 sort에 맞춰서 정렬
		UserServiceAsync userServiceAsync = (UserServiceAsync)GWT.create(UserService.class);
        ServiceDefTarget serviceDefTarget = (ServiceDefTarget)userServiceAsync;
        serviceDefTarget.setServiceEntryPoint("user");
        
        AsyncCallback<Vector<UserInfo>> asyncCallback = new AsyncCallback<Vector<UserInfo>>() {
    		public void onSuccess(Vector<UserInfo> result) {
    			if(result != null){

    				mUserList = result;
    			    Iterator <UserInfo >iter = result.iterator();
    			    
    			    int i = 0;
    			    while (iter.hasNext()) {
    			    	UserInfo ui = iter.next();
    			    	setWidget(i, 0, new HTML(ui.getName()));
    			    	setWidget(i, 1, new HTML(ui.getID()));
    			    	setWidget(i, 2, new HTML(ui.getDept()));
    			    	setWidget(i, 3, new HTML(ui.getPos()));
    			    	setWidget(i, 4, new HTML(ui.getEnterDate()));
    			    	if (mSelectedRow == -1)getRowFormatter().setStyleName(i, "unselectedRow");
    			    	else if (mSelectedRow == i) styleRow(mSelectedRow, true);
    			    	i++;
    			    }
    			}else{
                    
    			}
    		}
            public void onFailure(Throwable caught) {
                System.out.println(caught);
            }
        	
        };
        userServiceAsync.getUserList(sort, asyncCallback);
	}
	
	private void selectRow(int row) {
		// 선택시 유저 정보 팝업을 뛰움
        int left = getWidget(row + 1,1).getAbsoluteLeft() + 14;
        int top = getWidget(row + 1,1).getAbsoluteTop() + 14;

        if (mUserList.get(row + 1).getPic().equals(""))
        	mUserPic.setUrl("/_nopic.gif");
        else 
        	mUserPic.setUrl("/pictc/" + mUserList.get(row + 1).getPic());
        mUserPic.setSize("115px", "150px");
        
        mUserName.setHTML		("이     름 : " + mUserList.get(row + 1).getName());
    	mUserID.setHTML			("아이디 : " + mUserList.get(row + 1).getID());
    	mUserDept.setHTML		("부     서 : " + mUserList.get(row + 1).getDept() + " ( " + mUserList.get(row + 1).getDeptWhe() + " ) ");
    	mUserDeptTel.setHTML	("내     선 : " + mUserList.get(row + 1).getDeptTel());
    	mUserPos.setHTML		("직     위 : " + mUserList.get(row + 1).getPos());
    	mUserPosAuth.setHTML	("권     한 : " + ((mUserList.get(row + 1).getPosAuth()==1)? "웹접근가능" : "웹접근불가"));
    	mUserTel.setHTML		("연락처 : " + mUserList.get(row + 1).getTel());
    	mUserHP.setHTML			("휴대폰 : " + mUserList.get(row + 1).getHP());
    	mUserEmail.setHTML		("이메일 : " + mUserList.get(row + 1).getEmail());
    	mUserAddr.setHTML		("주     소 : " + mUserList.get(row + 1).getAddr());
    	mUserEnter.setHTML		("입사일 : " + mUserList.get(row + 1).getEnterDate());
    	if (mUserList.get(row + 1).getChkLogin())
    		mLogout.setHTML		("로그아웃 : " + "접속중");  
    	else 
    		mLogout.setHTML		("로그아웃 :" + mUserList.get(row + 1).getLogoutDate());  

        if ((top + 290 ) > Window.getClientHeight()) top = top - 308;
    	
        mUserInfoPopup.setPopupPosition(left, top);
        mUserInfoPopup.show();
	
		styleRow(mSelectedRow, false);
		styleRow(row, true);

		mSelectedRow = row;
	}
	  
	private void styleRow(int row, boolean selected) {
		if (row != -2) {
			if (selected) {
				getRowFormatter().addStyleName(row + 1, "SelectedRow");
			} else {
				getRowFormatter().removeStyleName(row + 1, "SelectedRow");
			}
		}
	}

	private void createDialog() {
		HorizontalPanel userPopupContainer = new HorizontalPanel();
		userPopupContainer.setHeight("260px");
		VerticalPanel userpicPopupContainer = new VerticalPanel();
	    VerticalPanel userinfoPopupContainer = new VerticalPanel();
	    userpicPopupContainer.add(mUserPic);
	    userinfoPopupContainer.setSpacing(5);
	    userinfoPopupContainer.add(mUserName);
	    userinfoPopupContainer.add(mUserID);
	    userinfoPopupContainer.add(mUserDept);
	    userinfoPopupContainer.add(mUserDeptTel);
	    userinfoPopupContainer.add(mUserPos);
	    userinfoPopupContainer.add(mUserPosAuth);
	    userinfoPopupContainer.add(mUserTel);
	    userinfoPopupContainer.add(mUserHP);
	    userinfoPopupContainer.add(mUserEmail);
	    userinfoPopupContainer.add(mUserAddr);
	    userinfoPopupContainer.add(mUserEnter);
	    userinfoPopupContainer.add(mLogout);
	    
	    userPopupContainer.add(userpicPopupContainer);
	    userPopupContainer.add(userinfoPopupContainer);

	    mUserInfoPopup.setWidget(userPopupContainer);

	}

	public UserInfo getSelectedItem() {
		return mUserList.get(mSelectedRow + 1);
	}
	
}