// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18
//

package ju.thecompany.web.client;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.logical.shared.ResizeEvent;
import com.google.gwt.event.logical.shared.ResizeHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.DecoratorPanel;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

public class BottomTablePanel extends VerticalPanel implements ResizeHandler{

	private static BottomTablePanel singleton = new BottomTablePanel();
	
	final DecoratorPanel listHeaderPanel = new DecoratorPanel();
	final ScrollPanel listBottomPanel = new ScrollPanel();
	
	// DB 칼럼 
	private static final String SORT_COL_ID = "id";
	private static final String SORT_COL_NAME = "name";
	private static final String SORT_COL_DEPT = "deptname";
	private static final String SORT_COL_POS = "posname";
	private static final String SORT_COL_CON = "chklogin";
	private static final String SORT_COL_ENTER = "enterdate";
	
	private BottomTablePanel() {
		initPanel();
	}
	
	public static BottomTablePanel getInstance() {
		return singleton;
	}
	
	public void initPanel() {
		setSize("800px", "100%");
		setSpacing(1);
		setStyleName("bottomRightPanel");
		
		final HTML nameHTML = new HTML("Name");
		final HTML idHTML = new HTML("ID");
		final HTML deptHTML = new HTML("DEPT");
		final HTML posHTML = new HTML("POS");
		final HTML dateHTML = new HTML("DATE");

		Window.addResizeHandler(this);
	    Window.enableScrolling(false);
	    Window.setMargin("0px");

	    listHeaderPanel.setSize("796px", "25px");
	    listHeaderPanel.setStyleName("listHeaderPanel");

		FlexTable table = new FlexTable();
		table.setSize("798px","25px");
	    table.setCellSpacing(0);
	    table.setCellPadding(0);
		table.setWidget(0, 0, nameHTML);
	    table.setWidget(0, 1, idHTML);
	    table.setWidget(0, 2, deptHTML);
	    table.setWidget(0, 3, posHTML);
	    table.setWidget(0, 4, dateHTML);
	    table.getRowFormatter().setStyleName(0, "listHeader");
	    table.getCellFormatter().setWidth(0, 0, "150px");
	    table.getCellFormatter().setWidth(0, 1, "250px");
	    table.getCellFormatter().setWidth(0, 2, "100px");
	    table.getCellFormatter().setWidth(0, 3, "100px");
	    table.getCellFormatter().setWidth(0, 4, "198px");
	    listHeaderPanel.setWidget(table);
	    
	    listBottomPanel.setSize("796px", "" + (Window.getClientHeight()- 240) + "px");
	    listBottomPanel.setStyleName("listBottomPanel");
	    listBottomPanel.setAlwaysShowScrollBars(false);
	    
	    listBottomPanel.add(BottomTableContentsPanel.getInstance());
	    
	    // 테이블명 클릭시 유저리스트 정렬
	    nameHTML.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				BottomTableContentsPanel.getInstance().sortItem(SORT_COL_NAME);
			}
        });
	    
	    idHTML.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				BottomTableContentsPanel.getInstance().sortItem(SORT_COL_ID);
			}
        });
	    
	    deptHTML.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				BottomTableContentsPanel.getInstance().sortItem(SORT_COL_DEPT);
			}
        });
	    
	    posHTML.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				BottomTableContentsPanel.getInstance().sortItem(SORT_COL_POS);
			}
        });
	    
	    dateHTML.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				BottomTableContentsPanel.getInstance().sortItem(SORT_COL_ENTER);
			}
        });
	    

		add(listHeaderPanel);
		add(listBottomPanel);

	}

	@Override
	public void onResize(ResizeEvent event) {
		onWindowResized(event.getWidth(), event.getHeight());
	}
	
	public void onWindowResized(int width, int height) {
		if (height > 240)
			listBottomPanel.setHeight("" + (height - 240) + "px");
	}
	
}