// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18
//

package ju.thecompany.web.client;

import ju.thecompany.web.client.data.SessionId;
import ju.thecompany.web.client.data.User;
import ju.thecompany.web.client.data.UserInfo;
import ju.thecompany.web.client.rpc.service.GetUserInfoService;
import ju.thecompany.web.client.rpc.service.GetUserInfoServiceAsync;
import ju.thecompany.web.client.rpc.service.LoginService;
import ju.thecompany.web.client.rpc.service.LoginServiceAsync;
import ju.thecompany.web.client.rpc.service.SessionService;
import ju.thecompany.web.client.rpc.service.SessionServiceAsync;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Cookies;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.DecoratorPanel;
import com.google.gwt.user.client.ui.DialogBox;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.PasswordTextBox;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;

public class LoginPanel extends DecoratorPanel {

	private static LoginPanel singleton = new LoginPanel();

	private final User user = new User();
	
	
	//위젯
	private TextBox mIdTextBox;
	private TextBox mPwTextBox;
	private HTML mNameTextBox;
	private HTML mWelTextBox;
	private Button mLoginButton;
	private Button mFindButton;
	private Button mLogoutButton;
	
	private String mId;

	private LoginPanel(){
		validateSession();
	}
	
	public static LoginPanel getInstance() {
		
		return singleton;
	}
	
	// 새션에 따른 화면 바뀜
    private void validateSession(){
    	
        SessionServiceAsync myServiceAsync = (SessionServiceAsync)GWT.create(SessionService.class);
	    ServiceDefTarget serviceDefTarget = (ServiceDefTarget)myServiceAsync;
        serviceDefTarget.setServiceEntryPoint("session");

        AsyncCallback<SessionId> asyncCallback = new AsyncCallback<SessionId>(){
        	public void onFailure(Throwable caught) {
        		System.out.println(caught);
        	}
        	public void onSuccess(SessionId result) {
        		if(result == null){
                	setChildPanel(getLoginPanel(), false);
                }else if(!TheCompanyWeb.sessionId.getSessionId().equals(result.getSessionId())){
                	setChildPanel(getLoginPanel(), false);
                }else if(TheCompanyWeb.sessionId.getSessionId().equals(result.getSessionId())){
                	setChildPanel(getSuccessPanel(), true);
                	mNameTextBox.setText(TheCompanyWeb.myUserInfo.getName() + "(" + TheCompanyWeb.myUserInfo.getID() + ") 님");
                }
            }
        };
        myServiceAsync.session(TheCompanyWeb.sessionId, asyncCallback);
    }
    
	//내부메소드
    private void setChildPanel(Panel panel, boolean flag) {
    	setWidget(panel);
    	if (flag) setStyleName("loginPanel_success");
    	else setStyleName("loginPanel_login");
    	
    }
    
	//로그인 위젯 게터
	private Panel getLoginPanel() {
		mIdTextBox = new TextBox();
		mPwTextBox = new PasswordTextBox();
		mLoginButton = new Button();
		mFindButton = new Button();
		
		final VerticalPanel fullLoginPanel = new VerticalPanel();
		final DecoratorPanel topLoginPanel = new DecoratorPanel();
		final FlexTable layoutLogin = new FlexTable();
		final HorizontalPanel bottomLoginPanel = new HorizontalPanel();
		final FlowPanel buttonWrapPanel = new FlowPanel();
		
		mIdTextBox.setSize("150px","100%");
		mPwTextBox.setSize("150px","100%");
		mLoginButton.setText("로그인");	
		mLoginButton.setWidth("90px");
		mFindButton.setText("비번찾기");
		mFindButton.setWidth("90px");
		
		// 로그인 패널
		layoutLogin.setCellSpacing(5);

		layoutLogin.setHTML(0, 0, "ID");
		layoutLogin.setWidget(0, 1, mIdTextBox);
		layoutLogin.setHTML(1, 0, "Password");
		layoutLogin.setWidget(1, 1, mPwTextBox);
		
		topLoginPanel.add(layoutLogin);
		topLoginPanel.setStyleName("topLoginPanel_login");
		topLoginPanel.setSize("240px", "100px");

		bottomLoginPanel.setStyleName("bottomLoginPanel_login");
		bottomLoginPanel.setSize("240px", "30px");
		bottomLoginPanel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_RIGHT);
		buttonWrapPanel.add(mLoginButton);
		buttonWrapPanel.add(mFindButton);
		bottomLoginPanel.add(buttonWrapPanel);

		fullLoginPanel.setSpacing(5);
		fullLoginPanel.add(topLoginPanel);
		fullLoginPanel.add(bottomLoginPanel);
		fullLoginPanel.setStyleName("loginPanel_login");
		fullLoginPanel.setSize("250px", "140px");
		
		mLoginButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				System.out.println("click mLoginButton");
				
				user.setUser(mIdTextBox.getText());
				user.setPasswd(mPwTextBox.getText());
				// 로긴 서비스
	            LoginServiceAsync loginServiceAsync = (LoginServiceAsync)GWT.create(LoginService.class);
	            ServiceDefTarget serviceDefTarget = (ServiceDefTarget)loginServiceAsync;
	            serviceDefTarget.setServiceEntryPoint("login");
	            
	            AsyncCallback<String> asyncCallback = new AsyncCallback<String>() {
	        		public void onSuccess(String result) {
	        			if(result != null){
	        				mId = mIdTextBox.getText();
	                    	setChildPanel(getSuccessPanel(), true);
	                    	TopPanel.getInstance().changeMode(true);
	                    	BottomPanel.getInstance().changeMode(true);
	                    	
	                        Cookies.setCookie("session", result);
	                        TheCompanyWeb.sessionId.setSessionId(result);

	                        System.out.println("login session => "+result);
	                        
	                        // 로긴한 대상의 정보를 가져옴
	                		GetUserInfoServiceAsync getUserInfoServiceAsync = (GetUserInfoServiceAsync)GWT.create(GetUserInfoService.class);
	                        ServiceDefTarget serviceDefTarget = (ServiceDefTarget)getUserInfoServiceAsync;
	                        serviceDefTarget.setServiceEntryPoint("getuserinfo");
	                        
	                        AsyncCallback<UserInfo> asyncCallback = new AsyncCallback<UserInfo>() {
	                    		public void onSuccess(UserInfo result) {
	                    			if(result != null){

	                    				TheCompanyWeb.myUserInfo.setID(result.getID());
	                    				TheCompanyWeb.myUserInfo.setName(result.getName());
	                    				mNameTextBox.setText(result.getName() + "(" + result.getID() + ") 님");
	                    				Cookies.setCookie("tcid", result.getID());
	                    				Cookies.setCookie("tcname", result.getName());
	                    			}else{
	                                    
	                    			}
	                    		}
	                            public void onFailure(Throwable caught) {
	                                System.out.println(caught);
	                            }
	                        	
	                        };
	                        getUserInfoServiceAsync.get(mId, asyncCallback);

	        			}else{
	                        Window.alert("아이디나 비밀번호를 확인하세요.");
	        			}
	        		}
	                public void onFailure(Throwable caught) {
	                    System.out.println(caught);
	                }
	            	
	            };
	            loginServiceAsync.login(user, asyncCallback);
			}
		});
		
		
		final DialogBox dialogBox = new DialogBox();
		dialogBox.setText("비번찾기");
		dialogBox.setAnimationEnabled(true);
		
		final Button closeButton = new Button("Close");
		closeButton.setWidth("90px");
		final Button sendButton = new Button("Send");
		sendButton.setWidth("90px");

		sendButton.setEnabled(false);	// 이메일 모듈 포함시 삭제
		
		final TextBox emailTextBox = new TextBox();
		emailTextBox.setWidth("130px");
		
		final FlexTable flexTable = new FlexTable();
		flexTable.setHeight("40px");
		flexTable.setHTML(0, 0, "<b>찾을ID : </b>");
		flexTable.setWidget(0, 1, emailTextBox);
		
		final HorizontalPanel dialogButtonPanel = new HorizontalPanel();
		final FlowPanel dialogButtonWrapPanel = new FlowPanel();

		dialogButtonPanel.setSize("200px", "30px");
		dialogButtonPanel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_RIGHT);
		dialogButtonWrapPanel.add(sendButton);
		dialogButtonWrapPanel.add(closeButton);
		dialogButtonPanel.add(dialogButtonWrapPanel);

		VerticalPanel dialogVPanel = new VerticalPanel();
		dialogVPanel.setSpacing(10);
		dialogVPanel.add(flexTable);
		dialogVPanel.add(new HTML("등록된 e-mail로 전송됩니다."));
		dialogVPanel.add(dialogButtonPanel);

		dialogBox.setWidget(dialogVPanel);

		sendButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				
				// 이 곳에 비번을 찾아 이 메일을 
				// 보내는 모듈을 추가하십시오.
				
			}
		});
		
		closeButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				dialogBox.hide();
			}
		});
		
		mFindButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				dialogBox.center();
				closeButton.setFocus(true);
			}
		});
		
		
		
		// 로그인 패널 끝
		return fullLoginPanel;
	}
	
	//로그인 성공 위젯 게터
	private Panel getSuccessPanel() {
		mNameTextBox = new HTML();
		mWelTextBox = new HTML();
		//mModifyButton = new Button();
		mLogoutButton = new Button();

		final VerticalPanel fullSuccessPanel = new VerticalPanel();
		final DecoratorPanel topSuccessPanel = new DecoratorPanel();
		final FlexTable layoutSuccess = new FlexTable();
		final HorizontalPanel bottomSuccessPanel = new HorizontalPanel();
		final FlowPanel buttonWrapPanel = new FlowPanel();
		
		mNameTextBox.setSize("224px","100%");
		mNameTextBox.setStyleName("text_login");
		
		mWelTextBox.setSize("224px","100%");
		mWelTextBox.setStyleName("text_login");
		
		mNameTextBox.setText("님");
		mWelTextBox.setText("환영합니다!");
		
		mLogoutButton.setText("로그아웃");
		mLogoutButton.setWidth("90px");

		// 로그인 완료 패널
		layoutSuccess.setCellPadding(5);

		layoutSuccess.setWidget(0, 0, mNameTextBox);
		layoutSuccess.setWidget(1, 0, mWelTextBox);

		topSuccessPanel.add(layoutSuccess);
		topSuccessPanel.setStyleName("topLoginPanel_success");
		topSuccessPanel.setSize("240px", "100px");

		bottomSuccessPanel.setStyleName("bottomLoginPanel_success");
		bottomSuccessPanel.setSize("240px", "30px");
		bottomSuccessPanel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_RIGHT);
		buttonWrapPanel.add(mLogoutButton);
		//buttonWrapPanel.add(mModifyButton);
		bottomSuccessPanel.add(buttonWrapPanel);
		
		fullSuccessPanel.setSpacing(5);
		fullSuccessPanel.add(topSuccessPanel);
		fullSuccessPanel.add(bottomSuccessPanel);
		fullSuccessPanel.setStyleName("loginPanel_success");
		fullSuccessPanel.setSize("250px", "140px");

		mLogoutButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				Cookies.setCookie("session", "");
				setChildPanel(getLoginPanel(), false);
				TopPanel.getInstance().changeMode(false);
				BottomPanel.getInstance().changeMode(false);
			}
		});
		// 로그인 완료 패널 끝

		
		return fullSuccessPanel;
	}




}
