// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18
//

package ju.thecompany.web.db;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import ju.thecompany.web.client.data.DeptInfo;
import ju.thecompany.web.client.data.Notice;
import ju.thecompany.web.client.data.PosInfo;
import ju.thecompany.web.client.data.UserInfo;
import ju.thecompany.web.server.util.Setting;

public class DBUtil {

	// 로그인 체크
	public static boolean chkLogin(String id, String pw) throws SQLException {
		
		Connection connection = null;
		ResultSet rs = null;
		PreparedStatement pstmt = null;
		boolean result = false;

		String ip = Setting.getInstance().getData("MysqlIP");
		String port = Setting.getInstance().getData("MysqlPort");
		String dbname = Setting.getInstance().getData("MysqlDBName");
		String dbid = Setting.getInstance().getData("MysqlID"); 
		String dbpw = Setting.getInstance().getData("MysqlPassword");
		
		String jdbcUrl="jdbc:mysql://" + ip + ":" + port + "/" + dbname;
		
		// 로그인을 확인하기 위해 해당 id로 select한 후 존재여부와 passwd를 확인한 후 리턴
		try {
			DriverManager.registerDriver(new com.mysql.jdbc.Driver());
			connection = DriverManager.getConnection(jdbcUrl, dbid, dbpw);
			
			pstmt = connection.prepareStatement("SELECT * FROM MEMBER a join POS b WHERE a.posname = b.posname and a.id = ?");
			pstmt.setString(1, id);
			rs = pstmt.executeQuery();
			if ( (rs.last()) && (rs.getRow() == 1)) {
				String dbPassword = rs.getString("passwd");
				int auth = rs.getInt("auth");
				
				if ( dbPassword.equals(pw) && (auth == 1)) {
					result = true;
				}
			}
		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (pstmt != null) try { pstmt.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (connection != null) try { connection.close(); } catch (SQLException e) { e.printStackTrace(); }
		}

		if (result) System.err.println("true") ; 
		else System.err.println("false");
		return result;
	}
	
	// 유저 정보 획득
	public static UserInfo getUserInfo(String id) throws SQLException {
		
		Connection connection = null;
		ResultSet rs = null;
		PreparedStatement pstmt = null;
		UserInfo result = null;

		String ip = Setting.getInstance().getData("MysqlIP");
		String port = Setting.getInstance().getData("MysqlPort");
		String dbname = Setting.getInstance().getData("MysqlDBName");
		String dbid = Setting.getInstance().getData("MysqlID"); 
		String dbpw = Setting.getInstance().getData("MysqlPassword");
		
		String jdbcUrl="jdbc:mysql://" + ip + ":" + port + "/" + dbname;
		
		// 로그인을 확인하기 위해 해당 id로 select한 후 존재여부와 passwd를 확인한 후 리턴
		try {
			DriverManager.registerDriver(new com.mysql.jdbc.Driver());
			connection = DriverManager.getConnection(jdbcUrl, dbid, dbpw);
			
			result = new UserInfo();
			
			pstmt = connection.prepareStatement("SELECT * FROM MEMBER a join DEPT b join POS c WHERE a.deptname = b.deptname and a.posname = c. posname and a.id = ?");
			pstmt.setString(1, id);
			rs = pstmt.executeQuery();
			if ( (rs.last()) && (rs.getRow() == 1)) {
				result.setID(rs.getString("id"));
				result.setPW(rs.getString("passwd"));
				result.setName(rs.getString("name"));
				result.setEmail(rs.getString("email"));
				result.setAddr(rs.getString("addr"));
				result.setTel(rs.getString("a.tel"));
				result.setHP(rs.getString("hp"));
				result.setPic(rs.getString("pic"));
				result.setDept(rs.getString("deptname"));
				result.setDeptTel(rs.getString("b.tel"));
				result.setDeptWhe(rs.getString("whe"));
				result.setPos(rs.getString("posname"));
				result.setPosAuth(rs.getInt("auth"));
				result.setChkLogin(rs.getBoolean("chklogin"));
				result.setLogoutDate(rs.getTimestamp("logoutdate"));
				result.setEnterDate(rs.getTimestamp("enterdate"));
			}
		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (pstmt != null) try { pstmt.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (connection != null) try { connection.close(); } catch (SQLException e) { e.printStackTrace(); }
		}

		return result;
	}
	
	// 유저 리스트 획득
	public static Vector <UserInfo> getUserList(String sort) throws SQLException {
		
		Connection connection = null;
		ResultSet rs = null;
		Statement stmt = null;
		UserInfo userinfo = null;
		Vector <UserInfo> result = null;
		
		String ip = Setting.getInstance().getData("MysqlIP");
		String port = Setting.getInstance().getData("MysqlPort");
		String dbname = Setting.getInstance().getData("MysqlDBName");
		String dbid = Setting.getInstance().getData("MysqlID"); 
		String dbpw = Setting.getInstance().getData("MysqlPassword");
		
		String jdbcUrl="jdbc:mysql://" + ip + ":" + port + "/" + dbname;
		
		try {
			DriverManager.registerDriver(new com.mysql.jdbc.Driver());
			connection = DriverManager.getConnection(jdbcUrl, dbid, dbpw);
			
			
			
			stmt = connection.createStatement();
			rs = stmt.executeQuery("SELECT * FROM MEMBER a join DEPT b join POS c WHERE a.deptname = b.deptname and a.posname = c. posname ORDER BY " + sort + " ASC ");
			result = new Vector <UserInfo>();
			
			while (rs.next()){
				userinfo = new UserInfo();
				
				userinfo.setID(rs.getString("id"));
				userinfo.setPW(rs.getString("passwd"));
				userinfo.setName(rs.getString("name"));
				userinfo.setEmail(rs.getString("email"));
				userinfo.setAddr(rs.getString("addr"));
				userinfo.setTel(rs.getString("a.tel"));
				userinfo.setHP(rs.getString("hp"));
				userinfo.setPic(rs.getString("pic"));
				userinfo.setDept(rs.getString("deptname"));
				userinfo.setDeptTel(rs.getString("b.tel"));
				userinfo.setDeptWhe(rs.getString("whe"));
				userinfo.setPos(rs.getString("posname"));
				userinfo.setPosAuth(rs.getInt("auth"));
				userinfo.setChkLogin(rs.getBoolean("chklogin"));
				userinfo.setLogoutDate(rs.getTimestamp("logoutdate"));
				userinfo.setEnterDate(rs.getTimestamp("enterdate"));
				
				result.add(userinfo);
			}
			
		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (connection != null) try { connection.close(); } catch (SQLException e) { e.printStackTrace(); }
		}

		return result;
	}
	
	// 유저 추가
	public static void insertUser(UserInfo user) throws SQLException {
		
		Connection connection = null;
		ResultSet rs = null;
		Statement stmt = null;
		
		String ip = Setting.getInstance().getData("MysqlIP");
		String port = Setting.getInstance().getData("MysqlPort");
		String dbname = Setting.getInstance().getData("MysqlDBName");
		String dbid = Setting.getInstance().getData("MysqlID"); 
		String dbpw = Setting.getInstance().getData("MysqlPassword");
		
		String jdbcUrl="jdbc:mysql://" + ip + ":" + port + "/" + dbname;
		
		try {
			DriverManager.registerDriver(new com.mysql.jdbc.Driver());
			connection = DriverManager.getConnection(jdbcUrl, dbid, dbpw);

			stmt = connection.createStatement();
			stmt.executeUpdate("INSERT INTO MEMBER VALUES ('" + 
					user.getID() + "', '" + user.getPW() + "', '" + user.getName() + "', '" + 
					user.getEmail() + "', '" + user.getAddr() + "', '" + user.getTel() + "', '" + 
					user.getHP() + "', '" + user.getPic().trim() + "', '" + user.getDept() + "', '" + 
					user.getPos() + "', '0', now(), now() )"); 
			stmt.close();
	
		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (connection != null) try { connection.close(); } catch (SQLException e) { e.printStackTrace(); }
		}
	}
	
	// 유저 수정
	public static void updateUser(UserInfo user) throws SQLException {
		
		Connection connection = null;
		ResultSet rs = null;
		Statement stmt = null;

		String ip = Setting.getInstance().getData("MysqlIP");
		String port = Setting.getInstance().getData("MysqlPort");
		String dbname = Setting.getInstance().getData("MysqlDBName");
		String dbid = Setting.getInstance().getData("MysqlID"); 
		String dbpw = Setting.getInstance().getData("MysqlPassword");
		
		String jdbcUrl="jdbc:mysql://" + ip + ":" + port + "/" + dbname;
		
		try {
			DriverManager.registerDriver(new com.mysql.jdbc.Driver());
			connection = DriverManager.getConnection(jdbcUrl, dbid, dbpw);

			stmt = connection.createStatement();
			stmt.executeUpdate("UPDATE MEMBER SET " +
					"passwd = '" + user.getPW() + 
					"' , name = '" + user.getName()  + 
					"' , email = '" + user.getEmail()  + 
					"', addr = '" + user.getAddr()  + 
					"', tel = '" + user.getTel()  + 
					"', hp = '" + user.getHP()  + 
					"', pic = '" + user.getPic().trim()  + 
					"', deptname = '" + user.getDept()  + 
					"', posname = '" + user.getPos()  + 
					"' WHERE id = '" + user.getID() + "'"); 
			
			stmt.close();

		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (connection != null) try { connection.close(); } catch (SQLException e) { e.printStackTrace(); }
		}
	}
	
	// 유저 삭제
	public static void deleteUser(String id) throws SQLException {
		
		Connection connection = null;
		ResultSet rs = null;
		Statement stmt = null;

		String ip = Setting.getInstance().getData("MysqlIP");
		String port = Setting.getInstance().getData("MysqlPort");
		String dbname = Setting.getInstance().getData("MysqlDBName");
		String dbid = Setting.getInstance().getData("MysqlID"); 
		String dbpw = Setting.getInstance().getData("MysqlPassword");
		
		String jdbcUrl="jdbc:mysql://" + ip + ":" + port + "/" + dbname;
		
		try {
			DriverManager.registerDriver(new com.mysql.jdbc.Driver());
			connection = DriverManager.getConnection(jdbcUrl, dbid, dbpw);

			stmt = connection.createStatement();
			stmt.executeUpdate("DELETE FROM MEMBER WHERE id='" + id + "'");
			
			stmt.close();

		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (connection != null) try { connection.close(); } catch (SQLException e) { e.printStackTrace(); }
		}
	}
	
	// 부서 리스트 획득
	public static Vector <DeptInfo> getDeptList() throws SQLException {
		
		Connection connection = null;
		ResultSet rs = null;
		Statement stmt = null;
		DeptInfo deptinfo = null;
		Vector <DeptInfo> result = null;
		
		String ip = Setting.getInstance().getData("MysqlIP");
		String port = Setting.getInstance().getData("MysqlPort");
		String dbname = Setting.getInstance().getData("MysqlDBName");
		String dbid = Setting.getInstance().getData("MysqlID"); 
		String dbpw = Setting.getInstance().getData("MysqlPassword");
		
		String jdbcUrl="jdbc:mysql://" + ip + ":" + port + "/" + dbname;
		
		try {
			DriverManager.registerDriver(new com.mysql.jdbc.Driver());
			connection = DriverManager.getConnection(jdbcUrl, dbid, dbpw);
			
			
			
			stmt = connection.createStatement();
			rs = stmt.executeQuery("SELECT * FROM DEPT ORDER BY deptname ASC ");
			result = new Vector <DeptInfo>();
			
			while (rs.next()){
				deptinfo = new DeptInfo();
				
				deptinfo.setDeptName(rs.getString("deptname"));
				deptinfo.setTel(rs.getString("tel"));
				deptinfo.setWhere(rs.getString("whe"));

				result.add(deptinfo);
			}
			
		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (connection != null) try { connection.close(); } catch (SQLException e) { e.printStackTrace(); }
		}

		return result;
	}
	
	// 부서 추가
	public static void insertDept(DeptInfo dept) throws SQLException {
		
		Connection connection = null;
		ResultSet rs = null;
		Statement stmt = null;
		
		String ip = Setting.getInstance().getData("MysqlIP");
		String port = Setting.getInstance().getData("MysqlPort");
		String dbname = Setting.getInstance().getData("MysqlDBName");
		String dbid = Setting.getInstance().getData("MysqlID"); 
		String dbpw = Setting.getInstance().getData("MysqlPassword");
		
		String jdbcUrl="jdbc:mysql://" + ip + ":" + port + "/" + dbname;
		
		try {
			DriverManager.registerDriver(new com.mysql.jdbc.Driver());
			connection = DriverManager.getConnection(jdbcUrl, dbid, dbpw);

			stmt = connection.createStatement();
			stmt.executeUpdate("INSERT INTO DEPT VALUES ('" + 
					dept.getDeptName() + "', '" + dept.getTel() + "', '" + 
					dept.getWhere() + "' )"); 
			stmt.close();
	
		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (connection != null) try { connection.close(); } catch (SQLException e) { e.printStackTrace(); }
		}
	}
	
	// 부서 수정
	public static void updateDept(DeptInfo dept, DeptInfo oridept) throws SQLException {
		
		Connection connection = null;
		ResultSet rs = null;
		Statement stmt = null;

		String ip = Setting.getInstance().getData("MysqlIP");
		String port = Setting.getInstance().getData("MysqlPort");
		String dbname = Setting.getInstance().getData("MysqlDBName");
		String dbid = Setting.getInstance().getData("MysqlID"); 
		String dbpw = Setting.getInstance().getData("MysqlPassword");
		
		String jdbcUrl="jdbc:mysql://" + ip + ":" + port + "/" + dbname;
		
		try {
			DriverManager.registerDriver(new com.mysql.jdbc.Driver());
			connection = DriverManager.getConnection(jdbcUrl, dbid, dbpw);

			stmt = connection.createStatement();
			stmt.executeUpdate("UPDATE DEPT SET deptname='" + dept.getDeptName() + 
					"', tel='" + dept.getTel() + "', whe='" + dept.getWhere() + 
					"' where deptname='" + oridept.getDeptName() + "'"); 
			
			stmt.close();

		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (connection != null) try { connection.close(); } catch (SQLException e) { e.printStackTrace(); }
		}
	}
	
	// 부서 삭제
	public static void deleteDept(DeptInfo dept) throws SQLException {
		
		Connection connection = null;
		ResultSet rs = null;
		Statement stmt = null;

		String ip = Setting.getInstance().getData("MysqlIP");
		String port = Setting.getInstance().getData("MysqlPort");
		String dbname = Setting.getInstance().getData("MysqlDBName");
		String dbid = Setting.getInstance().getData("MysqlID"); 
		String dbpw = Setting.getInstance().getData("MysqlPassword");
		
		String jdbcUrl="jdbc:mysql://" + ip + ":" + port + "/" + dbname;
		
		try {
			DriverManager.registerDriver(new com.mysql.jdbc.Driver());
			connection = DriverManager.getConnection(jdbcUrl, dbid, dbpw);

			stmt = connection.createStatement();
			stmt.executeUpdate("DELETE FROM DEPT WHERE deptname='" + dept.getDeptName() + "'");
			
			stmt.close();

		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (connection != null) try { connection.close(); } catch (SQLException e) { e.printStackTrace(); }
		}
	}
	
	// 직책 리스트 획득
	public static Vector <PosInfo> getPosList() throws SQLException {
		
		Connection connection = null;
		ResultSet rs = null;
		Statement stmt = null;
		PosInfo posinfo = null;
		Vector <PosInfo> result = null;
		
		String ip = Setting.getInstance().getData("MysqlIP");
		String port = Setting.getInstance().getData("MysqlPort");
		String dbname = Setting.getInstance().getData("MysqlDBName");
		String dbid = Setting.getInstance().getData("MysqlID"); 
		String dbpw = Setting.getInstance().getData("MysqlPassword");
		
		String jdbcUrl="jdbc:mysql://" + ip + ":" + port + "/" + dbname;
		
		try {
			DriverManager.registerDriver(new com.mysql.jdbc.Driver());
			connection = DriverManager.getConnection(jdbcUrl, dbid, dbpw);

			stmt = connection.createStatement();
			rs = stmt.executeQuery("SELECT * FROM POS ORDER BY posname ASC ");
			result = new Vector <PosInfo>();
			
			while (rs.next()){
				posinfo = new PosInfo();
				
				posinfo.setPosName(rs.getString("posname"));
				posinfo.setAuth(rs.getString("auth"));

				result.add(posinfo);
			}
			
		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (connection != null) try { connection.close(); } catch (SQLException e) { e.printStackTrace(); }
		}

		return result;
	}
	
	// 직책 추가
	public static void insertPos(PosInfo pos) throws SQLException {
		
		Connection connection = null;
		ResultSet rs = null;
		Statement stmt = null;
		
		String ip = Setting.getInstance().getData("MysqlIP");
		String port = Setting.getInstance().getData("MysqlPort");
		String dbname = Setting.getInstance().getData("MysqlDBName");
		String dbid = Setting.getInstance().getData("MysqlID"); 
		String dbpw = Setting.getInstance().getData("MysqlPassword");
		
		String jdbcUrl="jdbc:mysql://" + ip + ":" + port + "/" + dbname;
		
		try {
			DriverManager.registerDriver(new com.mysql.jdbc.Driver());
			connection = DriverManager.getConnection(jdbcUrl, dbid, dbpw);

			stmt = connection.createStatement();
			stmt.executeUpdate("INSERT INTO POS VALUES ('" + 
					pos.getPosName() + "', '" + pos.getAuth() + "' )"); 
			stmt.close();
	
		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (connection != null) try { connection.close(); } catch (SQLException e) { e.printStackTrace(); }
		}
	}
	
	//직책 수정
	public static void updatePos(PosInfo pos, PosInfo oripos) throws SQLException {
		
		Connection connection = null;
		ResultSet rs = null;
		Statement stmt = null;

		String ip = Setting.getInstance().getData("MysqlIP");
		String port = Setting.getInstance().getData("MysqlPort");
		String dbname = Setting.getInstance().getData("MysqlDBName");
		String dbid = Setting.getInstance().getData("MysqlID"); 
		String dbpw = Setting.getInstance().getData("MysqlPassword");
		
		String jdbcUrl="jdbc:mysql://" + ip + ":" + port + "/" + dbname;
		
		try {
			DriverManager.registerDriver(new com.mysql.jdbc.Driver());
			connection = DriverManager.getConnection(jdbcUrl, dbid, dbpw);

			stmt = connection.createStatement();
			stmt.executeUpdate("UPDATE POS SET posname='" + pos.getPosName() + 
					"', auth='" + pos.getAuth() +
					"' where posname='" + oripos.getPosName() + "'"); 
			
			stmt.close();

		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (connection != null) try { connection.close(); } catch (SQLException e) { e.printStackTrace(); }
		}
	}
	
	//직책 삭제
	public static void deletePos(PosInfo pos) throws SQLException {
		
		Connection connection = null;
		ResultSet rs = null;
		Statement stmt = null;

		String ip = Setting.getInstance().getData("MysqlIP");
		String port = Setting.getInstance().getData("MysqlPort");
		String dbname = Setting.getInstance().getData("MysqlDBName");
		String dbid = Setting.getInstance().getData("MysqlID"); 
		String dbpw = Setting.getInstance().getData("MysqlPassword");
		
		String jdbcUrl="jdbc:mysql://" + ip + ":" + port + "/" + dbname;
		
		try {
			DriverManager.registerDriver(new com.mysql.jdbc.Driver());
			connection = DriverManager.getConnection(jdbcUrl, dbid, dbpw);

			stmt = connection.createStatement();
			stmt.executeUpdate("DELETE FROM POS WHERE posname='" + pos.getPosName() + "'");
			
			stmt.close();

		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (connection != null) try { connection.close(); } catch (SQLException e) { e.printStackTrace(); }
		}
	}
	
	// 공지 보내기
	public static void insertNotice(Notice notice) throws SQLException {
		
		Connection connection = null;
		ResultSet rs = null;
		Statement stmt = null;
		
		String ip = Setting.getInstance().getData("MysqlIP");
		String port = Setting.getInstance().getData("MysqlPort");
		String dbname = Setting.getInstance().getData("MysqlDBName");
		String dbid = Setting.getInstance().getData("MysqlID"); 
		String dbpw = Setting.getInstance().getData("MysqlPassword");
		
		String jdbcUrl="jdbc:mysql://" + ip + ":" + port + "/" + dbname;
		
		try {
			DriverManager.registerDriver(new com.mysql.jdbc.Driver());
			connection = DriverManager.getConnection(jdbcUrl, dbid, dbpw);

			stmt = connection.createStatement();
			stmt.executeUpdate("INSERT INTO NOTICE(send_id, date, title, text) VALUES ('" + 
					notice.getSendID() + "', now(), '" + notice.getTitle() + "', '" + 
					notice.getText() + "' )"); 
			stmt.close();
	
		} finally {
			if (rs != null) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (connection != null) try { connection.close(); } catch (SQLException e) { e.printStackTrace(); }
		}
	}
}
