// License : Apache License v2.0
// Juhyun Lee (xog2000@gmail.com)
// 2009/10/18
//

package ju.thecompany.web.helpserver.server;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SocketChannel;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import javax.xml.stream.XMLStreamException;

import ju.thecompany.data.CodeDesc;
import ju.thecompany.data.DateParser;
import ju.thecompany.data.Setting;
import ju.thecompany.data.TCXMLParser;
import ju.thecompany.data.TheCompanyDataToken;
import ju.thecompany.web.helpserver.dbprocess.MysqlDBControl;


// DB를 확인하고 공지가 있다면 유저아이디를 불러와 연속적으로 
// 제어서버로 메시지를 보냄. 보낸 후 메시지 삭제.
public class SendXMLServer extends Thread{
	
	private static final String SEPT = "/!`/~";
	
	public SendXMLServer() {
	}
	
	@Override
	public void run() {
		String serverip = Setting.getInstance().getData("ControlServerIP");
		int serverport = Integer.parseInt(Setting.getInstance().getData("ControlServerPort"));
		String mysqlip = Setting.getInstance().getData("MysqlIP");
		String mysqlport = Setting.getInstance().getData("MysqlPort");
		String mysqlid = Setting.getInstance().getData("MysqlID");
		String mysqlpw = Setting.getInstance().getData("MysqlPassword");
		String mysqldb = Setting.getInstance().getData("MysqlDBName");
		
		ResultSet rs = null;
		Statement stmt = null;
		PreparedStatement pstmt = null;
		SocketChannel sc = null;
		Connection conn = null;
		
		MysqlDBControl mdbc = new MysqlDBControl();
		
		
		try {
			sc = SocketChannel.open(new InetSocketAddress(serverip, serverport));
			conn = mdbc.Open(mysqlip, mysqlport, mysqldb, mysqlid, mysqlpw);
			
			while(true) {
				
				Vector <HashMap<String, String>> notices = new Vector <HashMap<String, String>>();
				Vector <String> recv_ids = new Vector <String>();
				
				HashMap<String, String> noticeHashMap = null;
	
				stmt = conn.createStatement();
				rs = stmt.executeQuery("SELECT * FROM NOTICE");
	
				while (rs.next()) {
					
					noticeHashMap = new HashMap<String, String>();
					// 결과 ResultSet를 분석하여 HashMap 객체에 저장
	
					noticeHashMap.put("num", "" + rs.getInt("num"));
					noticeHashMap.put("send_id", rs.getString("send_id"));
					noticeHashMap.put("date", DateParser.dateToString(rs.getDate("date")));
					noticeHashMap.put("title", rs.getString("title"));
					noticeHashMap.put("text", rs.getString("text"));
					
					notices.add(noticeHashMap);
				}
				rs.close();
				stmt.close();
				rs = null;
				stmt = null;
				
				if (!notices.isEmpty()) {
					stmt = conn.createStatement();
					rs = stmt.executeQuery("SELECT id FROM MEMBER");
		
					while (rs.next()) {
						recv_ids.add(rs.getString("id"));
					}
					rs.close();
					stmt.close();
					rs = null;
					stmt = null;
					
					Iterator<HashMap<String, String>> i = notices.iterator();
					Iterator<String> recv_i = recv_ids.iterator();
					
					while (i.hasNext()) {
						HashMap<String, String> nhm = i.next();
						
						while (recv_i.hasNext()) {
							String recv_id = recv_i.next();
							if (recv_id.equals(nhm.get("send_id"))) continue;
						
							TheCompanyDataToken sendToken = new TheCompanyDataToken();
							TheCompanyDataToken recvToken = new TheCompanyDataToken();
							Object obj;
							sendToken.code = CodeDesc.NOTICE;
							sendToken.id = nhm.get("send_id");
							sendToken.date = nhm.get("date");
							recvToken.id = recv_id;
							obj = nhm.get("title") + SEPT + nhm.get("text");
							
							String sendXML = TCXMLParser.parseString(sendToken, recvToken, obj);
							
							System.err.println(sendXML);
							
							ByteBuffer encodedBuffer = ByteBuffer.wrap(sendXML.getBytes("UTF-8"));
							
							int bt = 0;
							
							if (sc != null)
								bt = sc.write(encodedBuffer);
							
							encodedBuffer = null;
							
							Thread.sleep(300);
						}
					}
					
					i = notices.iterator();
					
					pstmt = conn.prepareStatement("DELETE FROM NOTICE WHERE num = ? ");
					
					while (i.hasNext()) {
						HashMap<String, String> nhm = i.next();
						pstmt.setString(1, nhm.get("num"));
						pstmt.executeUpdate();
					}
					
					pstmt.close();
					pstmt = null;

				}

				Thread.sleep(5000);
				
			}
			
		} catch (SQLException e) {
			e.printStackTrace();
		} catch (XMLStreamException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
			e.printStackTrace();
		} finally {
			if (rs != null ) try { rs.close(); } catch (SQLException e) { e.printStackTrace(); }
			if (stmt != null) try { stmt.close(); } catch (SQLException e) { e.printStackTrace(); }		
			if (pstmt != null) try { pstmt.close(); } catch (SQLException e) { e.printStackTrace(); }		
			if (sc != null)try {sc.close();} catch (IOException e) {e.printStackTrace();}
			if (conn != null)try {conn.close();} catch (SQLException e) {e.printStackTrace();}
		}
	}
}
